/*
 * GrammarAccess.cpp
 *
 *  Created on: 20. 9. 2014
 *	  Author: Jan Travnicek
 */

#include "GrammarAccess.h"

#include <grammar/Regular/RightRG.h>
#include <grammar/Regular/RightLG.h>

#include <factory/XmlDataFactory.hpp>
#include <container/ObjectsSet.h>
#include <container/ObjectsMap.h>
#include <container/ObjectsTuple.h>
#include <container/ObjectsVector.h>
#include <container/ObjectsVariant.h>
#include <container/ObjectsPair.h>
#include <container/ObjectsTree.h>

#include "common/AccessHelpers.hpp"

void GrammarAccess::access ( grammar::Grammar & grammar, const GrammarSettings::Settings & settings, const OperationSettings::Settings & operation, std::deque < sax::Token > & argument ) {
	dispatch ( grammar.getData ( ), settings, operation, argument );
}

void GrammarAccess::access ( grammar::RightRG < > & grammar, const GrammarSettings::Settings & settings, const OperationSettings::Settings & operation, std::deque < sax::Token > & argument ) {
	if ( settings == GrammarSettings::Settings::NONTERMINALS )
		return handleComponent < grammar::NonterminalAlphabet > ( grammar, operation, argument );

	if ( settings == GrammarSettings::Settings::TERMINALS )
		return handleComponent < grammar::TerminalAlphabet > ( grammar, operation, argument );

	if ( settings == GrammarSettings::Settings::INITIAL_SYMBOLS )
		return handleElement < grammar::InitialSymbol > ( grammar, operation, argument );

	if ( settings == GrammarSettings::Settings::RULES && operation == OperationSettings::Settings::GET )
		alib::XmlDataFactory::toStdout ( grammar.getRules ( ) );

	throw exception::CommonException ( "Component not available" );
}

auto GrammarAccessRightRG = GrammarAccess::RegistratorWrapper < void, grammar::RightRG < > > ( GrammarAccess::access );

void GrammarAccess::access ( grammar::RightLG < > & grammar, const GrammarSettings::Settings & settings, const OperationSettings::Settings & operation, std::deque < sax::Token > & argument ) {
	if ( settings == GrammarSettings::Settings::NONTERMINALS )
		return handleComponent < grammar::NonterminalAlphabet > ( grammar, operation, argument );

	if ( settings == GrammarSettings::Settings::TERMINALS )
		return handleComponent < grammar::TerminalAlphabet > ( grammar, operation, argument );

	if ( settings == GrammarSettings::Settings::INITIAL_SYMBOLS )
		return handleElement < grammar::InitialSymbol > ( grammar, operation, argument );

	if ( settings == GrammarSettings::Settings::RULES && operation == OperationSettings::Settings::GET )
		alib::XmlDataFactory::toStdout ( grammar.getRules ( ) );

	throw exception::CommonException ( "Component not available" );
}

auto GrammarAccessRightLG = GrammarAccess::RegistratorWrapper < void, grammar::RightLG < > > ( GrammarAccess::access );
