/*
 * DotConverter.h
 *
 *  Created on: Apr 1, 2013
 *      Author: Martin Zak
 */

#ifndef DOT_CONVERTER_H_
#define DOT_CONVERTER_H_

#include <ostream>
#include <set>
#include <map>
#include <list>
#include <utility>
#include <vector>
#include <typeinfo>

#include <core/multipleDispatch.hpp>
#include <exception/CommonException.h>

#include <string/String.h>

#include <automaton/Automaton.h>
#include <automaton/AutomatonFeatures.h>

#include <automaton/FSM/NFA.h>
#include <automaton/FSM/EpsilonNFA.h>
#include <automaton/FSM/MultiInitialStateNFA.h>
#include <automaton/FSM/DFA.h>
#include <automaton/FSM/ExtendedNFA.h>
#include <automaton/FSM/CompactNFA.h>
#include <automaton/TA/NFTA.h>
#include <automaton/TA/DFTA.h>
#include <automaton/PDA/NPDA.h>
#include <automaton/PDA/SinglePopNPDA.h>
#include <automaton/PDA/InputDrivenDPDA.h>
#include <automaton/PDA/InputDrivenNPDA.h>
#include <automaton/PDA/VisiblyPushdownDPDA.h>
#include <automaton/PDA/VisiblyPushdownNPDA.h>
#include <automaton/PDA/RealTimeHeightDeterministicDPDA.h>
#include <automaton/PDA/RealTimeHeightDeterministicNPDA.h>
#include <automaton/PDA/DPDA.h>
#include <automaton/PDA/SinglePopDPDA.h>
#include <automaton/TM/OneTapeDTM.h>

#include <factory/StringDataFactory.hpp>

#include "common/converterCommon.hpp"

class DotConverter : public std::SingleDispatchFirstStaticParam<DotConverter, void, std::ostream&, const automaton::AutomatonBase &> {
	template < class SymbolType, class StateType >
	static void transitions(const automaton::DFA < SymbolType, StateType > & fsm, const std::map < StateType, int > & states, std::ostream & out);

	template < class SymbolType, class StateType >
	static void transitions(const automaton::NFA < SymbolType, StateType > & fsm, const std::map < StateType, int > & states, std::ostream & out);

	template < class SymbolType, class StateType >
	static void transitions(const automaton::MultiInitialStateNFA < SymbolType, StateType > & fsm, const std::map < StateType, int > & states, std::ostream & out);

	template < class SymbolType, class EpsilonType, class StateType >
	static void transitions(const automaton::EpsilonNFA < SymbolType, EpsilonType, StateType > & fsm, const std::map < StateType, int > & states, std::ostream & out);

	template < class SymbolType, class StateType >
	static void transitions(const automaton::ExtendedNFA < SymbolType, StateType > & fsm, const std::map < StateType, int > & states, std::ostream & out);

	template < class SymbolType, class StateType >
	static void transitions(const automaton::CompactNFA < SymbolType, StateType > & fsm, const std::map < StateType, int > & states, std::ostream & out);

	template < class SymbolType, class RankType, class StateType >
	static void transitions(const automaton::NFTA < SymbolType, RankType, StateType > & fsm, const std::map < StateType, int > & states, std::ostream & out);

	template < class SymbolType, class RankType, class StateType >
	static void transitions(const automaton::DFTA < SymbolType, RankType, StateType > & fsm, const std::map < StateType, int > & states, std::ostream & out);

	template < class SymbolType, class EpsilonType, class StateType >
	static void transitions(const automaton::DPDA < SymbolType, EpsilonType, StateType > & pda, const std::map < StateType, int > & states, std::ostream & out);

	template < class SymbolType, class EpsilonType, class StateType >
	static void transitions(const automaton::NPDA < SymbolType, EpsilonType, StateType > & pda, const std::map < StateType, int > & states, std::ostream & out);

	template < class SymbolType, class EpsilonType, class StateType >
	static void transitions(const automaton::SinglePopDPDA < SymbolType, EpsilonType, StateType > & tm, const std::map < StateType, int > & states, std::ostream & out);

	template < class SymbolType, class EpsilonType, class StateType >
	static void transitions(const automaton::SinglePopNPDA < SymbolType, EpsilonType, StateType > & tm, const std::map < StateType, int > & states, std::ostream & out);

	template < class SymbolType, class StateType >
	static void transitions(const automaton::InputDrivenDPDA < SymbolType, StateType > & pda, const std::map < StateType, int > & states, std::ostream & out);

	template < class SymbolType, class StateType >
	static void transitions(const automaton::InputDrivenNPDA < SymbolType, StateType > & pda, const std::map < StateType, int > & states, std::ostream & out);

	template < class SymbolType, class StateType >
	static void transitions(const automaton::VisiblyPushdownDPDA < SymbolType, StateType > & tm, const std::map < StateType, int > & states, std::ostream & out);

	template < class SymbolType, class StateType >
	static void transitions(const automaton::VisiblyPushdownNPDA < SymbolType, StateType > & tm, const std::map < StateType, int > & states, std::ostream & out);

	template < class SymbolType, class EpsilonType, class StateType >
	static void transitions(const automaton::RealTimeHeightDeterministicDPDA < SymbolType, EpsilonType, StateType > & tm, const std::map < StateType, int > & states, std::ostream & out);

	template < class SymbolType, class EpsilonType, class StateType >
	static void transitions(const automaton::RealTimeHeightDeterministicNPDA < SymbolType, EpsilonType, StateType > & tm, const std::map < StateType, int > & states, std::ostream & out);

	template < class SymbolType, class StateType >
	static void transitions(const automaton::OneTapeDTM < SymbolType, StateType > & tm, const std::map < StateType, int > & states, std::ostream & out);
public:
	static void convert(std::ostream& out, const automaton::Automaton& a);

	template < class SymbolType, class StateType >
	static void convert(std::ostream& out, const automaton::DFA < SymbolType, StateType > & a);

	template < class SymbolType, class StateType >
	static void convert(std::ostream& out, const automaton::NFA < SymbolType, StateType > & a);

	template < class SymbolType, class StateType >
	static void convert(std::ostream& out, const automaton::MultiInitialStateNFA < SymbolType, StateType >& a);

	template < class SymbolType, class EpsilonType, class StateType >
	static void convert(std::ostream& out, const automaton::EpsilonNFA < SymbolType, EpsilonType, StateType > & a);

	template < class SymbolType, class StateType >
	static void convert(std::ostream& out, const automaton::ExtendedNFA < SymbolType, StateType > & a);

	template < class SymbolType, class StateType >
	static void convert(std::ostream& out, const automaton::CompactNFA < SymbolType, StateType > & a);

	template < class SymbolType, class RankType, class StateType >
	static void convert(std::ostream& out, const automaton::NFTA < SymbolType, RankType, StateType > & a);

	template < class SymbolType, class RankType, class StateType >
	static void convert(std::ostream& out, const automaton::DFTA < SymbolType, RankType, StateType > & a);

	template < class SymbolType, class EpsilonType, class StateType >
	static void convert(std::ostream& out, const automaton::DPDA < SymbolType, EpsilonType, StateType > & a);

	template < class SymbolType, class EpsilonType, class StateType >
	static void convert(std::ostream& out, const automaton::NPDA < SymbolType, EpsilonType, StateType > & a);

	template < class SymbolType, class EpsilonType, class StateType >
	static void convert(std::ostream& out, const automaton::SinglePopDPDA < SymbolType, EpsilonType, StateType > & a);

	template < class SymbolType, class EpsilonType, class StateType >
	static void convert(std::ostream& out, const automaton::SinglePopNPDA < SymbolType, EpsilonType, StateType > & a);

	template < class SymbolType, class StateType >
	static void convert(std::ostream& out, const automaton::InputDrivenDPDA < SymbolType, StateType > & a);

	template < class SymbolType, class StateType >
	static void convert(std::ostream& out, const automaton::InputDrivenNPDA < SymbolType, StateType > & a);

	template < class SymbolType, class StateType >
	static void convert(std::ostream& out, const automaton::VisiblyPushdownDPDA < SymbolType, StateType > & a);

	template < class SymbolType, class StateType >
	static void convert(std::ostream& out, const automaton::VisiblyPushdownNPDA < SymbolType, StateType > & a);

	template < class SymbolType, class EpsilonType, class StateType >
	static void convert(std::ostream& out, const automaton::RealTimeHeightDeterministicDPDA < SymbolType, EpsilonType, StateType > & a);

	template < class SymbolType, class EpsilonType, class StateType >
	static void convert(std::ostream& out, const automaton::RealTimeHeightDeterministicNPDA < SymbolType, EpsilonType, StateType > & a);

	template < class SymbolType, class StateType >
	static void convert(std::ostream& out, const automaton::OneTapeDTM < SymbolType, StateType > & a);
};

template < class SymbolType, class EpsilonType, class StateType >
void DotConverter::convert(std::ostream& out, const automaton::EpsilonNFA < SymbolType, EpsilonType, StateType > & a) {
	out << "digraph automaton {\n";
	out << "rankdir=LR;\n";
	int cnt = 1;

	//Map states to indices
	std::map<StateType, int> states;
	for (const StateType& state : a.getStates()) {
		states.insert(std::make_pair(state, cnt++));
	}

	//Print final states
	for (const StateType& state : a.getFinalStates()) {
		out << "node [shape = doublecircle, label=\"" << alib::StringDataFactory::toString(state) << "\"]; " << states.find(state)->second << ";\n";
	}

	//Print nonfinal states
	for (const auto& state : states) {
		if (!a.getFinalStates().count(state.first)) {
			out << "node [shape = circle, label=\"" << alib::StringDataFactory::toString(state.first) << "\" ]; " << state.second << ";\n";
		}
	}

	//Mark initial states
	out << "node [shape = plaintext, label=\"start\"]; 0; \n";
	out << "0 -> " << states.find(a.getInitialState())->second << ";\n";

	transitions(a, states, out);
	out << "}";
}

template < class SymbolType, class StateType >
void DotConverter::convert(std::ostream& out, const automaton::MultiInitialStateNFA < SymbolType, StateType >& a) {
	out << "digraph automaton {\n";
	out << "rankdir=LR;\n";
	int cnt = 1;

	//Map states to indices
	std::map<StateType, int> states;
	for (const StateType& state : a.getStates()) {
		states.insert(std::make_pair(state, cnt++));
	}

	//Print final states
	for (const StateType& state : a.getFinalStates()) {
		out << "node [shape = doublecircle, label=\"" << alib::StringDataFactory::toString(state) << "\"]; " << states.find(state)->second << ";\n";
	}

	//Print nonfinal states
	for (const auto& state : states) {
		if (!a.getFinalStates().count(state.first)) {
			out << "node [shape = circle, label=\"" << alib::StringDataFactory::toString(state.first) << "\" ]; " << state.second << ";\n";
		}
	}

	//Mark initial states
	out << "node [shape = plaintext, label=\"start\"]; 0; \n";
	for (const StateType& state : a.getInitialStates()) {
		out << "0 -> " << states.find(state)->second << ";\n";
	}

	transitions(a, states, out);
	out << "}";
}

template < class SymbolType, class StateType >
void DotConverter::convert(std::ostream& out, const automaton::NFA < SymbolType, StateType > & a) {
	out << "digraph automaton {\n";
	out << "rankdir=LR;\n";
	int cnt = 1;

	//Map states to indices
	std::map<StateType, int> states;
	for (const StateType& state : a.getStates()) {
		states.insert(std::make_pair(state, cnt++));
	}

	//Print final states
	for (const StateType& state : a.getFinalStates()) {
		out << "node [shape = doublecircle, label=\"" << alib::StringDataFactory::toString(state) << "\"]; " << states.find(state)->second << ";\n";
	}

	//Print nonfinal states
	for (const auto& state : states) {
		if (!a.getFinalStates().count(state.first)) {
			out << "node [shape = circle, label=\"" << alib::StringDataFactory::toString(state.first) << "\" ]; " << state.second << ";\n";
		}
	}

	//Mark initial states
	out << "node [shape = plaintext, label=\"start\"]; 0; \n";
	out << "0 -> " << states.find(a.getInitialState())->second << ";\n";

	transitions(a, states, out);
	out << "}";
}

template < class SymbolType, class StateType >
void DotConverter::convert(std::ostream& out, const automaton::DFA < SymbolType, StateType > & a) {
	out << "digraph automaton {\n";
	out << "rankdir=LR;\n";
	int cnt = 1;

	//Map states to indices
	std::map<StateType, int> states;
	for (const StateType& state : a.getStates()) {
		states.insert(std::make_pair(state, cnt++));
	}

	//Print final states
	for (const StateType& state : a.getFinalStates()) {
		out << "node [shape = doublecircle, label=\"" << alib::StringDataFactory::toString(state) << "\"]; " << states.find(state)->second << ";\n";
	}

	//Print nonfinal states
	for (const auto& state : states) {
		if (!a.getFinalStates().count(state.first)) {
			out << "node [shape = circle, label=\"" << alib::StringDataFactory::toString(state.first) << "\" ]; " << state.second << ";\n";
		}
	}

	//Mark initial states
	out << "node [shape = plaintext, label=\"start\"]; 0; \n";
	out << "0 -> " << states.find(a.getInitialState())->second << ";\n";

	transitions(a, states, out);
	out << "}";
}

template < class SymbolType, class StateType >
void DotConverter::convert(std::ostream& out, const automaton::ExtendedNFA < SymbolType, StateType > & a) {
	out << "digraph automaton {\n";
	out << "rankdir=LR;\n";
	int cnt = 1;

	//Map states to indices
	std::map<StateType, int> states;
	for (const StateType& state : a.getStates()) {
		states.insert(std::make_pair(state, cnt++));
	}

	//Print final states
	for (const StateType& state : a.getFinalStates()) {
		out << "node [shape = doublecircle, label=\"" << alib::StringDataFactory::toString(state) << "\"]; " << states.find(state)->second << ";\n";
	}

	//Print nonfinal states
	for (const auto& state : states) {
		if (!a.getFinalStates().count(state.first)) {
			out << "node [shape = circle, label=\"" << alib::StringDataFactory::toString(state.first) << "\" ]; " << state.second << ";\n";
		}
	}

	//Mark initial states
	out << "node [shape = plaintext, label=\"start\"]; 0; \n";
	out << "0 -> " << states.find(a.getInitialState())->second << ";\n";

	transitions(a, states, out);
	out << "}";
}

template < class SymbolType, class StateType >
void DotConverter::convert(std::ostream& out, const automaton::CompactNFA < SymbolType, StateType > & a) {
	out << "digraph automaton {\n";
	out << "rankdir=LR;\n";
	int cnt = 1;

	//Map states to indices
	std::map<StateType, int> states;
	for (const StateType& state : a.getStates()) {
		states.insert(std::make_pair(state, cnt++));
	}

	//Print final states
	for (const StateType& state : a.getFinalStates()) {
		out << "node [shape = doublecircle, label=\"" << alib::StringDataFactory::toString(state) << "\"]; " << states.find(state)->second << ";\n";
	}

	//Print nonfinal states
	for (const auto& state : states) {
		if (!a.getFinalStates().count(state.first)) {
			out << "node [shape = circle, label=\"" << alib::StringDataFactory::toString(state.first) << "\" ]; " << state.second << ";\n";
		}
	}

	//Mark initial states
	out << "node [shape = plaintext, label=\"start\"]; 0; \n";
	out << "0 -> " << states.find(a.getInitialState())->second << ";\n";

	transitions(a, states, out);
	out << "}";
}

template < class SymbolType, class RankType, class StateType >
void DotConverter::convert(std::ostream& out, const automaton::NFTA < SymbolType, RankType, StateType > & a) {
	out << "digraph automaton {\n";
	out << "rankdir=LR;\n";
	int cnt = 1;

	//Map states to indices
	std::map<StateType, int> states;
	for (const StateType& state : a.getStates()) {
		states.insert(std::make_pair(state, cnt++));
	}

	//Print final states
	for (const StateType& state : a.getFinalStates()) {
		out << "node [shape = doublecircle, label=\"" << alib::StringDataFactory::toString(state) << "\"]; " << states.find(state)->second << ";\n";
	}

	//Print nonfinal states
	for (const auto& state : states) {
		if (!a.getFinalStates().count(state.first)) {
			out << "node [shape = circle, label=\"" << alib::StringDataFactory::toString(state.first) << "\" ]; " << state.second << ";\n";
		}
	}

	transitions(a, states, out);
	out << "}";
}

template < class SymbolType, class RankType, class StateType >
void DotConverter::convert(std::ostream& out, const automaton::DFTA < SymbolType, RankType, StateType > & a) {
	out << "digraph automaton {\n";
	out << "rankdir=LR;\n";
	int cnt = 1;

	//Map states to indices
	std::map<StateType, int> states;
	for (const StateType& state : a.getStates()) {
		states.insert(std::make_pair(state, cnt++));
	}

	//Print final states
	for (const StateType& state : a.getFinalStates()) {
		out << "node [shape = doublecircle, label=\"" << alib::StringDataFactory::toString(state) << "\"]; " << states.find(state)->second << ";\n";
	}

	//Print nonfinal states
	for (const auto& state : states) {
		if (!a.getFinalStates().count(state.first)) {
			out << "node [shape = circle, label=\"" << alib::StringDataFactory::toString(state.first) << "\" ]; " << state.second << ";\n";
		}
	}

	transitions(a, states, out);
	out << "}";
}

template < class SymbolType, class EpsilonType, class StateType >
void DotConverter::convert(std::ostream& out, const automaton::DPDA < SymbolType, EpsilonType, StateType > & a) {
	out << "digraph automaton {\n";
	out << "rankdir=LR;\n";
	int cnt = 1;

	//Map states to indices
	std::map<StateType, int> states;
	for (const StateType& state : a.getStates()) {
		states.insert(std::make_pair(state, cnt++));
	}

	//Print final states
	for (const StateType& state : a.getFinalStates()) {
		out << "node [shape = doublecircle, label=\"" << alib::StringDataFactory::toString(state) << "\"]; " << states.find(state)->second << ";\n";
	}

	//Print nonfinal states
	for (const auto& state : states) {
		if (!a.getFinalStates().count(state.first)) {
			out << "node [shape = circle, label=\"" << alib::StringDataFactory::toString(state.first) << "\" ]; " << state.second << ";\n";
		}
	}

	//Mark initial states
	out << "node [shape = plaintext, label=\"start\"]; 0; \n";
	out << "0 -> " << states.find(a.getInitialState())->second << ";\n";

	transitions(a, states, out);
	out << "}";
}

template < class SymbolType, class EpsilonType, class StateType >
void DotConverter::convert(std::ostream& out, const automaton::SinglePopDPDA < SymbolType, EpsilonType, StateType > & a) {
	out << "digraph automaton {\n";
	out << "rankdir=LR;\n";
	int cnt = 1;

	//Map states to indices
	std::map<StateType, int> states;
	for (const StateType& state : a.getStates()) {
		states.insert(std::make_pair(state, cnt++));
	}

	//Print final states
	for (const StateType& state : a.getFinalStates()) {
		out << "node [shape = doublecircle, label=\"" << alib::StringDataFactory::toString(state) << "\"]; " << states.find(state)->second << ";\n";
	}

	//Print nonfinal states
	for (const auto& state : states) {
		if (!a.getFinalStates().count(state.first)) {
			out << "node [shape = circle, label=\"" << alib::StringDataFactory::toString(state.first) << "\" ]; " << state.second << ";\n";
		}
	}

	//Mark initial states
	out << "node [shape = plaintext, label=\"start\"]; 0; \n";
	out << "0 -> " << states.find(a.getInitialState())->second << ";\n";

	transitions(a, states, out);
	out << "}";
}

template < class SymbolType, class StateType >
void DotConverter::convert(std::ostream& out, const automaton::InputDrivenDPDA < SymbolType, StateType > & a) {
	out << "digraph automaton {\n";
	out << "rankdir=LR;\n";
	int cnt = 1;

	//Map states to indices
	std::map<StateType, int> states;
	for (const StateType& state : a.getStates()) {
		states.insert(std::make_pair(state, cnt++));
	}

	//Print final states
	for (const StateType& state : a.getFinalStates()) {
		out << "node [shape = doublecircle, label=\"" << alib::StringDataFactory::toString(state) << "\"]; " << states.find(state)->second << ";\n";
	}

	//Print nonfinal states
	for (const auto& state : states) {
		if (!a.getFinalStates().count(state.first)) {
			out << "node [shape = circle, label=\"" << alib::StringDataFactory::toString(state.first) << "\" ]; " << state.second << ";\n";
		}
	}

	//Mark initial states
	out << "node [shape = plaintext, label=\"start\"]; 0; \n";
	out << "0 -> " << states.find(a.getInitialState())->second << ";\n";

	transitions(a, states, out);
	out << "}";
}

template < class SymbolType, class StateType >
void DotConverter::convert(std::ostream& out, const automaton::InputDrivenNPDA < SymbolType, StateType > & a) {
	out << "digraph automaton {\n";
	out << "rankdir=LR;\n";
	int cnt = 1;

	//Map states to indices
	std::map<StateType, int> states;
	for (const StateType& state : a.getStates()) {
		states.insert(std::make_pair(state, cnt++));
	}

	//Print final states
	for (const StateType& state : a.getFinalStates()) {
		out << "node [shape = doublecircle, label=\"" << alib::StringDataFactory::toString(state) << "\"]; " << states.find(state)->second << ";\n";
	}

	//Print nonfinal states
	for (const auto& state : states) {
		if (!a.getFinalStates().count(state.first)) {
			out << "node [shape = circle, label=\"" << alib::StringDataFactory::toString(state.first) << "\" ]; " << state.second << ";\n";
		}
	}

	//Mark initial states
	out << "node [shape = plaintext, label=\"start\"]; 0; \n";
	out << "0 -> " << states.find(a.getInitialState())->second << ";\n";

	transitions(a, states, out);
	out << "}";
}

template < class SymbolType, class StateType >
void DotConverter::convert(std::ostream& out, const automaton::VisiblyPushdownDPDA < SymbolType, StateType > & a) {
	out << "digraph automaton {\n";
	out << "rankdir=LR;\n";
	int cnt = 1;

	//Map states to indices
	std::map<StateType, int> states;
	for (const StateType& state : a.getStates()) {
		states.insert(std::make_pair(state, cnt++));
	}

	//Print final states
	for (const StateType& state : a.getFinalStates()) {
		out << "node [shape = doublecircle, label=\"" << alib::StringDataFactory::toString(state) << "\"]; " << states.find(state)->second << ";\n";
	}

	//Print nonfinal states
	for (const auto& state : states) {
		if (!a.getFinalStates().count(state.first)) {
			out << "node [shape = circle, label=\"" << alib::StringDataFactory::toString(state.first) << "\" ]; " << state.second << ";\n";
		}
	}

	//Mark initial states
	out << "node [shape = plaintext, label=\"start\"]; 0; \n";
	out << "0 -> " << states.find(a.getInitialState())->second << ";\n";

	transitions(a, states, out);
	out << "}";
}

template < class SymbolType, class StateType >
void DotConverter::convert(std::ostream& out, const automaton::VisiblyPushdownNPDA < SymbolType, StateType > & a) {
	out << "digraph automaton {\n";
	out << "rankdir=LR;\n";
	int cnt = 1;

	//Map states to indices
	std::map<StateType, int> states;
	for (const StateType& state : a.getStates()) {
		states.insert(std::make_pair(state, cnt++));
	}

	//Print final states
	for (const StateType& state : a.getFinalStates()) {
		out << "node [shape = doublecircle, label=\"" << alib::StringDataFactory::toString(state) << "\"]; " << states.find(state)->second << ";\n";
	}

	//Print nonfinal states
	for (const auto& state : states) {
		if (!a.getFinalStates().count(state.first)) {
			out << "node [shape = circle, label=\"" << alib::StringDataFactory::toString(state.first) << "\" ]; " << state.second << ";\n";
		}
	}

	//Mark initial states
	out << "node [shape = plaintext, label=\"start\"]; 0; \n";
	for (const StateType& state : a.getInitialStates()) {
		out << "0 -> " << states.find(state)->second << ";\n";
	}

	transitions(a, states, out);
	out << "}";
}

template < class SymbolType, class EpsilonType, class StateType >
void DotConverter::convert(std::ostream& out, const automaton::RealTimeHeightDeterministicDPDA < SymbolType, EpsilonType, StateType > & a) {
	out << "digraph automaton {\n";
	out << "rankdir=LR;\n";
	int cnt = 1;

	//Map states to indices
	std::map<StateType, int> states;
	for (const StateType& state : a.getStates()) {
		states.insert(std::make_pair(state, cnt++));
	}

	//Print final states
	for (const StateType& state : a.getFinalStates()) {
		out << "node [shape = doublecircle, label=\"" << alib::StringDataFactory::toString(state) << "\"]; " << states.find(state)->second << ";\n";
	}

	//Print nonfinal states
	for (const auto& state : states) {
		if (!a.getFinalStates().count(state.first)) {
			out << "node [shape = circle, label=\"" << alib::StringDataFactory::toString(state.first) << "\" ]; " << state.second << ";\n";
		}
	}

	//Mark initial states
	out << "node [shape = plaintext, label=\"start\"]; 0; \n";
	out << "0 -> " << states.find(a.getInitialState())->second << ";\n";

	transitions(a, states, out);
	out << "}";
}

template < class SymbolType, class EpsilonType, class StateType >
void DotConverter::convert(std::ostream& out, const automaton::RealTimeHeightDeterministicNPDA < SymbolType, EpsilonType, StateType > & a) {
	out << "digraph automaton {\n";
	out << "rankdir=LR;\n";
	int cnt = 1;

	//Map states to indices
	std::map<StateType, int> states;
	for (const StateType& state : a.getStates()) {
		states.insert(std::make_pair(state, cnt++));
	}

	//Print final states
	for (const StateType& state : a.getFinalStates()) {
		out << "node [shape = doublecircle, label=\"" << alib::StringDataFactory::toString(state) << "\"]; " << states.find(state)->second << ";\n";
	}

	//Print nonfinal states
	for (const auto& state : states) {
		if (!a.getFinalStates().count(state.first)) {
			out << "node [shape = circle, label=\"" << alib::StringDataFactory::toString(state.first) << "\" ]; " << state.second << ";\n";
		}
	}

	//Mark initial states
	out << "node [shape = plaintext, label=\"start\"]; 0; \n";
	for (const StateType& state : a.getInitialStates()) {
		out << "0 -> " << states.find(state)->second << ";\n";
	}

	transitions(a, states, out);
	out << "}";
}

template < class SymbolType, class EpsilonType, class StateType >
void DotConverter::convert(std::ostream& out, const automaton::NPDA < SymbolType, EpsilonType, StateType > & a) {
	out << "digraph automaton {\n";
	out << "rankdir=LR;\n";
	int cnt = 1;

	//Map states to indices
	std::map<StateType, int> states;
	for (const StateType& state : a.getStates()) {
		states.insert(std::make_pair(state, cnt++));
	}

	//Print final states
	for (const StateType& state : a.getFinalStates()) {
		out << "node [shape = doublecircle, label=\"" << alib::StringDataFactory::toString(state) << "\"]; " << states.find(state)->second << ";\n";
	}

	//Print nonfinal states
	for (const auto& state : states) {
		if (!a.getFinalStates().count(state.first)) {
			out << "node [shape = circle, label=\"" << alib::StringDataFactory::toString(state.first) << "\" ]; " << state.second << ";\n";
		}
	}

	//Mark initial states
	out << "node [shape = plaintext, label=\"start\"]; 0; \n";
	out << "0 -> " << states.find(a.getInitialState())->second << ";\n";

	transitions(a, states, out);
	out << "}";
}

template < class SymbolType, class EpsilonType, class StateType >
void DotConverter::convert(std::ostream& out, const automaton::SinglePopNPDA < SymbolType, EpsilonType, StateType > & a) {
	out << "digraph automaton {\n";
	out << "rankdir=LR;\n";
	int cnt = 1;

	//Map states to indices
	std::map<StateType, int> states;
	for (const StateType& state : a.getStates()) {
		states.insert(std::make_pair(state, cnt++));
	}

	//Print final states
	for (const StateType& state : a.getFinalStates()) {
		out << "node [shape = doublecircle, label=\"" << alib::StringDataFactory::toString(state) << "\"]; " << states.find(state)->second << ";\n";
	}

	//Print nonfinal states
	for (const auto& state : states) {
		if (!a.getFinalStates().count(state.first)) {
			out << "node [shape = circle, label=\"" << alib::StringDataFactory::toString(state.first) << "\" ]; " << state.second << ";\n";
		}
	}

	//Mark initial states
	out << "node [shape = plaintext, label=\"start\"]; 0; \n";
	out << "0 -> " << states.find(a.getInitialState())->second << ";\n";

	transitions(a, states, out);
	out << "}";
}

template < class SymbolType, class StateType >
void DotConverter::convert(std::ostream& out, const automaton::OneTapeDTM < SymbolType, StateType > & a) {
	out << "digraph automaton {\n";
	out << "rankdir=LR;\n";
	int cnt = 1;

	//Map states to indices
	std::map<StateType, int> states;
	for (const StateType& state : a.getStates()) {
		states.insert(std::make_pair(state, cnt++));
	}

	//Print final states
	for (const StateType& state : a.getFinalStates()) {
		out << "node [shape = doublecircle, label=\"" << alib::StringDataFactory::toString(state) << "\"]; " << states.find(state)->second << ";\n";
	}

	//Print nonfinal states
	for (const auto& state : states) {
		if (!a.getFinalStates().count(state.first)) {
			out << "node [shape = circle, label=\"" << alib::StringDataFactory::toString(state.first) << "\" ]; " << state.second << ";\n";
		}
	}

	//Mark initial states
	out << "node [shape = plaintext, label=\"start\"]; 0; \n";
	out << "0 -> " << states.find(a.getInitialState())->second << ";\n";

	transitions(a, states, out);
	out << "}";
}

template < class SymbolType, class EpsilonType, class StateType >
void DotConverter::transitions(const automaton::EpsilonNFA < SymbolType, EpsilonType, StateType > & fsm, const std::map<StateType, int>& states, std::ostream& out) {
	std::map<std::pair<int, int>, std::string> transitions;

	//put transitions from automaton to "transitions"
	for (const auto& transition : fsm.getTransitions()) {
		std::string symbol;
		if (transition.first.second.template is< EpsilonType >()) {
			symbol = "&epsilon;";
		} else {
			symbol = alib::StringDataFactory::toString(std::get<1>(transition.first).template get<SymbolType>());
		}

		for(const StateType& to : transition.second) {
			std::pair<int, int> key(states.find(transition.first.first)->second, states.find(to)->second);
			std::map<std::pair<int, int>, std::string>::iterator mapit = transitions.find(key);

			if (mapit == transitions.end()) {
				transitions.insert(std::make_pair(key, symbol));
			} else {
				mapit->second += ",";

				size_t pos = mapit->second.find_last_of("\n");
				if(pos == std::string::npos) pos = 0;
				if(mapit->second.size() - pos > 100) mapit->second += "\n";
				else mapit->second += " ";

				mapit->second += symbol;
			}
		}
	}

	//print the map
	for (std::pair<const std::pair<int, int>, std::string>& transition : transitions) {
		out << transition.first.first << " -> " << transition.first.second;
		replace(transition.second, "\n", "\\n");
		out << "[label=\"" << transition.second << "\"]\n";
	}
}

template < class SymbolType, class StateType >
void DotConverter::transitions(const automaton::MultiInitialStateNFA < SymbolType, StateType >& fsm, const std::map<StateType, int>& states, std::ostream& out) {
	std::map<std::pair<int, int>, std::string> transitions;

	//put transitions from automaton to "transitions"
	for (const auto& transition : fsm.getTransitions()) {
		std::string symbol = alib::StringDataFactory::toString(transition.first.second);

		for(const StateType& to : transition.second) {
			std::pair<int, int> key(states.find(transition.first.first)->second, states.find(to)->second);
			std::map<std::pair<int, int>, std::string>::iterator mapit = transitions.find(key);

			if (mapit == transitions.end()) {
				transitions.insert(std::make_pair(key, symbol));
			} else {
				mapit->second += ",";

				size_t pos = mapit->second.find_last_of("\n");
				if(pos == std::string::npos) pos = 0;
				if(mapit->second.size() - pos > 100) mapit->second += "\n";
				else mapit->second += " ";

				mapit->second += symbol;
			}
		}
	}

	//print the map
	for (std::pair< const std::pair<int, int>, std::string>& transition : transitions) {
		out << transition.first.first << " -> " << transition.first.second;
		replace(transition.second, "\n", "\\n");
		out << "[label=\"" << transition.second << "\"]\n";
	}
}

template < class SymbolType, class StateType >
void DotConverter::transitions(const automaton::NFA < SymbolType, StateType > & fsm, const std::map<StateType, int>& states, std::ostream& out) {
	std::map<std::pair<int, int>, std::string> transitions;

	//put transitions from automaton to "transitions"
	for (const auto& transition : fsm.getTransitions()) {
		std::string symbol = alib::StringDataFactory::toString(transition.first.second);

		for(const StateType& to : transition.second) {
			std::pair<int, int> key(states.find(transition.first.first)->second, states.find(to)->second);
			std::map<std::pair<int, int>, std::string>::iterator mapit = transitions.find(key);

			if (mapit == transitions.end()) {
				transitions.insert(std::make_pair(key, symbol));
			} else {
				mapit->second += ",";

				size_t pos = mapit->second.find_last_of("\n");
				if(pos == std::string::npos) pos = 0;
				if(mapit->second.size() - pos > 100) mapit->second += "\n";
				else mapit->second += " ";

				mapit->second += symbol;
			}
		}
	}

	//print the map
	for (std::pair<const std::pair<int, int>, std::string>& transition : transitions) {
		out << transition.first.first << " -> " << transition.first.second;
		replace(transition.second, "\n", "\\n");
		out << "[label=\"" << transition.second << "\"]\n";
	}
}

template < class SymbolType, class StateType >
void DotConverter::transitions(const automaton::DFA < SymbolType, StateType > & fsm, const std::map<StateType, int>& states, std::ostream& out) {
	std::map<std::pair<int, int>, std::string> transitions;

	//put transitions from automaton to "transitions"
	for (const auto& transition : fsm.getTransitions()) {
		std::string symbol = alib::StringDataFactory::toString(transition.first.second);

		std::pair<int, int> key(states.find(transition.first.first)->second, states.find(transition.second)->second);
		std::map<std::pair<int, int>, std::string>::iterator mapit = transitions.find(key);

		if (mapit == transitions.end()) {
			transitions.insert(std::make_pair(key, symbol));
		} else {
			mapit->second += ",";

			size_t pos = mapit->second.find_last_of("\n");
			if(pos == std::string::npos) pos = 0;
			if(mapit->second.size() - pos > 100) mapit->second += "\n";
			else mapit->second += " ";

			mapit->second += symbol;
		}
	}

	//print the map
	for (std::pair<const std::pair<int, int>, std::string>& transition : transitions) {
		out << transition.first.first << " -> " << transition.first.second;
		replace(transition.second, "\n", "\\n");
		out << "[label=\"" << transition.second << "\"]\n";
	}
}

template < class SymbolType, class StateType >
void DotConverter::transitions(const automaton::ExtendedNFA < SymbolType, StateType > & fsm, const std::map<StateType, int>& states, std::ostream& out) {
	std::map<std::pair<int, int>, std::string> transitions;

	//put transitions from automaton to "transitions"
	for (const auto& transition : fsm.getTransitions()) {
		std::string symbol = alib::StringDataFactory::toString(transition.first.second);

		for(const StateType& to : transition.second) {
			std::pair<int, int> key(states.find(transition.first.first)->second, states.find(to)->second);
			std::map<std::pair<int, int>, std::string>::iterator mapit = transitions.find(key);

			if (mapit == transitions.end()) {
				transitions.insert(std::make_pair(key, symbol));
			} else {
				mapit->second += ",";

				size_t pos = mapit->second.find_last_of("\n");
				if(pos == std::string::npos) pos = 0;
				if(mapit->second.size() - pos > 100) mapit->second += "\n";
				else mapit->second += " ";

				mapit->second += symbol;
			}
		}
	}

	//print the map
	for (std::pair<const std::pair<int, int>, std::string>& transition : transitions) {
		out << transition.first.first << " -> " << transition.first.second;
		replace(transition.second, "\n", "\\n");
		out << "[label=\"" << transition.second << "\"]\n";
	}
}

template < class SymbolType, class StateType >
void DotConverter::transitions(const automaton::CompactNFA < SymbolType, StateType > & fsm, const std::map<StateType, int>& states, std::ostream& out) {
	std::map<std::pair<int, int>, std::string> transitions;

	//put transitions from automaton to "transitions"
	for (const auto& transition : fsm.getTransitions()) {
		std::string symbol = alib::StringDataFactory::toString(string::stringFrom ( transition.first.second ) );

		for(const StateType& to : transition.second) {
			std::pair<int, int> key(states.find(transition.first.first)->second, states.find(to)->second);
			std::map<std::pair<int, int>, std::string>::iterator mapit = transitions.find(key);

			if (mapit == transitions.end()) {
				transitions.insert(std::make_pair(key, symbol));
			} else {
				mapit->second += ",";

				size_t pos = mapit->second.find_last_of("\n");
				if(pos == std::string::npos) pos = 0;
				if(mapit->second.size() - pos > 100) mapit->second += "\n";
				else mapit->second += " ";

				mapit->second += symbol;
			}
		}
	}

	//print the map
	for (std::pair<const std::pair<int, int>, std::string>& transition : transitions) {
		out << transition.first.first << " -> " << transition.first.second;
		replace(transition.second, "\n", "\\n");
		out << "[label=\"" << transition.second << "\"]\n";
	}
}

template < class SymbolType, class RankType, class StateType >
void DotConverter::transitions(const automaton::NFTA < SymbolType, RankType, StateType > & fta, const std::map<StateType, int>& states, std::ostream& out) {
	std::map<std::pair<int, std::vector<int>>, std::string> transitions;

	//put transitions from automaton to "transitions"
	for (const auto& transition : fta.getTransitions()) {
		std::string symbol = alib::StringDataFactory::toString(transition.first.first.getSymbol());
		symbol += std::to_string(transition.first.first.getRank());

		for(const StateType& to : transition.second) {
			std::pair<int, std::vector<int>> key(states.find(to)->second, {});
			for(const StateType& state : transition.first.second) {
				key.second.push_back(states.find(state)->second);
			}
			std::map<std::pair<int, std::vector<int>>, std::string>::iterator mapit = transitions.find(key);

			if (mapit == transitions.end()) {
				transitions.insert(std::make_pair(key, symbol));
			} else {
				mapit->second += ",";

				size_t pos = mapit->second.find_last_of("\n");
				if(pos == std::string::npos) pos = 0;
				if(mapit->second.size() - pos > 100) mapit->second += "\n";
				else mapit->second += " ";

				mapit->second += symbol;
			}
		}
	}

	//Print auxilary dots
	for (unsigned i = 1; i < transitions.size(); i++) {
		out << "node [shape = point, label=\"\"]; " << states.size() + i << ";\n";
	}

	//print the map
	unsigned i = states.size() + 1;
	for (std::pair<const std::pair<int, std::vector<int>>, std::string>& transition : transitions) {
		out << i << " -> " << transition.first.first;
		replace(transition.second, "\n", "\\n");
		out << "[label=\"" << transition.second << "\"]\n";
		unsigned j = 0;
		for(int from : transition.first.second) {
			out << from << " -> " << i;
			out << "[label=\"" << j << "\"]\n";
			j++;
		}
		i++;
	}
}

template < class SymbolType, class RankType, class StateType >
void DotConverter::transitions(const automaton::DFTA < SymbolType, RankType, StateType > & fta, const std::map<StateType, int>& states, std::ostream& out) {
	std::map<std::pair<int, std::vector<int>>, std::string> transitions;

	//put transitions from automaton to "transitions"
	for (const auto& transition : fta.getTransitions()) {
		std::string symbol = alib::StringDataFactory::toString(transition.first.first.getSymbol());
		symbol += std::to_string(transition.first.first.getRank());

		std::pair<int, std::vector<int>> key(states.find(transition.second)->second, {});
		for(const StateType& state : transition.first.second) {
			key.second.push_back(states.find(state)->second);
		}
		std::map<std::pair<int, std::vector<int>>, std::string>::iterator mapit = transitions.find(key);

		if (mapit == transitions.end()) {
			transitions.insert(std::make_pair(key, symbol));
		} else {
			mapit->second += ",";

			size_t pos = mapit->second.find_last_of("\n");
			if(pos == std::string::npos) pos = 0;
			if(mapit->second.size() - pos > 100) mapit->second += "\n";
			else mapit->second += " ";

			mapit->second += symbol;
		}
	}

	//Print auxilary dots
	for (unsigned i = 1; i < transitions.size(); i++) {
		out << "node [shape = point, label=\"\"]; " << states.size() + i << ";\n";
	}

	//print the map
	unsigned i = states.size() + 1;
	for (std::pair<const std::pair<int, std::vector<int>>, std::string>& transition : transitions) {
		out << i << " -> " << transition.first.first;
		replace(transition.second, "\n", "\\n");
		out << "[label=\"" << transition.second << "\"]\n";
		unsigned j = 0;
		for(int from : transition.first.second) {
			out << from << " -> " << i;
			out << "[label=\"" << j << "\"]\n";
			j++;
		}
		i++;
	}
}

template < class SymbolType, class EpsilonType, class StateType >
void DotConverter::transitions(const automaton::DPDA < SymbolType, EpsilonType, StateType > & pda, const std::map<StateType, int>& states, std::ostream& out) {
	std::map<std::pair<int, int>, std::string> transitions;

	for (const auto& transition : pda.getTransitions()) {
		std::string symbol;

		//input symbol
		if (std::get<1>(transition.first).template is< EpsilonType >()) {
			symbol = "&epsilon;";
		} else {
			symbol = alib::StringDataFactory::toString(std::get<1>(transition.first).template get<SymbolType>());
		}

		symbol += " |";

		//Pop part
		if (std::get<2>(transition.first).size() == 0) {
			symbol += " &epsilon;";
		} else {
			for (SymbolType symb : std::get<2>(transition.first)) {
				symbol += " " + alib::StringDataFactory::toString(symb);
			}

		}

		symbol += " ->";

		//Push part
		if (transition.second.second.size() == 0) {
			symbol += " &epsilon;";
		} else {
			for (SymbolType symb : transition.second.second) {
				symbol += " " + alib::StringDataFactory::toString(symb);
			}

		}

		//Insert into map
		std::pair<int, int> key(states.find(std::get<0>(transition.first))->second, states.find(transition.second.first)->second);
		std::map<std::pair<int, int>, std::string>::iterator mapit = transitions.find(key);

		if (mapit == transitions.end()) {
			transitions.insert(std::make_pair(key, symbol));
		} else {
			mapit->second += ",";
			
			size_t pos = mapit->second.find_last_of("\n");
			if(pos == std::string::npos) pos = 0;
			if(mapit->second.size() - pos > 100) mapit->second += "\n";
			else mapit->second += " ";

			mapit->second += symbol;
		}
	}

	//print the map
	for (std::pair<const std::pair<int, int>, std::string>& transition : transitions) {
		out << transition.first.first << " -> " << transition.first.second;
		replace(transition.second, "\n", "\\n");
		out << "[label=\"" <<  transition.second << "\"]\n";
	}
}

template < class SymbolType, class EpsilonType, class StateType >
void DotConverter::transitions(const automaton::SinglePopDPDA < SymbolType, EpsilonType, StateType > & pda, const std::map<StateType, int>& states, std::ostream& out) {
	std::map<std::pair<int, int>, std::string> transitions;

	for (const auto& transition : pda.getTransitions()) {
		std::string symbol;

		//input symbol
		if (std::get<1>(transition.first).template is< EpsilonType >()) {
			symbol = "&epsilon;";
		} else {
			symbol = alib::StringDataFactory::toString(std::get<1>(transition.first).template get<SymbolType>());
		}

		symbol += " |";

		//Pop part
		symbol += " " + alib::StringDataFactory::toString(std::get<2>(transition.first));

		symbol += " ->";

		//Push part
		if (transition.second.second.size() == 0) {
			symbol += " &epsilon;";
		} else {
			for (SymbolType symb : transition.second.second) {
				symbol += " " + alib::StringDataFactory::toString(symb);
			}

		}

		//Insert into map
		std::pair<int, int> key(states.find(std::get<0>(transition.first))->second, states.find(transition.second.first)->second);
		std::map<std::pair<int, int>, std::string>::iterator mapit = transitions.find(key);

		if (mapit == transitions.end()) {
			transitions.insert(std::make_pair(key, symbol));
		} else {
			mapit->second += ",";

			size_t pos = mapit->second.find_last_of("\n");
			if(pos == std::string::npos) pos = 0;
			if(mapit->second.size() - pos > 100) mapit->second += "\n";
			else mapit->second += " ";

			mapit->second += symbol;
		}
	}

	//print the map
	for (std::pair<const std::pair<int, int>, std::string>& transition : transitions) {
		out << transition.first.first << " -> " << transition.first.second;
		replace(transition.second, "\n", "\\n");
		out << "[label=\"" << transition.second << "\"]\n";
	}
}

template < class SymbolType, class StateType >
void DotConverter::transitions(const automaton::InputDrivenDPDA < SymbolType, StateType > & pda, const std::map<StateType, int>& states, std::ostream& out) {
	std::map<std::pair<int, int>, std::string> transitions;

	const auto& symbolToPDSOperation = pda.getPushdownStoreOperations();
	for (const auto& transition : pda.getTransitions()) {
		const auto& pop = symbolToPDSOperation.find(transition.first.second)->second.first;
		const auto& push = symbolToPDSOperation.find(transition.first.second)->second.second;

		std::string symbol;

		//input symbol
		symbol = alib::StringDataFactory::toString(transition.first.second);

		symbol += " |";

		//Pop part
		if (pop.size() == 0) {
			symbol += " &epsilon;";
		} else {
			for (SymbolType symb : pop) {
				symbol += " " + alib::StringDataFactory::toString(symb);
			}

		}

		symbol += " ->";

		const auto& to = transition.second;
		//Push part
		if (push.size() == 0) {
			symbol += " &epsilon;";
		} else {
			for (SymbolType symb : push) {
				symbol += " " + alib::StringDataFactory::toString(symb);
			}

		}

		//Insert into map
		std::pair<int, int> key(states.find(transition.first.first)->second, states.find(to)->second);
		std::map<std::pair<int, int>, std::string>::iterator mapit = transitions.find(key);

		if (mapit == transitions.end()) {
			transitions.insert(std::make_pair(key, symbol));
		} else {
			mapit->second += ",";

			size_t pos = mapit->second.find_last_of("\n");
			if(pos == std::string::npos) pos = 0;
			if(mapit->second.size() - pos > 100) mapit->second += "\n";
			else mapit->second += " ";

			mapit->second += symbol;
		}
	}

	//print the map
	for (std::pair<const std::pair<int, int>, std::string>& transition : transitions) {
		out << transition.first.first << " -> " << transition.first.second;
		replace(transition.second, "\n", "\\n");
		out << "[label=\"" << transition.second << "\"]\n";
	}
}

template < class SymbolType, class StateType >
void DotConverter::transitions(const automaton::InputDrivenNPDA < SymbolType, StateType > & pda, const std::map<StateType, int>& states, std::ostream& out) {
	std::map<std::pair<int, int>, std::string> transitions;

	const auto& symbolToPDSOperation = pda.getPushdownStoreOperations();
	for (const auto& transition : pda.getTransitions()) {
		const auto& pop = symbolToPDSOperation.find(transition.first.second)->second.first;
		const auto& push = symbolToPDSOperation.find(transition.first.second)->second.second;

		std::string symbol;

		//input symbol
		symbol = alib::StringDataFactory::toString(transition.first.second);

		symbol += " |";

		//Pop part
		if (pop.size() == 0) {
			symbol += " &epsilon;";
		} else {
			for (SymbolType symb : pop) {
				symbol += " " + alib::StringDataFactory::toString(symb);
			}

		}

		symbol += " ->";

		std::string symbol2;
		for(const auto& to : transition.second) {
			symbol2 = symbol;
			//Push part
			if (push.size() == 0) {
				symbol2 += " &epsilon;";
			} else {
				for (SymbolType symb : push) {
					symbol2 += " " + alib::StringDataFactory::toString(symb);
				}

			}

			//Insert into map
			std::pair<int, int> key(states.find(transition.first.first)->second, states.find(to)->second);
			std::map<std::pair<int, int>, std::string>::iterator mapit = transitions.find(key);

			if (mapit == transitions.end()) {
				transitions.insert(std::make_pair(key, symbol2));
			} else {
				mapit->second += ",";

				size_t pos = mapit->second.find_last_of("\n");
				if(pos == std::string::npos) pos = 0;
				if(mapit->second.size() - pos > 100) mapit->second += "\n";
				else mapit->second += " ";

				mapit->second += symbol2;
			}
		}
	}

	//print the map
	for (std::pair<const std::pair<int, int>, std::string>& transition : transitions) {
		out << transition.first.first << " -> " << transition.first.second;
		replace(transition.second, "\n", "\\n");
		out << "[label=\"" << transition.second << "\"]\n";
	}
}

template < class SymbolType, class StateType >
void DotConverter::transitions(const automaton::VisiblyPushdownDPDA < SymbolType, StateType > & pda, const std::map<StateType, int>& states, std::ostream& out) {
	std::map<std::pair<int, int>, std::string> transitions;

	for (const auto& transition : pda.getCallTransitions()) {
		std::string symbol;

		//input symbol
		symbol = alib::StringDataFactory::toString(transition.first.second);

		symbol += " |";

		//Pop part
		symbol += " &epsilon;";
		symbol += " ->";

		symbol += " " + alib::StringDataFactory::toString(transition.second.second);

		//Insert into map
		std::pair<int, int> key(states.find(transition.first.first)->second, states.find(transition.second.first)->second);
		std::map<std::pair<int, int>, std::string>::iterator mapit = transitions.find(key);

		if (mapit == transitions.end()) {
			transitions.insert(std::make_pair(key, symbol));
		} else {
			mapit->second += ",";

			size_t pos = mapit->second.find_last_of("\n");
			if(pos == std::string::npos) pos = 0;
			if(mapit->second.size() - pos > 100) mapit->second += "\n";
			else mapit->second += " ";

			mapit->second += symbol;
		}
	}

	for (const auto& transition : pda.getReturnTransitions()) {
		std::string symbol;

		//input symbol
		symbol = alib::StringDataFactory::toString(std::get<1>(transition.first));

		symbol += " |";

		//Pop part
		symbol += " " + alib::StringDataFactory::toString(std::get<2>(transition.first));
		symbol += " ->";

		symbol += " &epsilon;";

		//Insert into map
		std::pair<int, int> key(states.find(std::get<0>(transition.first))->second, states.find(transition.second)->second);
		std::map<std::pair<int, int>, std::string>::iterator mapit = transitions.find(key);

		if (mapit == transitions.end()) {
			transitions.insert(std::make_pair(key, symbol));
		} else {
			mapit->second += ",";

			size_t pos = mapit->second.find_last_of("\n");
			if(pos == std::string::npos) pos = 0;
			if(mapit->second.size() - pos > 100) mapit->second += "\n";
			else mapit->second += " ";

			mapit->second += symbol;
		}
	}

	for (const auto& transition : pda.getLocalTransitions()) {
		std::string symbol;

		//input symbol
		symbol = alib::StringDataFactory::toString(transition.first.second);

		symbol += " |";

		//Pop part
		symbol += " &epsilon;";
		symbol += " ->";

		symbol += " &epsilon;";

		//Insert into map
		std::pair<int, int> key(states.find(transition.first.first)->second, states.find(transition.second)->second);
		std::map<std::pair<int, int>, std::string>::iterator mapit = transitions.find(key);

		if (mapit == transitions.end()) {
			transitions.insert(std::make_pair(key, symbol));
		} else {
			mapit->second += ",";

			size_t pos = mapit->second.find_last_of("\n");
			if(pos == std::string::npos) pos = 0;
			if(mapit->second.size() - pos > 100) mapit->second += "\n";
			else mapit->second += " ";

			mapit->second += symbol;
		}
	}

	//print the map
	for (std::pair<const std::pair<int, int>, std::string>& transition : transitions) {
		out << transition.first.first << " -> " << transition.first.second;
		replace(transition.second, "\n", "\\n");
		out << "[label=\"" << transition.second << "\"]\n";
	}
}

template < class SymbolType, class StateType >
void DotConverter::transitions(const automaton::VisiblyPushdownNPDA < SymbolType, StateType > & pda, const std::map<StateType, int>& states, std::ostream& out) {
	std::map<std::pair<int, int>, std::string> transitions;

	for (const auto& transition : pda.getCallTransitions()) {
		std::string symbol;

		//input symbol
		symbol = alib::StringDataFactory::toString(transition.first.second);

		symbol += " |";

		//Pop part
		symbol += " &epsilon;";
		symbol += " ->";

		std::string symbol2;
		for(const auto& to : transition.second) {
			symbol2 = symbol;
			symbol2 += " " + alib::StringDataFactory::toString(to.second);

			//Insert into map
			std::pair<int, int> key(states.find(transition.first.first)->second, states.find(to.first)->second);
			std::map<std::pair<int, int>, std::string>::iterator mapit = transitions.find(key);

			if (mapit == transitions.end()) {
				transitions.insert(std::make_pair(key, symbol2));
			} else {
				mapit->second += ",";

				size_t pos = mapit->second.find_last_of("\n");
				if(pos == std::string::npos) pos = 0;
				if(mapit->second.size() - pos > 100) mapit->second += "\n";
				else mapit->second += " ";

				mapit->second += symbol2;
			}
		}
	}

	for (const auto& transition : pda.getReturnTransitions()) {
		std::string symbol;

		//input symbol
		symbol = alib::StringDataFactory::toString(std::get<1>(transition.first));

		symbol += " |";

		//Pop part
		symbol += " " + alib::StringDataFactory::toString(std::get<2>(transition.first));
		symbol += " ->";

		std::string symbol2;
		for(const auto& to : transition.second) {
			symbol2 = symbol;
			symbol2 += " &epsilon;";

			//Insert into map
			std::pair<int, int> key(states.find(std::get<0>(transition.first))->second, states.find(to)->second);
			std::map<std::pair<int, int>, std::string>::iterator mapit = transitions.find(key);

			if (mapit == transitions.end()) {
				transitions.insert(std::make_pair(key, symbol2));
			} else {
				mapit->second += ",";

				size_t pos = mapit->second.find_last_of("\n");
				if(pos == std::string::npos) pos = 0;
				if(mapit->second.size() - pos > 100) mapit->second += "\n";
				else mapit->second += " ";

				mapit->second += symbol2;
			}
		}
	}

	for (const auto& transition : pda.getLocalTransitions()) {
		std::string symbol;

		//input symbol
		symbol = alib::StringDataFactory::toString(transition.first.second);

		symbol += " |";

		//Pop part
		symbol += " &epsilon;";
		symbol += " ->";

		std::string symbol2;
		for(const auto& to : transition.second) {
			symbol2 = symbol;
			symbol2 += " &epsilon;";

			//Insert into map
			std::pair<int, int> key(states.find(transition.first.first)->second, states.find(to)->second);
			std::map<std::pair<int, int>, std::string>::iterator mapit = transitions.find(key);

			if (mapit == transitions.end()) {
				transitions.insert(std::make_pair(key, symbol2));
			} else {
				mapit->second += ",";

				size_t pos = mapit->second.find_last_of("\n");
				if(pos == std::string::npos) pos = 0;
				if(mapit->second.size() - pos > 100) mapit->second += "\n";
				else mapit->second += " ";

				mapit->second += symbol2;
			}
		}
	}

	//print the map
	for (std::pair<const std::pair<int, int>, std::string>& transition : transitions) {
		out << transition.first.first << " -> " << transition.first.second;
		replace(transition.second, "\n", "\\n");
		out << "[label=\"" << transition.second << "\"]\n";
	}
}

template < class SymbolType, class EpsilonType, class StateType >
void DotConverter::transitions(const automaton::RealTimeHeightDeterministicDPDA < SymbolType, EpsilonType, StateType > & pda, const std::map<StateType, int>& states, std::ostream& out) {
	std::map<std::pair<int, int>, std::string> transitions;

	for (const auto& transition : pda.getCallTransitions()) {
		std::string symbol;

		//input symbol
		if(transition.first.second.template is< EpsilonType >())
			symbol = "&epsilon";
		else
			symbol = alib::StringDataFactory::toString(transition.first.second.template get<SymbolType>());

		symbol += " |";

		//Pop part
		symbol += " &epsilon;";
		symbol += " ->";

		symbol += " " + alib::StringDataFactory::toString(transition.second.second);

		//Insert into map
		std::pair<int, int> key(states.find(transition.first.first)->second, states.find(transition.second.first)->second);
		std::map<std::pair<int, int>, std::string>::iterator mapit = transitions.find(key);

		if (mapit == transitions.end()) {
			transitions.insert(std::make_pair(key, symbol));
		} else {
			mapit->second += ",";

			size_t pos = mapit->second.find_last_of("\n");
			if(pos == std::string::npos) pos = 0;
			if(mapit->second.size() - pos > 100) mapit->second += "\n";
			else mapit->second += " ";

			mapit->second += symbol;
		}
	}

	for (const auto& transition : pda.getReturnTransitions()) {
		std::string symbol;

		//input symbol
		if(std::get<1>(transition.first).template is< EpsilonType >())
			symbol = "&epsilon";
		else
			symbol = alib::StringDataFactory::toString(std::get<1>(transition.first).template get<SymbolType>());

		symbol += " |";

		//Pop part
		symbol += " " + alib::StringDataFactory::toString(std::get<2>(transition.first));
		symbol += " ->";

		symbol += " &epsilon;";

		//Insert into map
		std::pair<int, int> key(states.find(std::get<0>(transition.first))->second, states.find(transition.second)->second);
		std::map<std::pair<int, int>, std::string>::iterator mapit = transitions.find(key);

		if (mapit == transitions.end()) {
			transitions.insert(std::make_pair(key, symbol));
		} else {
			mapit->second += ",";

			size_t pos = mapit->second.find_last_of("\n");
			if(pos == std::string::npos) pos = 0;
			if(mapit->second.size() - pos > 100) mapit->second += "\n";
			else mapit->second += " ";

			mapit->second += symbol;
		}
	}

	for (const auto& transition : pda.getLocalTransitions()) {
		std::string symbol;

		//input symbol
		if(transition.first.second.template is< EpsilonType >())
			symbol = "&epsilon";
		else
			symbol = alib::StringDataFactory::toString(transition.first.second.template get<SymbolType>());

		symbol += " |";

		//Pop part
		symbol += " &epsilon;";
		symbol += " ->";

		symbol += " &epsilon;";

		//Insert into map
		std::pair<int, int> key(states.find(transition.first.first)->second, states.find(transition.second)->second);
		std::map<std::pair<int, int>, std::string>::iterator mapit = transitions.find(key);

		if (mapit == transitions.end()) {
			transitions.insert(std::make_pair(key, symbol));
		} else {
			mapit->second += ",";

			size_t pos = mapit->second.find_last_of("\n");
			if(pos == std::string::npos) pos = 0;
			if(mapit->second.size() - pos > 100) mapit->second += "\n";
			else mapit->second += " ";

			mapit->second += symbol;
		}
	}

	//print the map
	for (std::pair<const std::pair<int, int>, std::string>& transition : transitions) {
		out << transition.first.first << " -> " << transition.first.second;
		replace(transition.second, "\n", "\\n");
		out << "[label=\"" << transition.second << "\"]\n";
	}
}

template < class SymbolType, class EpsilonType, class StateType >
void DotConverter::transitions(const automaton::RealTimeHeightDeterministicNPDA < SymbolType, EpsilonType, StateType > & pda, const std::map<StateType, int>& states, std::ostream& out) {
	std::map<std::pair<int, int>, std::string> transitions;

	for (const auto& transition : pda.getCallTransitions()) {
		std::string symbol;

		//input symbol
		if(transition.first.second.template is< EpsilonType >())
			symbol = "&epsilon";
		else
			symbol = alib::StringDataFactory::toString(transition.first.second.template get<SymbolType>());

		symbol += " |";

		//Pop part
		symbol += " &epsilon;";
		symbol += " ->";

		std::string symbol2;
		for(const auto& to : transition.second) {
			symbol2 = symbol;
			symbol2 += " " + alib::StringDataFactory::toString(to.second);

			//Insert into map
			std::pair<int, int> key(states.find(transition.first.first)->second, states.find(to.first)->second);
			std::map<std::pair<int, int>, std::string>::iterator mapit = transitions.find(key);

			if (mapit == transitions.end()) {
				transitions.insert(std::make_pair(key, symbol2));
			} else {
				mapit->second += ",";

				size_t pos = mapit->second.find_last_of("\n");
				if(pos == std::string::npos) pos = 0;
				if(mapit->second.size() - pos > 100) mapit->second += "\n";
				else mapit->second += " ";

				mapit->second += symbol2;
			}
		}
	}

	for (const auto& transition : pda.getReturnTransitions()) {
		std::string symbol;

		//input symbol
		if(std::get<1>(transition.first).template is< EpsilonType >())
			symbol = "&epsilon;";
		else
			symbol = alib::StringDataFactory::toString(std::get<1>(transition.first).template get<SymbolType>());

		symbol += " |";

		//Pop part
		symbol += " " + alib::StringDataFactory::toString(std::get<2>(transition.first));
		symbol += " ->";

		std::string symbol2;
		for(const auto& to : transition.second) {
			symbol2 = symbol;
			symbol2 += " &epsilon;";

			//Insert into map
			std::pair<int, int> key(states.find(std::get<0>(transition.first))->second, states.find(to)->second);
			std::map<std::pair<int, int>, std::string>::iterator mapit = transitions.find(key);

			if (mapit == transitions.end()) {
				transitions.insert(std::make_pair(key, symbol2));
			} else {
				mapit->second += ",";

				size_t pos = mapit->second.find_last_of("\n");
				if(pos == std::string::npos) pos = 0;
				if(mapit->second.size() - pos > 100) mapit->second += "\n";
				else mapit->second += " ";

				mapit->second += symbol2;
			}
		}
	}

	for (const auto& transition : pda.getLocalTransitions()) {
		std::string symbol;

		//input symbol
		if(transition.first.second.template is< EpsilonType >())
			symbol = "&epsilon;";
		else
			symbol = alib::StringDataFactory::toString(transition.first.second.template get<SymbolType>());

		symbol += " |";

		//Pop part
		symbol += " &epsilon;";
		symbol += " ->";

		std::string symbol2;
		for(const auto& to : transition.second) {
			symbol2 = symbol;
			symbol2 += " &epsilon;";

			//Insert into map
			std::pair<int, int> key(states.find(transition.first.first)->second, states.find(to)->second);
			std::map<std::pair<int, int>, std::string>::iterator mapit = transitions.find(key);

			if (mapit == transitions.end()) {
				transitions.insert(std::make_pair(key, symbol2));
			} else {
				mapit->second += ",";

				size_t pos = mapit->second.find_last_of("\n");
				if(pos == std::string::npos) pos = 0;
				if(mapit->second.size() - pos > 100) mapit->second += "\n";
				else mapit->second += " ";

				mapit->second += symbol2;
			}
		}
	}

	//print the map
	for (std::pair<const std::pair<int, int>, std::string>& transition : transitions) {
		out << transition.first.first << " -> " << transition.first.second;
		replace(transition.second, "\n", "\\n");
		out << "[label=\"" << transition.second << "\"]\n";
	}
}

template < class SymbolType, class EpsilonType, class StateType >
void DotConverter::transitions(const automaton::NPDA < SymbolType, EpsilonType, StateType > & pda, const std::map<StateType, int>& states, std::ostream& out) {
	std::map<std::pair<int, int>, std::string> transitions;

	for (const auto& transition : pda.getTransitions()) {
		std::string symbol;

		//input symbol
		if (std::get<1>(transition.first).template is< EpsilonType >()) {
			symbol = "&epsilon;";
		} else {
			symbol = alib::StringDataFactory::toString(std::get<1>(transition.first).template get<SymbolType>());
		}

		symbol += " |";

		//Pop part
		if (std::get<2>(transition.first).size() == 0) {
			symbol += " &epsilon;";
		} else {
			for (SymbolType symb : std::get<2>(transition.first)) {
				symbol += " " + alib::StringDataFactory::toString(symb);
			}

		}

		symbol += " ->";

		std::string symbol2;
		for(const auto& to : transition.second) {
			symbol2 = symbol;
			//Push part
			if (to.second.size() == 0) {
				symbol2 += " &epsilon;";
			} else {
				for (SymbolType symb : to.second) {
					symbol2 += " " + alib::StringDataFactory::toString(symb);
				}

			}

			//Insert into map
			std::pair<int, int> key(states.find(std::get<0>(transition.first))->second, states.find(to.first)->second);
			std::map<std::pair<int, int>, std::string>::iterator mapit = transitions.find(key);

			if (mapit == transitions.end()) {
				transitions.insert(std::make_pair(key, symbol2));
			} else {
				mapit->second += ",";

				size_t pos = mapit->second.find_last_of("\n");
				if(pos == std::string::npos) pos = 0;
				if(mapit->second.size() - pos > 100) mapit->second += "\n";
				else mapit->second += " ";

				mapit->second += symbol2;
			}
		}
	}

	//print the map
	for (std::pair<const std::pair<int, int>, std::string>& transition : transitions) {
		out << transition.first.first << " -> " << transition.first.second;
		replace(transition.second, "\n", "\\n");
		out << "[label=\"" << transition.second << "\"]\n";
	}
}

template < class SymbolType, class EpsilonType, class StateType >
void DotConverter::transitions(const automaton::SinglePopNPDA < SymbolType, EpsilonType, StateType > & pda, const std::map<StateType, int>& states, std::ostream& out) {
	std::map<std::pair<int, int>, std::string> transitions;

	for (const auto& transition : pda.getTransitions()) {
		std::string symbol;

		//input symbol
		if (std::get<1>(transition.first).template is< EpsilonType >()) {
			symbol = "&epsilon;";
		} else {
			symbol = alib::StringDataFactory::toString(std::get<1>(transition.first).template get<SymbolType>());
		}

		symbol += " |";

		//Pop part
		symbol += " " + alib::StringDataFactory::toString(std::get<2>(transition.first));

		symbol += " ->";

		std::string symbol2;
		for(const auto& to : transition.second) {
			symbol2 = symbol;
			//Push part
			if (to.second.size() == 0) {
				symbol2 += " &epsilon;";
			} else {
				for (SymbolType symb : to.second) {
					symbol2 += " " + alib::StringDataFactory::toString(symb);
				}

			}

			//Insert into map
			std::pair<int, int> key(states.find(std::get<0>(transition.first))->second, states.find(to.first)->second);
			std::map<std::pair<int, int>, std::string>::iterator mapit = transitions.find(key);

			if (mapit == transitions.end()) {
				transitions.insert(std::make_pair(key, symbol2));
			} else {
				mapit->second += ",";

				size_t pos = mapit->second.find_last_of("\n");
				if(pos == std::string::npos) pos = 0;
				if(mapit->second.size() - pos > 100) mapit->second += "\n";
				else mapit->second += " ";

				mapit->second += symbol2;
			}
		}
	}

	//print the map
	for (std::pair<const std::pair<int, int>, std::string>& transition : transitions) {
		out << transition.first.first << " -> " << transition.first.second;
		replace(transition.second, "\n", "\\n");
		out << "[label=\"" << transition.second << "\"]\n";
	}
}

template < class SymbolType, class StateType >
void DotConverter::transitions(const automaton::OneTapeDTM < SymbolType, StateType > & tm, const std::map < StateType, int > & states, std::ostream& out) {
	std::map<std::pair<int, int>, std::string> transitions;
	for (const auto& transition : tm.getTransitions()) {
		std::string symbol;

		//input symbol
		symbol = "(";
		symbol += alib::StringDataFactory::toString(transition.first.second);
		symbol += ", ";
		symbol += alib::StringDataFactory::toString(std::get<1>(transition.second));
		symbol += " ";
		switch(std::get<2>(transition.second)) {
		case automaton::Shift::LEFT:
			symbol += "&larr;";
			break;
		case automaton::Shift::RIGHT:
			symbol += "&rarr;";
			break;
		case automaton::Shift::NONE:
			symbol += "&times;";
			break;
		default:
			throw exception::CommonException("Unexpected shift direction");
		}

		//Insert into map
		std::pair<int, int> key(states.find(transition.first.first)->second, states.find(std::get<0>(transition.second))->second);
		std::map<std::pair<int, int>, std::string>::iterator mapit = transitions.find(key);

		if (mapit == transitions.end()) {
			transitions.insert(std::make_pair(key, symbol));
		} else {
			mapit->second += ",";

			size_t pos = mapit->second.find_last_of("\n");
			if(pos == std::string::npos) pos = 0;
			if(mapit->second.size() - pos > 100) mapit->second += "\n";
			else mapit->second += " ";

			mapit->second += symbol;
		}
	}

	//print the map
	for (std::pair<const std::pair<int, int>, std::string>& transition : transitions) {
		out << transition.first.first << " -> " << transition.first.second;
		replace(transition.second, "\n", "\\n");
		out << "[label=\"" << transition.second << "\"]\n";
	}
}

#endif /* DOT_CONVERTER_H_ */
