/*
 * GasTexConverter.h
 *
 *  Created on: Jan 1, 2014
 *      Author: Martin Zak
 */

#ifndef GAS_TEX_CONVERTER_H_
#define GAS_TEX_CONVERTER_H_

#include <ostream>
#include <map>
#include <utility>
#include <vector>
#include <set>
#include <typeinfo>

#include <core/multipleDispatch.hpp>
#include <exception/CommonException.h>

#include <automaton/Automaton.h>
#include <automaton/AutomatonFeatures.h>

#include <string/String.h>

#include <automaton/FSM/NFA.h>
#include <automaton/FSM/MultiInitialStateNFA.h>
#include <automaton/FSM/EpsilonNFA.h>
#include <automaton/FSM/DFA.h>
#include <automaton/FSM/ExtendedNFA.h>
#include <automaton/FSM/CompactNFA.h>
#include <automaton/TA/NFTA.h>
#include <automaton/TA/DFTA.h>
#include <automaton/PDA/DPDA.h>
#include <automaton/PDA/SinglePopDPDA.h>
#include <automaton/PDA/InputDrivenDPDA.h>
#include <automaton/PDA/InputDrivenNPDA.h>
#include <automaton/PDA/VisiblyPushdownDPDA.h>
#include <automaton/PDA/VisiblyPushdownNPDA.h>
#include <automaton/PDA/RealTimeHeightDeterministicDPDA.h>
#include <automaton/PDA/RealTimeHeightDeterministicNPDA.h>
#include <automaton/PDA/NPDA.h>
#include <automaton/PDA/SinglePopNPDA.h>
#include <automaton/TM/OneTapeDTM.h>

#include <factory/StringDataFactory.hpp>

#include "common/converterCommon.hpp"

class GasTexConverter : public std::SingleDispatchFirstStaticParam<GasTexConverter, void, std::ostream&, const automaton::AutomatonBase &> {
	static void printTransitionMap( const std::map<std::pair<std::string, std::string>, std::string> transitionMap, std::ostream& out);

	template < class SymbolType >
	static std::string getStackSymbols(const std::vector<SymbolType>& stackSymbols);

	template < class SymbolType, class StateType >
	static void transitions(const automaton::DFA < SymbolType, StateType > & fsm, std::ostream & out);

	template < class SymbolType, class StateType >
	static void transitions(const automaton::NFA < SymbolType, StateType > & fsm, std::ostream & out);

	template < class SymbolType, class StateType >
	static void transitions(const automaton::MultiInitialStateNFA < SymbolType, StateType > & fsm, std::ostream & out);

	template < class SymbolType, class EpsilonType, class StateType >
	static void transitions(const automaton::EpsilonNFA < SymbolType, EpsilonType, StateType > & fsm, std::ostream & out);

	template < class SymbolType, class StateType >
	static void transitions(const automaton::ExtendedNFA < SymbolType, StateType > & fsm, std::ostream & out);

	template < class SymbolType, class StateType >
	static void transitions(const automaton::CompactNFA < SymbolType, StateType > & fsm, std::ostream & out);

	template < class SymbolType, class RankType, class StateType >
	static void transitions(const automaton::NFTA < SymbolType, RankType, StateType > & fsm, std::ostream & out);

	template < class SymbolType, class RankType, class StateType >
	static void transitions(const automaton::DFTA < SymbolType, RankType, StateType > & fsm, std::ostream & out);

	template < class SymbolType, class EpsilonType, class StateType >
	static void transitions(const automaton::DPDA < SymbolType, EpsilonType, StateType > & pda, std::ostream & out);

	template < class SymbolType, class EpsilonType, class StateType >
	static void transitions(const automaton::NPDA < SymbolType, EpsilonType, StateType > & pda, std::ostream & out);

	template < class SymbolType, class EpsilonType, class StateType >
	static void transitions(const automaton::SinglePopDPDA < SymbolType, EpsilonType, StateType > & tm, std::ostream & out);

	template < class SymbolType, class EpsilonType, class StateType >
	static void transitions(const automaton::SinglePopNPDA < SymbolType, EpsilonType, StateType > & tm, std::ostream & out);

	template < class SymbolType, class StateType >
	static void transitions(const automaton::InputDrivenDPDA < SymbolType, StateType > & pda, std::ostream & out);

	template < class SymbolType, class StateType >
	static void transitions(const automaton::InputDrivenNPDA < SymbolType, StateType > & pda, std::ostream & out);

	template < class SymbolType, class StateType >
	static void transitions(const automaton::VisiblyPushdownDPDA < SymbolType, StateType > & tm, std::ostream & out);

	template < class SymbolType, class StateType >
	static void transitions(const automaton::VisiblyPushdownNPDA < SymbolType, StateType > & tm, std::ostream & out);

	template < class SymbolType, class EpsilonType, class StateType >
	static void transitions(const automaton::RealTimeHeightDeterministicDPDA < SymbolType, EpsilonType, StateType > & tm, std::ostream & out);

	template < class SymbolType, class EpsilonType, class StateType >
	static void transitions(const automaton::RealTimeHeightDeterministicNPDA < SymbolType, EpsilonType, StateType > & tm, std::ostream & out);

	template < class SymbolType, class StateType >
	static void transitions(const automaton::OneTapeDTM < SymbolType, StateType > & tm, std::ostream & out);
public:
	static void convert(std::ostream& out, const automaton::Automaton& a);

	template < class SymbolType, class StateType >
	static void convert(std::ostream& out, const automaton::DFA < SymbolType, StateType > & a);

	template < class SymbolType, class StateType >
	static void convert(std::ostream& out, const automaton::NFA < SymbolType, StateType > & a);

	template < class SymbolType, class StateType >
	static void convert(std::ostream& out, const automaton::MultiInitialStateNFA < SymbolType, StateType >& a);

	template < class SymbolType, class EpsilonType, class StateType >
	static void convert(std::ostream& out, const automaton::EpsilonNFA < SymbolType, EpsilonType, StateType > & a);

	template < class SymbolType, class StateType >
	static void convert(std::ostream& out, const automaton::ExtendedNFA < SymbolType, StateType > & a);

	template < class SymbolType, class StateType >
	static void convert(std::ostream& out, const automaton::CompactNFA < SymbolType, StateType > & a);

	template < class SymbolType, class RankType, class StateType >
	static void convert(std::ostream& out, const automaton::NFTA < SymbolType, RankType, StateType > & a);

	template < class SymbolType, class RankType, class StateType >
	static void convert(std::ostream& out, const automaton::DFTA < SymbolType, RankType, StateType > & a);

	template < class SymbolType, class EpsilonType, class StateType >
	static void convert(std::ostream& out, const automaton::DPDA < SymbolType, EpsilonType, StateType > & a);

	template < class SymbolType, class EpsilonType, class StateType >
	static void convert(std::ostream& out, const automaton::NPDA < SymbolType, EpsilonType, StateType > & a);

	template < class SymbolType, class EpsilonType, class StateType >
	static void convert(std::ostream& out, const automaton::SinglePopDPDA < SymbolType, EpsilonType, StateType > & a);

	template < class SymbolType, class EpsilonType, class StateType >
	static void convert(std::ostream& out, const automaton::SinglePopNPDA < SymbolType, EpsilonType, StateType > & a);

	template < class SymbolType, class StateType >
	static void convert(std::ostream& out, const automaton::InputDrivenDPDA < SymbolType, StateType > & a);

	template < class SymbolType, class StateType >
	static void convert(std::ostream& out, const automaton::InputDrivenNPDA < SymbolType, StateType > & a);

	template < class SymbolType, class StateType >
	static void convert(std::ostream& out, const automaton::VisiblyPushdownDPDA < SymbolType, StateType > & a);

	template < class SymbolType, class StateType >
	static void convert(std::ostream& out, const automaton::VisiblyPushdownNPDA < SymbolType, StateType > & a);

	template < class SymbolType, class EpsilonType, class StateType >
	static void convert(std::ostream& out, const automaton::RealTimeHeightDeterministicDPDA < SymbolType, EpsilonType, StateType > & a);

	template < class SymbolType, class EpsilonType, class StateType >
	static void convert(std::ostream& out, const automaton::RealTimeHeightDeterministicNPDA < SymbolType, EpsilonType, StateType > & a);

	template < class SymbolType, class StateType >
	static void convert(std::ostream& out, const automaton::OneTapeDTM < SymbolType, StateType > & a);
};

template < class SymbolType, class EpsilonType, class StateType >
void GasTexConverter::convert(std::ostream& out, const automaton::EpsilonNFA < SymbolType, EpsilonType, StateType > & a) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialState() == state){
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state;
		out << ")(,){";
		out << state;
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

template < class SymbolType, class StateType >
void GasTexConverter::convert(std::ostream& out, const automaton::MultiInitialStateNFA < SymbolType, StateType >& a) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialStates().count(state)){
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state;
		out << ")(,){";
		out << state;
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

template < class SymbolType, class StateType >
void GasTexConverter::convert(std::ostream& out, const automaton::NFA < SymbolType, StateType > & a) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialState() == state){
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state;
		out << ")(,){";
		out << state;
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

template < class SymbolType, class StateType >
void GasTexConverter::convert(std::ostream& out, const automaton::DFA < SymbolType, StateType > & a) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialState() == state){
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state;
		out << ")(,){";
		out << state;
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

template < class SymbolType, class StateType >
void GasTexConverter::convert(std::ostream& out, const automaton::ExtendedNFA < SymbolType, StateType > & a) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialState() == state){
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state;
		out << ")(,){";
		out << state;
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

template < class SymbolType, class StateType >
void GasTexConverter::convert(std::ostream& out, const automaton::CompactNFA < SymbolType, StateType > & a) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialState() == state){
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state;
		out << ")(,){";
		out << state;
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

template < class SymbolType, class RankType, class StateType >
void GasTexConverter::convert(std::ostream&, const automaton::NFTA < SymbolType, RankType, StateType > &) {
	//TODO
}

template < class SymbolType, class RankType, class StateType >
void GasTexConverter::convert(std::ostream&, const automaton::DFTA < SymbolType, RankType, StateType > &) {
	//TODO
}

template < class SymbolType, class EpsilonType, class StateType >
void GasTexConverter::convert(std::ostream& out, const automaton::DPDA < SymbolType, EpsilonType, StateType > & a) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialState() == state) {
			initial = true;
		}
		if(a.getFinalStates().count(state)) {
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state;
		out << ")(,){";
		out << state;
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

template < class SymbolType, class EpsilonType, class StateType >
void GasTexConverter::convert(std::ostream& out, const automaton::SinglePopDPDA < SymbolType, EpsilonType, StateType > & a) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialState() == state) {
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state;
		out << ")(,){";
		out << state;
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

template < class SymbolType, class StateType >
void GasTexConverter::convert(std::ostream& out, const automaton::InputDrivenDPDA < SymbolType, StateType > & a) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialState() == state){
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state;
		out << ")(,){";
		out << state;
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

template < class SymbolType, class StateType >
void GasTexConverter::convert(std::ostream& out, const automaton::InputDrivenNPDA < SymbolType, StateType > & a) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialState() == state){
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state;
		out << ")(,){";
		out << state;
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

template < class SymbolType, class StateType >
void GasTexConverter::convert(std::ostream& out, const automaton::VisiblyPushdownDPDA < SymbolType, StateType > & a) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialState() == state){
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state;
		out << ")(,){";
		out << state;
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

template < class SymbolType, class StateType >
void GasTexConverter::convert(std::ostream& out, const automaton::VisiblyPushdownNPDA < SymbolType, StateType > & a) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialStates().count(state)){
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state;
		out << ")(,){";
		out << state;
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

template < class SymbolType, class EpsilonType, class StateType >
void GasTexConverter::convert(std::ostream& out, const automaton::RealTimeHeightDeterministicDPDA < SymbolType, EpsilonType, StateType > & a) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialState() == state){
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state;
		out << ")(,){";
		out << state;
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

template < class SymbolType, class EpsilonType, class StateType >
void GasTexConverter::convert(std::ostream& out, const automaton::RealTimeHeightDeterministicNPDA < SymbolType, EpsilonType, StateType > & a) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialStates().count(state)){
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state;
		out << ")(,){";
		out << state;
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

template < class SymbolType, class EpsilonType, class StateType >
void GasTexConverter::convert(std::ostream& out, const automaton::NPDA < SymbolType, EpsilonType, StateType > & a) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialState() == state){
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state;
		out << ")(,){";
		out << state;
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

template < class SymbolType, class EpsilonType, class StateType >
void GasTexConverter::convert(std::ostream& out, const automaton::SinglePopNPDA < SymbolType, EpsilonType, StateType > & a) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialState() == state){
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state;
		out << ")(,){";
		out << state;
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

template < class SymbolType, class StateType >
void GasTexConverter::convert(std::ostream& out, const automaton::OneTapeDTM < SymbolType, StateType > & a) {
	out << "\\begin{center}\n";
	out << "\\begin{picture}(,)(,)\n";

	for (auto& state : a.getStates()) {
		bool initial = false;
		bool final = false;

		if(a.getInitialState() == state){
			initial = true;
		}
		if(a.getFinalStates().count(state)){
			final = true;
		}

		if(initial || final) {
			out << "\\node[Nmarks=";
			if(initial){
				out << "i";
			}
			if(final){
				out << "r";
			}
			out<<"](";
		} else {
			out <<"\\node(";
		}

		out << state;
		out << ")(,){";
		out << state;
		out << "}\n";
	}

	transitions(a, out);
	out << "\\end{center}\n";
	out << "\\end{picture}\n";
}

template< class SymbolType >
std::string GasTexConverter::getStackSymbols(const std::vector<SymbolType>& stackSymbols) {
	if (stackSymbols.size() == 0) {
		return "$\\varepsilon$";
	}

	std::string symbols = "";
	int i=0;
	for (const SymbolType & symbol : stackSymbols) {
		if(i++ !=0) {
			symbols +=" ";
		}
		symbols += std::to_string ( symbol.getData() );
	}
	return symbols;
}

template<class SymbolType, class EpsilonType, class StateType>
void GasTexConverter::transitions(const automaton::EpsilonNFA < SymbolType, EpsilonType, StateType > & fsm, std::ostream& out) {
	std::map<std::pair<std::string, std::string>, std::string> transitionMap;
	for (const auto& transition : fsm.getTransitions()) {
		for(const auto& to : transition.second) {
			std::pair<std::string, std::string> key(std::to_string ( transition.first.first ), std::to_string ( to ) );

			std::string symbol;
			if (transition.first.second.template is<EpsilonType>()) {
				symbol = "$\\varepsilon$";
			} else {
				symbol = std::to_string ( transition.first.second.template get<SymbolType>() );
			}

			auto mapIterator = transitionMap.find(key);
			if (mapIterator == transitionMap.end()) {
				transitionMap.insert(make_pair(key, symbol));
			} else {
				mapIterator->second += ", " + symbol;
			}
		}
	}
	printTransitionMap(transitionMap, out);
}

template<class SymbolType, class StateType>
void GasTexConverter::transitions(const automaton::MultiInitialStateNFA < SymbolType, StateType >& fsm, std::ostream& out) {
	std::map<std::pair<std::string, std::string>, std::string> transitionMap;
	for (const auto& transition : fsm.getTransitions()) {
		for(const auto& to : transition.second) {
			std::pair<std::string, std::string> key(std::to_string ( transition.first.first ), std::to_string ( to ) );

			std::string symbol = std::to_string ( transition.first.second );

			auto mapIterator = transitionMap.find(key);
			if (mapIterator == transitionMap.end()) {
				transitionMap.insert(make_pair(key, symbol));
			} else {
				mapIterator->second += ", " + symbol;
			}
		}
	}
	printTransitionMap(transitionMap, out);
}

template<class SymbolType, class StateType>
void GasTexConverter::transitions(const automaton::NFA < SymbolType, StateType > & fsm, std::ostream& out) {
	std::map<std::pair<std::string, std::string>, std::string> transitionMap;
	for (const auto& transition : fsm.getTransitions()) {
		for(const auto& to : transition.second) {
			std::pair<std::string, std::string> key( std::to_string ( transition.first.first ), std::to_string ( to ) );

			std::string symbol = std::to_string ( transition.first.second );

			auto mapIterator = transitionMap.find(key);
			if (mapIterator == transitionMap.end()) {
				transitionMap.insert(make_pair(key, symbol));
			} else {
				mapIterator->second += ", " + symbol;
			}
		}
	}
	printTransitionMap(transitionMap, out);
}

template<class SymbolType, class StateType>
void GasTexConverter::transitions(const automaton::DFA < SymbolType, StateType > & fsm, std::ostream& out) {
	std::map<std::pair<std::string, std::string>, std::string> transitionMap;
	for (const auto& transition : fsm.getTransitions()) {
		std::pair<std::string, std::string> key( std::to_string ( transition.first.first ), std::to_string ( transition.second ) );

		std::string symbol = std::to_string ( transition.first.second );

		auto mapIterator = transitionMap.find(key);
		if (mapIterator == transitionMap.end()) {
			transitionMap.insert(make_pair(key, symbol));
		} else {
			mapIterator->second += ", " + symbol;
		}
	}
	printTransitionMap(transitionMap, out);
}

template<class SymbolType, class StateType>
void GasTexConverter::transitions(const automaton::ExtendedNFA < SymbolType, StateType > & fsm, std::ostream& out) {
	std::map<std::pair<std::string, std::string>, std::string> transitionMap;

	for (const auto& transition : fsm.getTransitions()) {
		for(const auto& to : transition.second) {
			std::pair<std::string, std::string> key( std::to_string ( transition.first.first ), std::to_string ( to ) );

			std::string symbol = alib::StringDataFactory::toString(transition.first.second);

			auto mapIterator = transitionMap.find(key);
			if (mapIterator == transitionMap.end()) {
				transitionMap.insert(make_pair(key, symbol));
			} else {
				mapIterator->second += ", " + symbol;
			}
		}
	}
	printTransitionMap(transitionMap, out);
}

template<class SymbolType, class StateType>
void GasTexConverter::transitions(const automaton::CompactNFA < SymbolType, StateType > & fsm, std::ostream& out) {
	std::map<std::pair<std::string, std::string>, std::string> transitionMap;

	for (const auto& transition : fsm.getTransitions()) {
		for(const auto& to : transition.second) {
			std::pair<std::string, std::string> key(std::to_string ( transition.first.first ), std::to_string ( to ) );

			std::string symbol = alib::StringDataFactory::toString(string::stringFrom(transition.first.second));

			auto mapIterator = transitionMap.find(key);
			if (mapIterator == transitionMap.end()) {
				transitionMap.insert(make_pair(key, symbol));
			} else {
				mapIterator->second += ", " + symbol;
			}
		}
	}
	printTransitionMap(transitionMap, out);
}

template<class SymbolType, class RankType, class StateType>
void GasTexConverter::transitions(const automaton::NFTA < SymbolType, RankType, StateType > &, std::ostream&) {
	//TODO
}

template<class SymbolType, class RankType, class StateType>
void GasTexConverter::transitions(const automaton::DFTA < SymbolType, RankType, StateType > &, std::ostream&) {
	//TODO
}

template<class SymbolType, class EpsilonType, class StateType>
void GasTexConverter::transitions(const automaton::DPDA < SymbolType, EpsilonType, StateType > & pda, std::ostream& out) {
	std::map<std::pair<std::string, std::string>, std::string> transitionMap;

	for (const auto& transition : pda.getTransitions()) {
		std::pair<std::string, std::string> key(std::to_string ( std::get<0>(transition.first) ), std::to_string ( transition.second.first ) );
		auto mapIterator = transitionMap.find(key);

		std::string symbol;
		if (std::get<1>(transition.first).template is<EpsilonType>()) {
			symbol = "$\\varepsilon;$";
		} else {
			symbol = alib::StringDataFactory::toString(std::get<1>(transition.first).template get<SymbolType>());
		}

		symbol += "|";

		symbol += getStackSymbols(std::get<2>(transition.first));
		symbol += "\\rarrow";
		symbol += getStackSymbols(transition.second.second);

		if (mapIterator == transitionMap.end()) {
			transitionMap.insert(std::make_pair(key, symbol));
		} else {
			mapIterator->second += "; " + symbol;
		}
	}

	printTransitionMap(transitionMap, out);
}

template<class SymbolType, class EpsilonType, class StateType>
void GasTexConverter::transitions(const automaton::SinglePopDPDA < SymbolType, EpsilonType, StateType > & pda, std::ostream& out) {
	std::map<std::pair<std::string, std::string>, std::string> transitionMap;

	for (const auto& transition : pda.getTransitions()) {
		std::pair<std::string, std::string> key(std::to_string ( std::get<0>(transition.first) ), std::to_string ( transition.second.first ) );
		auto mapIterator = transitionMap.find(key);

		std::string symbol;
		if (std::get<1>(transition.first).template is<EpsilonType>()) {
			symbol = "$\\varepsilon;$";
		} else {
			symbol = alib::StringDataFactory::toString(std::get<1>(transition.first).template get<SymbolType>());
		}

		symbol += "|";

		symbol += alib::StringDataFactory::toString(std::get<2>(transition.first));
		symbol += "\\rarrow";
		symbol += getStackSymbols(transition.second.second);

		if (mapIterator == transitionMap.end()) {
			transitionMap.insert(std::make_pair(key, symbol));
		} else {
			mapIterator->second += "; " + symbol;
		}
	}

	printTransitionMap(transitionMap, out);
}

template<class SymbolType, class StateType>
void GasTexConverter::transitions(const automaton::InputDrivenDPDA < SymbolType, StateType > & pda, std::ostream& out) {
	std::map<std::pair<std::string, std::string>, std::string> transitionMap;

	const auto& symbolToPDSOperation = pda.getPushdownStoreOperations();
	for (const auto& transition : pda.getTransitions()) {
		const auto& pop = symbolToPDSOperation.find(std::get<1>(transition.first))->second.first;
		const auto& push = symbolToPDSOperation.find(std::get<1>(transition.first))->second.second;

		const auto& to = transition.second;
		std::pair<std::string, std::string> key(std::to_string ( transition.first.first ), std::to_string ( to ) );
		auto mapIterator = transitionMap.find(key);

		std::string symbol = alib::StringDataFactory::toString(transition.first.second);

		symbol += "|";

		symbol += getStackSymbols(pop);
		symbol += "\\rarrow";
		symbol += getStackSymbols(push);

		if (mapIterator == transitionMap.end()) {
			transitionMap.insert(std::make_pair(key, symbol));
		} else {
			mapIterator->second += "; " + symbol;
		}
	}

	printTransitionMap(transitionMap, out);
}

template<class SymbolType, class StateType>
void GasTexConverter::transitions(const automaton::InputDrivenNPDA < SymbolType, StateType > & pda, std::ostream& out) {
	std::map<std::pair<std::string, std::string>, std::string> transitionMap;

	const auto& symbolToPDSOperation = pda.getPushdownStoreOperations();
	for (const auto& transition : pda.getTransitions()) {
		const auto& pop = symbolToPDSOperation.find(std::get<1>(transition.first))->second.first;
		const auto& push = symbolToPDSOperation.find(std::get<1>(transition.first))->second.second;

		for(const auto& to : transition.second) {
			std::pair<std::string, std::string> key( std::to_string ( transition.first.first ), std::to_string ( to ) );
			auto mapIterator = transitionMap.find(key);

			std::string symbol = alib::StringDataFactory::toString(transition.first.second);

			symbol += "|";

			symbol += getStackSymbols(pop);
			symbol += "\\rarrow";
			symbol += getStackSymbols(push);

			if (mapIterator == transitionMap.end()) {
				transitionMap.insert(std::make_pair(key, symbol));
			} else {
				mapIterator->second += "; " + symbol;
			}
		}
	}

	printTransitionMap(transitionMap, out);
}

template<class SymbolType, class StateType>
void GasTexConverter::transitions(const automaton::VisiblyPushdownDPDA < SymbolType, StateType > & pda, std::ostream& out) {
	std::map<std::pair<std::string, std::string>, std::string> transitionMap;

	for (const auto& transition : pda.getCallTransitions()) {
		std::pair<std::string, std::string> key( std::to_string ( transition.first.first ), std::to_string ( transition.second.first ) );
		auto mapIterator = transitionMap.find(key);

		std::string symbol = alib::StringDataFactory::toString(transition.first.second);

		symbol += "|";

		symbol += "$\\varepsilon;$";
		symbol += "\\rarrow";
		symbol += alib::StringDataFactory::toString(transition.second.second);

		if (mapIterator == transitionMap.end()) {
			transitionMap.insert(std::make_pair(key, symbol));
		} else {
			mapIterator->second += "; " + symbol;
		}
	}

	for (const auto& transition : pda.getReturnTransitions()) {
		std::pair<std::string, std::string> key( std::to_string ( std::get<0>(transition.first) ), std::to_string ( transition.second ) );
		auto mapIterator = transitionMap.find(key);

		std::string symbol = alib::StringDataFactory::toString(std::get<1>(transition.first));

		symbol += "|";

		symbol += alib::StringDataFactory::toString(std::get<2>(transition.first));
		symbol += "\\rarrow";
		symbol += "$\\varepsilon;$";

		if (mapIterator == transitionMap.end()) {
			transitionMap.insert(std::make_pair(key, symbol));
		} else {
			mapIterator->second += "; " + symbol;
		}
	}

	for (const auto& transition : pda.getLocalTransitions()) {
		std::pair<std::string, std::string> key( std::to_string ( transition.first.first ), std::to_string ( transition.second ) );
		auto mapIterator = transitionMap.find(key);

		std::string symbol = alib::StringDataFactory::toString(transition.first.second);

		symbol += "|";

		symbol += "$\\varepsilon;$";
		symbol += "\\rarrow";
		symbol += "$\\varepsilon;$";

		if (mapIterator == transitionMap.end()) {
			transitionMap.insert(std::make_pair(key, symbol));
		} else {
			mapIterator->second += "; " + symbol;
		}
	}

	printTransitionMap(transitionMap, out);
}

template<class SymbolType, class StateType>
void GasTexConverter::transitions(const automaton::VisiblyPushdownNPDA < SymbolType, StateType > & pda, std::ostream& out) {
	std::map<std::pair<std::string, std::string>, std::string> transitionMap;

	for (const auto& transition : pda.getCallTransitions()) {
		for(const auto& to : transition.second) {
			std::pair<std::string, std::string> key( std::to_string ( transition.first.first ), std::to_string ( to.first ) );
			auto mapIterator = transitionMap.find(key);

			std::string symbol = alib::StringDataFactory::toString(transition.first.second);

			symbol += "|";

			symbol += "$\\varepsilon;$";
			symbol += "\\rarrow";
			symbol += alib::StringDataFactory::toString(to.second);

			if (mapIterator == transitionMap.end()) {
				transitionMap.insert(std::make_pair(key, symbol));
			} else {
				mapIterator->second += "; " + symbol;
			}
		}
	}

	for (const auto& transition : pda.getReturnTransitions()) {
		for(const auto& to : transition.second) {
			std::pair<std::string, std::string> key( std::to_string ( std::get<0>(transition.first) ), std::to_string ( to ) );
			auto mapIterator = transitionMap.find(key);

			std::string symbol = alib::StringDataFactory::toString(std::get<1>(transition.first));

			symbol += "|";

			symbol += alib::StringDataFactory::toString(std::get<2>(transition.first));
			symbol += "\\rarrow";
			symbol += "$\\varepsilon;$";

			if (mapIterator == transitionMap.end()) {
				transitionMap.insert(std::make_pair(key, symbol));
			} else {
				mapIterator->second += "; " + symbol;
			}
		}
	}

	for (const auto& transition : pda.getLocalTransitions()) {
		for(const auto& to : transition.second) {
			std::pair<std::string, std::string> key( std::to_string ( transition.first.first ), std::to_string ( to ) );
			auto mapIterator = transitionMap.find(key);

			std::string symbol = alib::StringDataFactory::toString(transition.first.second);

			symbol += "|";

			symbol += "$\\varepsilon;$";
			symbol += "\\rarrow";
			symbol += "$\\varepsilon;$";

			if (mapIterator == transitionMap.end()) {
				transitionMap.insert(std::make_pair(key, symbol));
			} else {
				mapIterator->second += "; " + symbol;
			}
		}
	}

	printTransitionMap(transitionMap, out);
}

template<class SymbolType, class EpsilonType, class StateType>
void GasTexConverter::transitions(const automaton::RealTimeHeightDeterministicDPDA < SymbolType, EpsilonType, StateType > & pda, std::ostream& out) {
	std::map<std::pair<std::string, std::string>, std::string> transitionMap;

	for (const auto& transition : pda.getCallTransitions()) {
		std::pair<std::string, std::string> key( std::to_string ( transition.first.first ), std::to_string ( transition.second.first ) );
		auto mapIterator = transitionMap.find(key);

		std::string symbol;
		if(transition.first.second.template is<EpsilonType>())
			symbol = "$\\varepsilon;$";
		else
			symbol = alib::StringDataFactory::toString(transition.first.second.template get<SymbolType>());

		symbol += "|";

		symbol += "$\\varepsilon;$";
		symbol += "\\rarrow";
		symbol += alib::StringDataFactory::toString(transition.second.second);

		if (mapIterator == transitionMap.end()) {
			transitionMap.insert(std::make_pair(key, symbol));
		} else {
			mapIterator->second += "; " + symbol;
		}
	}

	for (const auto& transition : pda.getReturnTransitions()) {
		std::pair<std::string, std::string> key( std::to_string ( std::get<0>(transition.first) ), std::to_string ( transition.second ) );
		auto mapIterator = transitionMap.find(key);

		std::string symbol;
		if(std::get<1>(transition.first).template is<EpsilonType>())
			symbol = "$\\varepsilon;$";
		else
			symbol = alib::StringDataFactory::toString(std::get<1>(transition.first).template get<SymbolType>());

		symbol += "|";

		symbol += alib::StringDataFactory::toString(std::get<2>(transition.first));
		symbol += "\\rarrow";
		symbol += "$\\varepsilon;$";

		if (mapIterator == transitionMap.end()) {
			transitionMap.insert(std::make_pair(key, symbol));
		} else {
			mapIterator->second += "; " + symbol;
		}
	}

	for (const auto& transition : pda.getLocalTransitions()) {
		std::pair<std::string, std::string> key( std::to_string ( transition.first.first ), std::to_string ( transition.second ) );
		auto mapIterator = transitionMap.find(key);

		std::string symbol;
		if(transition.first.second.template is<EpsilonType>())
			symbol = "$\\varepsilon;$";
		else
			symbol = alib::StringDataFactory::toString(transition.first.second.template get<SymbolType>());

		symbol += "|";

		symbol += "$\\varepsilon;$";
		symbol += "\\rarrow";
		symbol += "$\\varepsilon;$";

		if (mapIterator == transitionMap.end()) {
			transitionMap.insert(std::make_pair(key, symbol));
		} else {
			mapIterator->second += "; " + symbol;
		}
	}

	printTransitionMap(transitionMap, out);
}

template<class SymbolType, class EpsilonType, class StateType>
void GasTexConverter::transitions(const automaton::RealTimeHeightDeterministicNPDA < SymbolType, EpsilonType, StateType > & pda, std::ostream& out) {
	std::map<std::pair<std::string, std::string>, std::string> transitionMap;

	for (const auto& transition : pda.getCallTransitions()) {
		for(const auto& to : transition.second) {
			std::pair<std::string, std::string> key( std::to_string ( transition.first.first ), std::to_string ( to.first ) );
			auto mapIterator = transitionMap.find(key);

			std::string symbol;
			if(transition.first.second.template is<EpsilonType>())
				symbol = "$\\varepsilon;$";
			else
				symbol = alib::StringDataFactory::toString(transition.first.second.template get<SymbolType>());

			symbol += "|";

			symbol += "$\\varepsilon;$";
			symbol += "\\rarrow";
			symbol += alib::StringDataFactory::toString(to.second);

			if (mapIterator == transitionMap.end()) {
				transitionMap.insert(std::make_pair(key, symbol));
			} else {
				mapIterator->second += "; " + symbol;
			}
		}
	}

	for (const auto& transition : pda.getReturnTransitions()) {
		for(const auto& to : transition.second) {
			std::pair<std::string, std::string> key( std::to_string ( std::get<0>(transition.first) ), std::to_string ( to ) );
			auto mapIterator = transitionMap.find(key);

			std::string symbol;
			if(std::get<1>(transition.first).template is<EpsilonType>())
				symbol = "$\\varepsilon;$";
			else
				symbol = alib::StringDataFactory::toString(std::get<1>(transition.first).template get<SymbolType>());

			symbol += "|";

			symbol += alib::StringDataFactory::toString(std::get<2>(transition.first));
			symbol += "\\rarrow";
			symbol += "$\\varepsilon;$";

			if (mapIterator == transitionMap.end()) {
				transitionMap.insert(std::make_pair(key, symbol));
			} else {
				mapIterator->second += "; " + symbol;
			}
		}
	}

	for (const auto& transition : pda.getLocalTransitions()) {
		for(const auto& to : transition.second) {
			std::pair<std::string, std::string> key( std::to_string ( transition.first.first ), std::to_string ( to ) );
			auto mapIterator = transitionMap.find(key);

			std::string symbol;
			if(transition.first.second.template is<EpsilonType>())
				symbol = "$\\varepsilon;$";
			else
				symbol = alib::StringDataFactory::toString(transition.first.second.template get<SymbolType>());

			symbol += "|";

			symbol += "$\\varepsilon;$";
			symbol += "\\rarrow";
			symbol += "$\\varepsilon;$";

			if (mapIterator == transitionMap.end()) {
				transitionMap.insert(std::make_pair(key, symbol));
			} else {
				mapIterator->second += "; " + symbol;
			}
		}
	}

	printTransitionMap(transitionMap, out);
}

template<class SymbolType, class EpsilonType, class StateType>
void GasTexConverter::transitions(const automaton::NPDA < SymbolType, EpsilonType, StateType > & pda, std::ostream& out) {
	std::map<std::pair<std::string, std::string>, std::string> transitionMap;

	for (const auto& transition : pda.getTransitions()) {
		for(const auto& to : transition.second) {
			std::pair<std::string, std::string> key( std::to_string ( std::get<0>(transition.first) ), std::to_string ( to.first ) );
			auto mapIterator = transitionMap.find(key);

			std::string symbol;
			if (std::get<1>(transition.first).template is<EpsilonType>()) {
				symbol = "$\\varepsilon;$";
			} else {
				symbol = alib::StringDataFactory::toString(std::get<1>(transition.first).template get<SymbolType>());
			}

			symbol += "|";

			symbol += getStackSymbols(std::get<2>(transition.first));
			symbol += "\\rarrow";
			symbol += getStackSymbols(to.second);

			if (mapIterator == transitionMap.end()) {
				transitionMap.insert(std::make_pair(key, symbol));
			} else {
				mapIterator->second += "; " + symbol;
			}
		}
	}

	printTransitionMap(transitionMap, out);
}

template<class SymbolType, class EpsilonType, class StateType>
void GasTexConverter::transitions(const automaton::SinglePopNPDA < SymbolType, EpsilonType, StateType > & pda, std::ostream& out) {
	std::map<std::pair<std::string, std::string>, std::string> transitionMap;

	for (const auto& transition : pda.getTransitions()) {
		for(const auto& to : transition.second) {
			std::pair<std::string, std::string> key( std::to_string ( std::get<0>(transition.first) ), std::to_string ( to.first ) );
			auto mapIterator = transitionMap.find(key);

			std::string symbol;
			if (std::get<1>(transition.first).template is<EpsilonType>()) {
				symbol = "$\\varepsilon;$";
			} else {
				symbol = alib::StringDataFactory::toString(std::get<1>(transition.first).template get<SymbolType>());
			}

			symbol += "|";

			symbol += alib::StringDataFactory::toString(std::get<2>(transition.first));
			symbol += "\\rarrow";
			symbol += getStackSymbols(to.second);

			if (mapIterator == transitionMap.end()) {
				transitionMap.insert(std::make_pair(key, symbol));
			} else {
				mapIterator->second += "; " + symbol;
			}
		}
	}

	printTransitionMap(transitionMap, out);
}

template<class SymbolType, class StateType>
void GasTexConverter::transitions(const automaton::OneTapeDTM < SymbolType, StateType > & tm, std::ostream& out) {
	std::map<std::pair<std::string, std::string>, std::string> transitionMap;

	for (auto& transition : tm.getTransitions()) {
		std::pair<std::string, std::string> key( std::to_string ( transition.first.first ), std::to_string ( std::get<0>(transition.second)));
		auto mapIterator = transitionMap.find(key);

		std::string symbol = alib::StringDataFactory::toString(transition.first.second);
		symbol += "/";
		symbol += alib::StringDataFactory::toString(std::get<1>(transition.second));
		symbol += ",";
		switch(std::get<2>(transition.second)) {
		case automaton::Shift::LEFT:
			symbol += "$\\leftarrow$";
			break;
		case automaton::Shift::RIGHT:
			symbol += "$\\rightarrow$";
			break;
		case automaton::Shift::NONE:
			symbol += "$\\times$";
			break;
		default:
			throw exception::CommonException("Unexpected shift direction");
		}

		if (mapIterator == transitionMap.end()) {
			transitionMap.insert(std::make_pair(key, symbol));
		} else {
			mapIterator->second += "; " + symbol;
		}
	}

	printTransitionMap(transitionMap, out);
}

#endif /* GAS_TEX_CONVERTER_H_ */
