/*
 * aecho.cpp
 *
 *  Created on: 24. 2. 2014
 *      Author: Martin Zak
 */

#include <tclap/CmdLine.h>
#include <global/GlobalData.h>
#include <measure>
#include <string>
#include <sax/FromXMLParserHelper.h>
#include <exception/CommonException.h>
#include <factory/XmlDataFactory.hpp>
#include <object/Object.h>

int main(int argc, char** argv) {
	try {
		common::GlobalData::argc = argc;
		common::GlobalData::argv = argv;

		TCLAP::CmdLine cmd("echo binary", ' ', "0.01");

		TCLAP::ValueArg<std::string> input(	"i",	"input",	"Input to echo",		false,	"-",		"file");
		cmd.add( input );

		TCLAP::SwitchArg measure(		"m",	"measure",	"Measure times",		false);
		cmd.add( measure );

		TCLAP::SwitchArg verbose(		"v",	"verbose",	"Be verbose",			false);
		cmd.add( verbose );

		TCLAP::SwitchArg no_optimize(		"n",	"no_optimize",	"Do not optimize xml",		false);
		cmd.add( no_optimize );

		cmd.parse(argc, argv);

		if(verbose.isSet())
			common::GlobalData::verbose = true;
		if(measure.isSet())
			common::GlobalData::measure = true;
		if(no_optimize.isSet())
			common::GlobalData::optimizeXml = false;

		measurements::start("Overal", measurements::Type::OVERALL);
		measurements::start("Input read", measurements::Type::AUXILIARY);

		alib::Object object = alib::XmlDataFactory::fromTokens<alib::Object>(sax::FromXMLParserHelper::parseInput(input));

		measurements::end();
		measurements::start("Output write", measurements::Type::AUXILIARY);

		alib::XmlDataFactory::toStdout(object);

		measurements::end();
		measurements::end();

		if(measure.getValue()) std::cmeasure << measurements::results() << std::endl;

		return 0;
	} catch(const exception::CommonException& exception) {
		alib::XmlDataFactory::toStdout(exception);
		return 1;
	} catch(const TCLAP::ArgException& exception) {
		std::cerr << exception.error() << std::endl;
		return 2;
	} catch (const std::exception& exception) {
		std::cerr << "Exception caught: " << exception.what() << std::endl;
		return 3;
	} catch(...) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
