/*
 * ToGrammar.h
 *
 *  Created on: 9. 2. 2014
 *      Author: Jan Travnicek
 */

#ifndef _AUTOMATON_TO_GRAMMAR_H__
#define _AUTOMATON_TO_GRAMMAR_H__

#include <core/multipleDispatch.hpp>

#include <grammar/Regular/LeftRG.h>
#include <automaton/FSM/NFA.h>
#include <automaton/FSM/DFA.h>

#include <grammar/Grammar.h>
#include <automaton/Automaton.h>

#include "ToGrammarRightRG.h"

namespace automaton {

namespace convert {

class ToGrammar : public std::SingleDispatch<ToGrammar, grammar::Grammar, const automaton::AutomatonBase &> {
public:
	/**
	 * Performs conversion.
	 * @return left regular grammar equivalent to source automaton.
	 */
	static grammar::Grammar convert(const automaton::Automaton& automaton);

	template < class SymbolType, class StateType >
	static grammar::Grammar convert(const automaton::NFA < SymbolType, StateType > & automaton);
	template < class SymbolType, class StateType >
	static grammar::Grammar convert(const automaton::DFA < SymbolType, StateType > & automaton);
};

template < class SymbolType, class StateType >
grammar::Grammar ToGrammar::convert(const automaton::NFA < SymbolType, StateType > & automaton) {
	return grammar::Grammar(ToGrammarRightRG::convert(automaton));
}

template < class SymbolType, class StateType >
grammar::Grammar ToGrammar::convert(const automaton::DFA < SymbolType, StateType > & automaton) {
	return grammar::Grammar(ToGrammarRightRG::convert(automaton));
}


} /* namespace convert */

} /* namespace automaton */

#endif /* _AUTOMATON_TO_GRAMMAR_H__ */
