/*
 * Translate.h
 *
 *  Created on: 12. 5. 2016
 *      Author: Jakub Doupal
 */

#ifndef _AUTOMATON_TRANSLATE_H__
#define _AUTOMATON_TRANSLATE_H__

#include <core/multipleDispatch.hpp>
#include <automaton/Automaton.h>
#include <automaton/AutomatonFeatures.h>
#include <object/Object.h>
#include <string/LinearString.h>
#include <alphabet/Symbol.h>

#include "Run.h"
#include <automaton/PDA/NPDTA.h>

#include <deque>
#include <set>

namespace automaton {

namespace run {

class Translate : public std::DoubleDispatch < Translate, std::set < string::LinearString < > >, const automaton::AutomatonBase &, const alib::ObjectBase & > {
public:
	/**
	 * Performs conversion.
	 * @return left regular grammar equivalent to source automaton.
	 */
	static std::set < string::LinearString < > > translate ( const automaton::Automaton & automaton, const alib::Object & object );
	static std::set < string::LinearString < > > translate ( const automaton::Automaton & automaton, const string::LinearString < > & string );

	template < class SymbolType, class EpsilonType, class StateType >
	static std::set < string::LinearString < SymbolType > > translate ( const automaton::NPDTA < SymbolType, EpsilonType, StateType > & automaton, const string::LinearString < SymbolType > & string );

};

template < class SymbolType, class EpsilonType, class StateType >
std::set < string::LinearString < SymbolType > > Translate::translate ( const automaton::NPDTA < SymbolType, EpsilonType, StateType > & automaton, const string::LinearString < SymbolType > & string ) {
	std::tuple < bool, std::set < StateType >, std::set < std::vector < SymbolType > > > res = Run::calculateStates ( automaton, string );
	std::set < string::LinearString < SymbolType > > strings;
	for ( auto iter = std::get<2>(res).begin(); iter != std::get<2>(res).end(); iter ++ ) {
		string::LinearString < SymbolType > str (* iter);
		strings . insert ( str );
	}
	return strings;
}

} /* namespace run */

} /* namespace automaton */

#endif /* _AUTOMATON_TRANSLATE_H__ */
