/*
 * AutomataIntersectionCartesianProduct.cpp
 *
 *  Created on: 20. 11. 2014
 *	  Author: Tomas Pecka
 */

#include "AutomataIntersectionCartesianProduct.h"
#include <common/DefaultStatesPairType.h>

namespace automaton {

namespace transform {

automaton::Automaton AutomataIntersectionCartesianProduct::intersection(const automaton::Automaton& first, const automaton::Automaton& second) {
	return dispatch(first.getData(), second.getData());
}

automaton::DFA<> AutomataIntersectionCartesianProduct::intersection(const automaton::DFA<>& first, const automaton::DFA<>& second) {
	DefaultStateType q0(first.getInitialState(), second.getInitialState());
	automaton::DFA<> res(q0);

	for(const auto& a : first.getInputAlphabet())
		res.addInputSymbol(a);
	for(const auto& a : second.getInputAlphabet())
		res.addInputSymbol(a);

	for(const auto& p : first.getStates())
		for(const auto& q : second.getStates())
			res.addState(DefaultStateType(p, q));

	for(const auto& p : first.getFinalStates())
		for(const auto& q : second.getFinalStates())
			res.addFinalState(DefaultStateType(p, q));

	for(const auto& state : res.getStates()) {
		const DefaultStateType& label_p = static_cast<const DefaultStatesPairType&>(state.getData()).first;
		const DefaultStateType& label_q = static_cast<const DefaultStatesPairType&>(state.getData()).second;

		for(const auto& tp : first.getTransitionsFromState(DefaultStateType(label_p)))
			for(const auto& tq : second.getTransitionsFromState(DefaultStateType(label_q)))
				if(tp.first.second == tq.first.second)
					res.addTransition(state, tp.first.second, DefaultStateType(tp.second, tq.second));
	}

	return res;
}

auto AutomataIntersectionCartesianProductDFA = AutomataIntersectionCartesianProduct::RegistratorWrapper<automaton::DFA<>, automaton::DFA<>>(AutomataIntersectionCartesianProduct::intersection);

automaton::NFA < > AutomataIntersectionCartesianProduct::intersection(const automaton::NFA < > & first, const automaton::NFA < > & second) {
	DefaultStateType q0(first.getInitialState(), second.getInitialState());
	automaton::NFA < > res(q0);

	for(const auto& a : first.getInputAlphabet())
		res.addInputSymbol(a);
	for(const auto& a : second.getInputAlphabet())
		res.addInputSymbol(a);

	for(const auto& p : first.getStates())
		for(const auto& q : second.getStates())
			res.addState(DefaultStateType(p, q));

	for(const auto& p : first.getFinalStates())
		for(const auto& q : second.getFinalStates())
			res.addFinalState(DefaultStateType(p, q));

	for(const auto& state : res.getStates()) {
		const DefaultStateType& label_p = static_cast<const DefaultStatesPairType&>(state.getData()).first;
		const DefaultStateType& label_q = static_cast<const DefaultStatesPairType&>(state.getData()).second;

		for(const auto& tp : first.getTransitionsFromState(DefaultStateType(label_p)))
			for(const auto& tq : second.getTransitionsFromState(DefaultStateType(label_q)))
				if(tp.first.second == tq.first.second)
					for(const auto& p : tp.second)
						for(const auto& q : tq.second)
							res.addTransition(state, tp.first.second, DefaultStateType(p, q));
	}

	return res;
}

auto AutomataIntersectionCartesianProductNFA = AutomataIntersectionCartesianProduct::RegistratorWrapper<automaton::NFA < > , automaton::NFA < > >(AutomataIntersectionCartesianProduct::intersection);

} /* namespace transform */

} /* namespace automaton */
