/*
 * ExactNondeterministicSubsequenceAutomaton.cpp
 *
 *  Created on: 7. 4. 2015
 *      Author: Jan Travnicek
 */

#include "ExactNondeterministicSubsequenceAutomaton.h"
#include <string/LinearString.h>
#include <string/Epsilon.h>

#include <deque>

namespace stringology {

namespace exact {

automaton::Automaton ExactNondeterministicSubsequenceAutomaton::construct(const string::String& text) {
	return dispatch(text.getData());
}

automaton::EpsilonNFA < > ExactNondeterministicSubsequenceAutomaton::construct(const string::LinearString < >& text) {
	automaton::EpsilonNFA < > res(DefaultStateType(0));
	res.addFinalState(DefaultStateType(0));
	res.setInputAlphabet(text.getAlphabet());

	int i = 1;
	for(const DefaultSymbolType& symbol : text.getContent()) {
		res.addState(DefaultStateType(i));
		res.addFinalState(DefaultStateType(i));

		res.addTransition(DefaultStateType(i-1), symbol, DefaultStateType(i));
		res.addTransition(DefaultStateType(i-1), DefaultStateType(i));
		i++;
	}
	return res;
}

auto ExactNondeterministicSubsequenceAutomatonLinearString = ExactNondeterministicSubsequenceAutomaton::RegistratorWrapper<automaton::EpsilonNFA < >, string::LinearString < >>( ExactNondeterministicSubsequenceAutomaton::construct );

} /* namespace exact */

} /* namespace stringology */
