/*
 * SuffixTrieNaive.h
 *
 *  Created on: 1. 11. 2014
 *      Author: Tomas Pecka
 */

#ifndef SUFFIX_TRIE_NAIVE_H_
#define SUFFIX_TRIE_NAIVE_H_

#include <indexes/SuffixTrie.h>
#include <string/String.h>
#include <string/LinearString.h>
#include <core/multipleDispatch.hpp>

namespace stringology {

namespace indexing {

/**
 * Constructs suffix trie for given string.
 *
 * Source: Lectures MI-EVY (CTU in Prague), Year 2014, Lecture 3, slide 4
 */

class SuffixTrieNaive : public std::SingleDispatch < SuffixTrieNaive, indexes::SuffixTrie < DefaultSymbolType, unsigned >, const string::StringBase & > {
public:
	/**
	 * Creates suffix trie
	 * @param string string to construct suffix trie for
	 * @return automaton
	 */
	static indexes::SuffixTrie < DefaultSymbolType, unsigned > construct ( const string::String & string );

	template < class SymbolType >
	static indexes::SuffixTrie < SymbolType, unsigned > construct ( const string::LinearString < SymbolType > & string );

};

template < class SymbolType >
indexes::SuffixTrie < SymbolType, unsigned > SuffixTrieNaive::construct ( const string::LinearString < SymbolType > & w ) {
	std::trie < SymbolType, std::variant < void, unsigned > > trie ( std::variant < void, unsigned >::from < void > ( ) );
	std::set < unsigned > nodeAlphabet;

	for ( unsigned i = 0; i < w.getContent ( ).size ( ); i++ ) {
		unsigned k = i;
		std::trie < SymbolType, std::variant < void, unsigned > > * n = & trie;

		 // inlined slow_find_one from MI-EVY lectures
		while ( k < w.getContent ( ).size ( ) && n->getChildren ( ).count ( w.getContent ( )[k] ) )
			n = & n->getChildren ( ).find ( w.getContent ( )[k++] )->second;

		for ( ; k < w.getContent ( ).size ( ); k++ ) {
			std::variant < void, unsigned > node = k + 1 < w.getContent ( ).size ( ) ? std::variant < void, unsigned >::from < void > ( ) : std::variant < void, unsigned > ( i );
			n = & n->getChildren ( ).insert ( std::make_pair ( w.getContent ( )[k], std::trie < SymbolType, std::variant < void, unsigned > > ( node ) ) ).first->second;
		}

		nodeAlphabet.insert ( i );
	}

	return indexes::SuffixTrie < SymbolType, unsigned > ( w.getAlphabet ( ), nodeAlphabet, trie );
}

} /* namespace indexing */

} /* namespace stringology */

#endif /* SUFFIX_TRIE_NAIVE_H_ */
