/*
 * ExactSubtreeRepeatsNaive.cpp
 *
 *  Created on: 1. 4. 2016
 *      Author: Jan Travnicek
 */

#include "ExactSubtreeRepeatsNaive.h"
#include "SubtreeJumpTable.h"

#include <tree/ranked/RankedTree.h>
#include <tree/ranked/PrefixRankedTree.h>
#include <tree/ranked/PrefixRankedBarTree.h>
#include <tree/Tree.h>
#include <global/GlobalData.h>

namespace tree {

namespace properties {

tree::Tree ExactSubtreeRepeatsNaive::repeats ( const tree::Tree & tree ) {
	return dispatch ( tree.getData ( ) );
}

std::tree < std::ranked_symbol < > > ExactSubtreeRepeatsNaive::repeats ( const std::tree < std::ranked_symbol < > > & node, std::map < std::pair < std::ranked_symbol < >, std::vector < std::ranked_symbol < > > >, int > & data, int & minId ) {
	std::vector < std::tree < std::ranked_symbol < > > > children;
	std::pair < std::ranked_symbol < >, std::vector < std::ranked_symbol < > > > childRepeatsKey ( node.getData ( ), std::vector < std::ranked_symbol < > > ( ) );

	for ( const std::tree < std::ranked_symbol < > > & child : node.getChildren() ) {
		children.push_back ( repeats ( child, data, minId ) );
		childRepeatsKey.second.push_back ( children.back ( ).getData ( ) );
	}

	int & uniqueRepeatId = data[childRepeatsKey];

	if ( uniqueRepeatId == 0 ) uniqueRepeatId = minId++;

	return std::tree < std::ranked_symbol < > > ( std::ranked_symbol < > ( DefaultSymbolType ( uniqueRepeatId ), node.getData ( ).getRank ( ) ), std::move ( children ) );
}

tree::RankedTree < > ExactSubtreeRepeatsNaive::repeats ( const tree::RankedTree < > & tree ) {
	int minId = 1;
	std::map < std::pair < std::ranked_symbol < >, std::vector < std::ranked_symbol < > > >, int > data;

	return tree::RankedTree < > ( repeats ( tree.getContent ( ), data, minId ) );
}

auto ExactRepeatsNaiveRankedTree = ExactSubtreeRepeatsNaive::RegistratorWrapper < tree::RankedTree < >, tree::RankedTree < > > ( ExactSubtreeRepeatsNaive::repeats );

std::ranked_symbol < > ExactSubtreeRepeatsNaive::repeatsPrefixRanked ( const std::vector < std::ranked_symbol < > > & symbols, std::vector < std::ranked_symbol < > > & res, std::map < std::pair < std::ranked_symbol < >, std::vector < std::ranked_symbol < > > >, int > & data, int & minId, int & index ) {
	int begin = index;
	std::pair < std::ranked_symbol < >, std::vector < std::ranked_symbol < > > > childRepeatsKey ( symbols[begin], std::vector < std::ranked_symbol < > > ( ) );

	res.push_back ( std::ranked_symbol < > ( DefaultSymbolType ( 0 ), symbols[begin].getRank ( ) ) );

	index++;

	for ( unsigned i = 0; i < ( unsigned ) symbols[begin].getRank ( ); ++i )
		childRepeatsKey.second.push_back ( repeatsPrefixRanked ( symbols, res, data, minId, index ) );

	int & uniqueRepeatId = data[childRepeatsKey];

	if ( uniqueRepeatId == 0 ) uniqueRepeatId = minId++;

	res[begin] = std::ranked_symbol < > ( DefaultSymbolType ( uniqueRepeatId ), symbols[begin].getRank ( ) );
	return res[begin];
}

tree::PrefixRankedTree < > ExactSubtreeRepeatsNaive::repeats ( const tree::PrefixRankedTree < > & tree ) {
	int minId = 1;
	int index = 0;
	std::vector < std::ranked_symbol < > > res;
	std::map < std::pair < std::ranked_symbol < >, std::vector < std::ranked_symbol < > > >, int > data;

	repeatsPrefixRanked ( tree.getContent ( ), res, data, minId, index );
	return tree::PrefixRankedTree < > ( res );
}

auto ExactRepeatsNaivePrefixRankedTree = ExactSubtreeRepeatsNaive::RegistratorWrapper < tree::PrefixRankedTree < >, tree::PrefixRankedTree < > > ( ExactSubtreeRepeatsNaive::repeats );

std::ranked_symbol < > ExactSubtreeRepeatsNaive::repeatsPrefixRankedBar ( const std::vector < std::ranked_symbol < > > & symbols, std::vector < std::ranked_symbol < > > & res, std::map < std::pair < std::ranked_symbol < >, std::vector < std::ranked_symbol < > > >, int > & data, int & minId, int & index ) {
	int begin = index;
	std::pair < std::ranked_symbol < >, std::vector < std::ranked_symbol < > > > childRepeatsKey ( symbols[begin], std::vector < std::ranked_symbol < > > ( ) );

	res.push_back ( std::ranked_symbol < > ( DefaultSymbolType ( 0 ), symbols[begin].getRank ( ) ) );

	index++;

	for ( unsigned i = 0; i < ( unsigned ) symbols[begin].getRank ( ); ++i )
		childRepeatsKey.second.push_back ( repeatsPrefixRankedBar ( symbols, res, data, minId, index ) );

	int & uniqueRepeatId = data[childRepeatsKey];

	if ( uniqueRepeatId == 0 ) uniqueRepeatId = minId++;

	res[begin] = std::ranked_symbol < > ( DefaultSymbolType ( uniqueRepeatId ), symbols[begin].getRank ( ) );
	res.push_back ( symbols[index] );
	index++;

	return res[begin];
}

tree::PrefixRankedBarTree < > ExactSubtreeRepeatsNaive::repeats ( const tree::PrefixRankedBarTree < > & tree ) {
	int minId = 1;
	int index = 0;
	std::vector < std::ranked_symbol < > > res;
	std::map < std::pair < std::ranked_symbol < >, std::vector < std::ranked_symbol < > > >, int > data;

	repeatsPrefixRankedBar ( tree.getContent ( ), res, data, minId, index );
	return tree::PrefixRankedBarTree < > ( tree.getBars ( ), res );
}

auto ExactRepeatsNaivePrefixRankedBarTree = ExactSubtreeRepeatsNaive::RegistratorWrapper < tree::PrefixRankedBarTree < >, tree::PrefixRankedBarTree < > > ( ExactSubtreeRepeatsNaive::repeats );

} /* namespace properties */

} /* namespace tree */
