#include "GrammarCFGtoPDATest.h"

#include <alphabet/BottomOfTheStackSymbol.h>
#include <grammar/ContextFree/CFG.h>

#include "grammar/convert/ToAutomaton.h"
#include "grammar/convert/ToAutomatonBottomUp.h"

#include <factory/XmlDataFactory.hpp>

#include <label/InitialStateLabel.h>
#include <label/FinalStateLabel.h>

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( GrammarCFGtoPDATest, "grammar" );
CPPUNIT_TEST_SUITE_REGISTRATION( GrammarCFGtoPDATest );

void GrammarCFGtoPDATest::setUp() {
}

void GrammarCFGtoPDATest::tearDown() {
}

void GrammarCFGtoPDATest::testTopDown()
{
	{
		DefaultSymbolType nE = DefaultSymbolType('E');
		DefaultSymbolType nT = DefaultSymbolType('T');
		DefaultSymbolType nF = DefaultSymbolType('F');

		DefaultSymbolType tP = DefaultSymbolType('+');
		DefaultSymbolType tS = DefaultSymbolType('*');
		DefaultSymbolType tL = DefaultSymbolType('(');
		DefaultSymbolType tR = DefaultSymbolType(')');
		DefaultSymbolType tA = DefaultSymbolType('a');

		grammar::CFG < > grammar(nE);
		grammar.setTerminalAlphabet(std::set<DefaultSymbolType>{tP, tS, tL, tR, tA});
		grammar.setNonterminalAlphabet(std::set<DefaultSymbolType>{nE, nT, nF});
		grammar.addRule(nE, std::vector<DefaultSymbolType>{nE, tP, nT});
		grammar.addRule(nE, std::vector<DefaultSymbolType>{nT});
		grammar.addRule(nT, std::vector<DefaultSymbolType>{nT, tS, nF});
		grammar.addRule(nT, std::vector<DefaultSymbolType>{nF});
		grammar.addRule(nF, std::vector<DefaultSymbolType>{tL, nE, tR});
		grammar.addRule(nF, std::vector<DefaultSymbolType>{tA});


		DefaultStateType q = label::InitialStateLabel::instance < DefaultStateType > ( );

		automaton::NPDA < > pda(q, nE);
		pda.setStates(std::set<DefaultStateType>{q});
		pda.setInputAlphabet(std::set<DefaultSymbolType>{tP, tS, tL, tR, tA});
		pda.setPushdownStoreAlphabet(std::set<DefaultSymbolType>{tP, tS, tL, tR, tA, nE, nT, nF});

		pda.addTransition(q, std::vector<DefaultSymbolType>{nE}, q, std::vector<DefaultSymbolType>{nE, tP, nT});
		pda.addTransition(q, std::vector<DefaultSymbolType>{nE}, q, std::vector<DefaultSymbolType>{nT});
		pda.addTransition(q, std::vector<DefaultSymbolType>{nT}, q, std::vector<DefaultSymbolType>{nT, tS, nF});
		pda.addTransition(q, std::vector<DefaultSymbolType>{nT}, q, std::vector<DefaultSymbolType>{nF});
		pda.addTransition(q, std::vector<DefaultSymbolType>{nF}, q, std::vector<DefaultSymbolType>{tL, nE, tR});
		pda.addTransition(q, std::vector<DefaultSymbolType>{nF}, q, std::vector<DefaultSymbolType>{tA});

		for(const auto& symbol: pda.getInputAlphabet())
			pda.addTransition(q, symbol, std::vector<DefaultSymbolType>{symbol}, q, std::vector<DefaultSymbolType>{});

		CPPUNIT_ASSERT(pda == grammar::convert::ToAutomaton::convert(grammar));
	}
}

void GrammarCFGtoPDATest::testBottomUp()
{
	{
		DefaultSymbolType nE = DefaultSymbolType('E');
		DefaultSymbolType nT = DefaultSymbolType('T');
		DefaultSymbolType nF = DefaultSymbolType('F');

		DefaultSymbolType tP = DefaultSymbolType('+');
		DefaultSymbolType tS = DefaultSymbolType('*');
		DefaultSymbolType tL = DefaultSymbolType('(');
		DefaultSymbolType tR = DefaultSymbolType(')');
		DefaultSymbolType tA = DefaultSymbolType('a');

		grammar::CFG < > grammar(nE);
		grammar.setTerminalAlphabet(std::set<DefaultSymbolType>{tP, tS, tL, tR, tA});
		grammar.setNonterminalAlphabet(std::set<DefaultSymbolType>{nE, nT, nF});
		grammar.addRule(nE, std::vector<DefaultSymbolType>{nE, tP, nT});
		grammar.addRule(nE, std::vector<DefaultSymbolType>{nT});
		grammar.addRule(nT, std::vector<DefaultSymbolType>{nT, tS, nF});
		grammar.addRule(nT, std::vector<DefaultSymbolType>{nF});
		grammar.addRule(nF, std::vector<DefaultSymbolType>{tL, nE, tR});
		grammar.addRule(nF, std::vector<DefaultSymbolType>{tA});


		DefaultStateType q = label::InitialStateLabel::instance < DefaultStateType > ( );
		DefaultStateType r = label::FinalStateLabel::instance < DefaultStateType > ( );
		DefaultSymbolType bots( alphabet::BottomOfTheStackSymbol::instance < DefaultSymbolType > ( ) );

		automaton::NPDA < > pda(q, bots);
		pda.addState(r);
		pda.addFinalState(r);
		pda.setInputAlphabet(std::set<DefaultSymbolType>{tP, tS, tL, tR, tA});
		pda.setPushdownStoreAlphabet(std::set<DefaultSymbolType>{tP, tS, tL, tR, tA, nE, nT, nF, bots});

		pda.addTransition(q, std::vector<DefaultSymbolType>{nE, tP, nT}, q, std::vector<DefaultSymbolType>{nE});
		pda.addTransition(q, std::vector<DefaultSymbolType>{nT}, q, std::vector<DefaultSymbolType>{nE});
		pda.addTransition(q, std::vector<DefaultSymbolType>{nT, tS, nF}, q, std::vector<DefaultSymbolType>{nT});
		pda.addTransition(q, std::vector<DefaultSymbolType>{nF}, q, std::vector<DefaultSymbolType>{nT});
		pda.addTransition(q, std::vector<DefaultSymbolType>{tL, nE, tR}, q, std::vector<DefaultSymbolType>{nF});
		pda.addTransition(q, std::vector<DefaultSymbolType>{tA}, q, std::vector<DefaultSymbolType>{nF});

		for(const auto& symbol: pda.getInputAlphabet())
			pda.addTransition(q, symbol, std::vector<DefaultSymbolType>{}, q, std::vector<DefaultSymbolType>{symbol});

		pda.addTransition(q, std::vector<DefaultSymbolType>{bots, nE}, r, std::vector<DefaultSymbolType>{});

		CPPUNIT_ASSERT(pda == grammar::convert::ToAutomatonBottomUp::convert(grammar));
	}
}
