#include "GrammarGenerateUpToLength.h"

#include "grammar/generate/GenerateUpToLength.h"

#include "grammar/ContextFree/EpsilonFreeCFG.h"
#include "string/LinearString.h"
#include <string/String.h>

#include <factory/StringDataFactory.hpp>

#define CPPUNIT_IMPLY(x, y) CPPUNIT_ASSERT(!(x) || (y))

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( GrammarGenerateUpToLength, "grammar" );
CPPUNIT_TEST_SUITE_REGISTRATION( GrammarGenerateUpToLength );

void GrammarGenerateUpToLength::setUp() {
}

void GrammarGenerateUpToLength::tearDown() {
}

void GrammarGenerateUpToLength::testGenerate1() {
	DefaultSymbolType S = DefaultSymbolType("S");
	DefaultSymbolType A = DefaultSymbolType("A");
	DefaultSymbolType B = DefaultSymbolType("B");
	DefaultSymbolType C = DefaultSymbolType("C");
	DefaultSymbolType D = DefaultSymbolType("D");

	DefaultSymbolType a = DefaultSymbolType('a');
	DefaultSymbolType b = DefaultSymbolType('b');

	grammar::EpsilonFreeCFG < > grammar1(S);
	grammar1.setNonterminalAlphabet({S, A, B, C, D});
	grammar1.setTerminalAlphabet({a, b});
	grammar1.setGeneratesEpsilon(false);

	std::set<string::LinearString < >> strings;

	CPPUNIT_ASSERT(strings == grammar::generate::GenerateUpToLength::generate(grammar1, 5));
}

void GrammarGenerateUpToLength::testGenerate2() {
	DefaultSymbolType S = DefaultSymbolType("S");
	DefaultSymbolType A = DefaultSymbolType("A");
	DefaultSymbolType B = DefaultSymbolType("B");
	DefaultSymbolType C = DefaultSymbolType("C");
	DefaultSymbolType D = DefaultSymbolType("D");

	DefaultSymbolType a = DefaultSymbolType('a');
	DefaultSymbolType b = DefaultSymbolType('b');
	DefaultSymbolType c = DefaultSymbolType('c');
	DefaultSymbolType d = DefaultSymbolType('d');

	grammar::EpsilonFreeCFG < > grammar1(S);
	grammar1.setNonterminalAlphabet({S, A, B, C, D});
	grammar1.setTerminalAlphabet({a, b, c, d});
	grammar1.addRule({S}, {A, D});
	grammar1.addRule({S}, {A});
	grammar1.addRule({S}, {D});
	grammar1.addRule({A}, {a, A, d});
	grammar1.addRule({A}, {a, d});
	grammar1.addRule({A}, {B, C});
	grammar1.addRule({A}, {B});
	grammar1.addRule({A}, {C});
	grammar1.addRule({B}, {b, B, c});
	grammar1.addRule({B}, {b, c});
	grammar1.addRule({C}, {c, C});
	grammar1.addRule({C}, {c});
	grammar1.addRule({D}, {d, D});
	grammar1.addRule({D}, {d});
	grammar1.setGeneratesEpsilon(true);

	std::set<string::LinearString < >> strings;
	strings.insert(string::LinearString < >(std::vector<DefaultSymbolType>{}));
	strings.insert(string::LinearString < >(std::vector<DefaultSymbolType>{c}));
	strings.insert(string::LinearString < >(std::vector<DefaultSymbolType>{c, c}));
	strings.insert(string::LinearString < >(std::vector<DefaultSymbolType>{c, c, c}));
	strings.insert(string::LinearString < >(std::vector<DefaultSymbolType>{d}));
	strings.insert(string::LinearString < >(std::vector<DefaultSymbolType>{d, d}));
	strings.insert(string::LinearString < >(std::vector<DefaultSymbolType>{d, d, d}));
	strings.insert(string::LinearString < >(std::vector<DefaultSymbolType>{a, d}));
	strings.insert(string::LinearString < >(std::vector<DefaultSymbolType>{a, d, d}));
	strings.insert(string::LinearString < >(std::vector<DefaultSymbolType>{b, c}));
	strings.insert(string::LinearString < >(std::vector<DefaultSymbolType>{b, c, c}));
	strings.insert(string::LinearString < >(std::vector<DefaultSymbolType>{c, d}));
	strings.insert(string::LinearString < >(std::vector<DefaultSymbolType>{c, c, d}));
	strings.insert(string::LinearString < >(std::vector<DefaultSymbolType>{c, d, d}));
	strings.insert(string::LinearString < >(std::vector<DefaultSymbolType>{a, c, d}));
	strings.insert(string::LinearString < >(std::vector<DefaultSymbolType>{b, c, d}));

	for(const string::LinearString < >& str : grammar::generate::GenerateUpToLength::generate(grammar1, 3)) {
		std::cout << alib::StringDataFactory::toString(string::String(str)) << std::endl;
	}

	CPPUNIT_ASSERT(strings == grammar::generate::GenerateUpToLength::generate(grammar1, 3));
}

void GrammarGenerateUpToLength::testGenerate3() {
	DefaultSymbolType S = DefaultSymbolType("S");
	DefaultSymbolType A = DefaultSymbolType("A");
	DefaultSymbolType B = DefaultSymbolType("B");
	DefaultSymbolType C = DefaultSymbolType("C");
	DefaultSymbolType D = DefaultSymbolType("D");

	DefaultSymbolType a = DefaultSymbolType('a');
	DefaultSymbolType b = DefaultSymbolType('b');
	DefaultSymbolType c = DefaultSymbolType('c');
	DefaultSymbolType d = DefaultSymbolType('d');

	grammar::EpsilonFreeCFG < > grammar1(S);
	grammar1.setNonterminalAlphabet({S, A, B, C, D});
	grammar1.setTerminalAlphabet({a, b, c, d});
	grammar1.addRule({S}, {A, D});
	grammar1.addRule({S}, {D});
	grammar1.addRule({A}, {a, A, d});
	grammar1.addRule({A}, {a, d});
	grammar1.addRule({A}, {B, C});
	grammar1.addRule({A}, {B});
	grammar1.addRule({A}, {C});
	grammar1.addRule({B}, {b, B, c});
	grammar1.addRule({B}, {b, c});
	grammar1.addRule({C}, {c, C});
	grammar1.addRule({C}, {c});
	grammar1.addRule({D}, {d, D});
	grammar1.addRule({D}, {d});

	std::set<string::LinearString < >> strings;
	strings.insert(string::LinearString < >(std::vector<DefaultSymbolType>{d}));
	strings.insert(string::LinearString < >(std::vector<DefaultSymbolType>{d, d}));
	strings.insert(string::LinearString < >(std::vector<DefaultSymbolType>{d, d, d}));
	strings.insert(string::LinearString < >(std::vector<DefaultSymbolType>{a, d, d}));
	strings.insert(string::LinearString < >(std::vector<DefaultSymbolType>{c, d}));
	strings.insert(string::LinearString < >(std::vector<DefaultSymbolType>{c, c, d}));
	strings.insert(string::LinearString < >(std::vector<DefaultSymbolType>{c, d, d}));
	strings.insert(string::LinearString < >(std::vector<DefaultSymbolType>{b, c, d}));

	for(const string::LinearString < >& str : grammar::generate::GenerateUpToLength::generate(grammar1, 3)) {
		std::cout << alib::StringDataFactory::toString(string::String(str)) << std::endl;
	}

	CPPUNIT_ASSERT(strings == grammar::generate::GenerateUpToLength::generate(grammar1, 3));
}
