/*
 * LeftFactorize.cpp
 *
 *  Created on: 9. 6. 2015
 *	  Author: Jan Travnicek
 */

#include "LeftFactorize.h"

#include <grammar/ContextFree/CFG.h>
#include <common/createUnique.hpp>

namespace grammar {

namespace parsing {

void LeftFactorize::leftFactorize ( grammar::CFG < > & grammar, const DefaultSymbolType & terminal, const DefaultSymbolType & nonterminal ) {
	grammar::CFG < > res ( grammar.getInitialSymbol ( ) );

	res.setNonterminalAlphabet ( grammar.getNonterminalAlphabet ( ) );
	res.setTerminalAlphabet ( grammar.getTerminalAlphabet ( ) );

	DefaultSymbolType primed = common::createUnique ( nonterminal, grammar.getTerminalAlphabet ( ), grammar.getNonterminalAlphabet ( ) );
	res.addNonterminalSymbol ( primed );

	for ( const std::pair < const DefaultSymbolType, std::set < std::vector < DefaultSymbolType > > > & rule : grammar.getRules ( ) ) {
		const DefaultSymbolType & lhs = rule.first;

		for ( const std::vector < DefaultSymbolType > & rhs : rule.second ) {
			if ( ( lhs == nonterminal ) && ( rhs.size ( ) > 0 ) && ( rhs[0] == terminal ) ) {
				res.addRule ( lhs, std::vector < DefaultSymbolType > { terminal, primed } );
				res.addRule ( primed, std::vector < DefaultSymbolType > ( rhs.begin ( ) + 1, rhs.end ( ) ) );
			} else {
				res.addRule ( lhs, rhs );
			}
		}
	}

	grammar = res;
}

} /* namespace parsing */

} /* namespace grammar */
