/*
 * Author: Radovan Cerveny
 */

#ifndef SUFFIX_AUTOMATON_H_
#define SUFFIX_AUTOMATON_H_

#include <automaton/Automaton.h>
#include <automaton/FSM/DFA.h>
#include <automaton/FSM/EpsilonNFA.h>
#include <string/LinearString.h>
#include <string/String.h>
#include <core/multipleDispatch.hpp>

namespace stringology {

namespace exact {

class SuffixAutomaton : public std::SingleDispatch < SuffixAutomaton, automaton::Automaton, const string::StringBase & > {
private:
    static void suffixAutomatonAddSymbol ( automaton::DFA<> & suffixAutomaton, const DefaultSymbolType & symbol, std::map < DefaultStateType, std::pair<DefaultStateType, int > > & suffixLinks, DefaultStateType & lastState );

public:
    /**
     * Naive construction of minimal suffix automaton for given pattern - EpsNFA -> NFA -> DFA -> minDFA -> removeErrorState.
     * @return minimal suffix automaton for given pattern.
     */
    static automaton::Automaton naiveConstruct ( const string::String & pattern );

    static automaton::DFA<> naiveConstruct ( const string::LinearString < > & pattern );

     /**
     * Linear time on-line construction of minimal suffix automaton for given pattern.
     * @return minimal suffix automaton for given pattern.
     */
    static automaton::Automaton construct ( const string::String & pattern );

    static automaton::DFA<> construct ( const string::LinearString < > & pattern );

};

} /* namespace exact */

} /* namespace stringology */

#endif /* SUFFIX_AUTOMATON_H_ */
