#include "FollowTest.h"

#include "grammar/parsing/Follow.h"
#include "string/Epsilon.h"
#include "grammar/ContextFree/CFG.h"

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( FollowTest, "grammar" );
CPPUNIT_TEST_SUITE_REGISTRATION ( FollowTest );

void FollowTest::setUp ( ) {
}

void FollowTest::tearDown ( ) {
}

void FollowTest::testFollow ( ) {
	{
		DefaultSymbolType nE = DefaultSymbolType ( 'E' );
		DefaultSymbolType nT = DefaultSymbolType ( 'T' );
		DefaultSymbolType nF = DefaultSymbolType ( 'F' );

		DefaultSymbolType tP = DefaultSymbolType ( '+' );
		DefaultSymbolType tS = DefaultSymbolType ( '*' );
		DefaultSymbolType tL = DefaultSymbolType ( '(' );
		DefaultSymbolType tR = DefaultSymbolType ( ')' );
		DefaultSymbolType tA = DefaultSymbolType ( 'a' );

		grammar::CFG < > grammar ( nE );
		grammar.setTerminalAlphabet ( std::set < DefaultSymbolType > { tP, tS, tL, tR, tA } );
		grammar.setNonterminalAlphabet ( std::set < DefaultSymbolType > { nE, nT, nF } );

		std::vector < DefaultSymbolType > rhsE1 ( { nE, tP, nT } );
		std::vector < DefaultSymbolType > rhsE2 ( { nT } );
		std::vector < DefaultSymbolType > rhsT1 ( { nT, tS, nF } );
		std::vector < DefaultSymbolType > rhsT2 ( { nF } );
		std::vector < DefaultSymbolType > rhsF1 ( { tA } );
		std::vector < DefaultSymbolType > rhsF2 ( { tL, nE, tR } );

		grammar.addRule ( nE, rhsE1 );
		grammar.addRule ( nE, rhsE2 );
		grammar.addRule ( nT, rhsT1 );
		grammar.addRule ( nT, rhsT2 );
		grammar.addRule ( nF, rhsF1 );
		grammar.addRule ( nF, rhsF2 );

		 // --------------------------------------------------
		std::map < DefaultSymbolType, std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > > follow;

		follow[nE] = std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > {
			string::Epsilon < >::EPSILON, tP, tR
		};
		follow[nT] = std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > {
			string::Epsilon < >::EPSILON, tP, tR, tS
		};
		follow[nF] = std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > {
			string::Epsilon < >::EPSILON, tP, tR, tS
		};

		// --------------------------------------------------

		std::map < DefaultSymbolType, std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > > followAlgo;

		for ( const auto & nt : grammar.getNonterminalAlphabet ( ) )
			followAlgo[nt] = grammar::parsing::Follow::follow ( grammar, nt );

		 // std::cout << follow << std::endl;
		 // std::cout << followAlgo << std::endl;
		CPPUNIT_ASSERT ( follow == followAlgo );
	}

	{
		DefaultSymbolType nS = DefaultSymbolType ( 'S' );
		DefaultSymbolType nA = DefaultSymbolType ( 'A' );
		DefaultSymbolType nB = DefaultSymbolType ( 'B' );
		DefaultSymbolType nC = DefaultSymbolType ( 'C' );
		DefaultSymbolType nD = DefaultSymbolType ( 'D' );
		DefaultSymbolType nE = DefaultSymbolType ( 'E' );
		DefaultSymbolType nF = DefaultSymbolType ( 'F' );

		DefaultSymbolType tA = DefaultSymbolType ( 'a' );
		DefaultSymbolType tB = DefaultSymbolType ( 'b' );
		DefaultSymbolType tC = DefaultSymbolType ( 'c' );
		DefaultSymbolType tD = DefaultSymbolType ( 'd' );
		DefaultSymbolType tE = DefaultSymbolType ( 'e' );

		grammar::CFG < > grammar ( nS );
		grammar.setTerminalAlphabet ( std::set < DefaultSymbolType > { tA, tB, tC, tD, tE } );
		grammar.setNonterminalAlphabet ( std::set < DefaultSymbolType > { nS, nA, nB, nC, nD, nE, nF } );

		std::vector < DefaultSymbolType > rhsS1 ( { nB, tD, nS } );
		std::vector < DefaultSymbolType > rhsS2 ( { tD, tD, nC } );
		std::vector < DefaultSymbolType > rhsS3 ( { tC, nA } );
		std::vector < DefaultSymbolType > rhsA1 ( { tA, tE, nE } );
		std::vector < DefaultSymbolType > rhsA2 ( { tB, tB, nE } );
		std::vector < DefaultSymbolType > rhsB1 ( { tA, nF } );
		std::vector < DefaultSymbolType > rhsB2 ( { tB, tB, nD } );
		std::vector < DefaultSymbolType > rhsC1 ( { tA, nB, tD } );
		std::vector < DefaultSymbolType > rhsC2 ( { tE, nA } );
		std::vector < DefaultSymbolType > rhsD1 ( { tC, tA, nF } );
		std::vector < DefaultSymbolType > rhsE1 ( { tC, tA, tE, nE } );
		std::vector < DefaultSymbolType > rhsE2 ( { } );
		std::vector < DefaultSymbolType > rhsF1 ( { tE, nD } );
		std::vector < DefaultSymbolType > rhsF2 ( { } );

		grammar.addRule ( nS, rhsS1 );
		grammar.addRule ( nS, rhsS2 );
		grammar.addRule ( nS, rhsS3 );
		grammar.addRule ( nA, rhsA1 );
		grammar.addRule ( nA, rhsA2 );
		grammar.addRule ( nB, rhsB1 );
		grammar.addRule ( nB, rhsB2 );
		grammar.addRule ( nC, rhsC1 );
		grammar.addRule ( nC, rhsC2 );
		grammar.addRule ( nD, rhsD1 );
		grammar.addRule ( nE, rhsE1 );
		grammar.addRule ( nE, rhsE2 );
		grammar.addRule ( nF, rhsF1 );
		grammar.addRule ( nF, rhsF2 );

		 // --------------------------------------------------
		std::map < DefaultSymbolType, std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > > follow;

		follow[nS] = std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > {
			string::Epsilon < >::EPSILON
		};
		follow[nA] = std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > {
			string::Epsilon < >::EPSILON
		};
		follow[nB] = std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > {
			tD
		};
		follow[nC] = std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > {
			string::Epsilon < >::EPSILON
		};
		follow[nD] = std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > {
			tD
		};
		follow[nE] = std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > {
			string::Epsilon < >::EPSILON
		};
		follow[nF] = std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > {
			tD
		};

		// --------------------------------------------------

		std::map < DefaultSymbolType, std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > > followAlgo;

		for ( const auto & nt : grammar.getNonterminalAlphabet ( ) )
			followAlgo[nt] = grammar::parsing::Follow::follow ( grammar, nt );

		 // std::cout << follow << std::endl;
		 // std::cout << followAlgo << std::endl;
		CPPUNIT_ASSERT ( follow == followAlgo );
	}
}
