#include "HandleFirstFollowConflict.h"

#include "grammar/ContextFree/CFG.h"
#include "common/DefaultSymbolsPairType.h"
#include "grammar/parsing/HandleFirstFollowConflict.h"
#include "factory/StringDataFactory.hpp"

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( HandleFirstFollowConflict, "grammar" );
CPPUNIT_TEST_SUITE_REGISTRATION ( HandleFirstFollowConflict );

void HandleFirstFollowConflict::setUp ( ) {
}

void HandleFirstFollowConflict::tearDown ( ) {
}

void HandleFirstFollowConflict::testHandleFirstFollowConflict ( ) {
	DefaultSymbolType A = DefaultSymbolType ( "A" );
	DefaultSymbolType B = DefaultSymbolType ( "B" );
	DefaultSymbolType C = DefaultSymbolType ( "C" );

	DefaultSymbolType a = DefaultSymbolType ( 'a' );
	DefaultSymbolType b = DefaultSymbolType ( 'b' );
	DefaultSymbolType c = DefaultSymbolType ( 'c' );
	DefaultSymbolType d = DefaultSymbolType ( 'd' );

	DefaultSymbolType Ba = DefaultSymbolType ( DefaultSymbolsPairType ( std::make_pair ( B, a ) ) );

	grammar::CFG < > grammar ( A );

	grammar.setTerminalAlphabet ( { a, b, c, d } );
	grammar.setNonterminalAlphabet ( { A, B, C } );
	grammar.setInitialSymbol ( A );

	grammar.addRule ( A, std::vector < DefaultSymbolType > { a, B, C } );
	grammar.addRule ( A, std::vector < DefaultSymbolType > { c, C } );
	grammar.addRule ( B, std::vector < DefaultSymbolType > { a, B, a, d } );
	grammar.addRule ( B, std::vector < DefaultSymbolType > { } );
	grammar.addRule ( C, std::vector < DefaultSymbolType > { a, C } );
	grammar.addRule ( C, std::vector < DefaultSymbolType > { b, B } );

	grammar::CFG < > res = grammar;
	grammar::parsing::HandleFirstFollowConflict::handleFirstFollowConflict ( res, a, B, { { a, B, C }, { } } );

	grammar::CFG < > comp ( A );

	comp.setTerminalAlphabet ( { a, b, c, d } );
	comp.setNonterminalAlphabet ( { A, B, C } );
	comp.setInitialSymbol ( A );

	comp.addRule ( A, std::vector < DefaultSymbolType > { a, B, a, C } );
	comp.addRule ( A, std::vector < DefaultSymbolType > { a, B, b, B } );
	comp.addRule ( A, std::vector < DefaultSymbolType > { c, C } );
	comp.addRule ( B, std::vector < DefaultSymbolType > { a, B, a, d } );
	comp.addRule ( B, std::vector < DefaultSymbolType > { } );
	comp.addRule ( C, std::vector < DefaultSymbolType > { a, C } );
	comp.addRule ( C, std::vector < DefaultSymbolType > { b, B } );

//	std::cout << "res " << alib::StringDataFactory::toString ( grammar::Grammar ( res ) ) << std::endl << "comp " << alib::StringDataFactory::toString ( grammar::Grammar ( comp ) ) << std::endl;

	CPPUNIT_ASSERT ( res == comp );

	grammar::parsing::HandleFirstFollowConflict::handleFirstFollowConflict ( res, a, B, { { a, B, C }, { } } );

	grammar::CFG < > comp2 ( A );

	comp2.setTerminalAlphabet ( { a, b, c, d } );
	comp2.setNonterminalAlphabet ( { A, B, Ba, C } );
	comp2.setInitialSymbol ( A );

	comp2.addRule ( A, std::vector < DefaultSymbolType > { a, Ba, C } );
	comp2.addRule ( A, std::vector < DefaultSymbolType > { a, B, b, B } );
	comp2.addRule ( A, std::vector < DefaultSymbolType > { c, C } );
	comp2.addRule ( B, std::vector < DefaultSymbolType > { a, Ba, d } );
	comp2.addRule ( B, std::vector < DefaultSymbolType > { } );
	comp2.addRule ( Ba, std::vector < DefaultSymbolType > { a, Ba, d, a } );
	comp2.addRule ( Ba, std::vector < DefaultSymbolType > { a } );
	comp2.addRule ( C, std::vector < DefaultSymbolType > { a, C } );
	comp2.addRule ( C, std::vector < DefaultSymbolType > { b, B } );

//	std::cout << "res2 " << alib::StringDataFactory::toString ( grammar::Grammar ( res ) ) << std::endl << "comp2 " << alib::StringDataFactory::toString ( grammar::Grammar ( comp2 ) ) << std::endl;

	CPPUNIT_ASSERT ( res == comp2 );
}

void HandleFirstFollowConflict::testHandleFirstFollowConflict2 ( ) {
	DefaultSymbolType A = DefaultSymbolType ( "A" );
	DefaultSymbolType B = DefaultSymbolType ( "B" );
	DefaultSymbolType C = DefaultSymbolType ( "C" );
	DefaultSymbolType X = DefaultSymbolType ( "X" );

	DefaultSymbolType a = DefaultSymbolType ( 'a' );
	DefaultSymbolType b = DefaultSymbolType ( 'b' );
	DefaultSymbolType c = DefaultSymbolType ( 'c' );
	DefaultSymbolType d = DefaultSymbolType ( 'd' );

	DefaultSymbolType Ba = DefaultSymbolType ( DefaultSymbolsPairType ( std::make_pair ( B, a ) ) );
	DefaultSymbolType Xa = DefaultSymbolType ( DefaultSymbolsPairType ( std::make_pair ( X, a ) ) );

	grammar::CFG < > grammar ( A );

	grammar.setTerminalAlphabet ( { a, b, c, d } );
	grammar.setNonterminalAlphabet ( { A, B, C, X } );
	grammar.setInitialSymbol ( A );

	grammar.addRule ( A, std::vector < DefaultSymbolType > { a, X, C, a } );
	grammar.addRule ( A, std::vector < DefaultSymbolType > { c, C } );
	grammar.addRule ( X, { c, B });
	grammar.addRule ( B, std::vector < DefaultSymbolType > { a, B, a, d } );
	grammar.addRule ( B, std::vector < DefaultSymbolType > { } );
	grammar.addRule ( C, std::vector < DefaultSymbolType > { a, C } );
	grammar.addRule ( C, std::vector < DefaultSymbolType > { b, B, c } );

	grammar::CFG < > res = grammar;
	grammar::parsing::HandleFirstFollowConflict::handleFirstFollowConflict ( res, a, B, { { a, B, C }, { } } );

	grammar::CFG < > comp ( A );

	comp.setTerminalAlphabet ( { a, b, c, d } );
	comp.setNonterminalAlphabet ( { A, B, C, X } );
	comp.setInitialSymbol ( A );

	comp.addRule ( A, std::vector < DefaultSymbolType > { a, X, a, C, a } );
	comp.addRule ( A, std::vector < DefaultSymbolType > { a, X, b, B, c, a } );
	comp.addRule ( A, std::vector < DefaultSymbolType > { c, C } );
	comp.addRule ( X, { c, B } );
	comp.addRule ( B, std::vector < DefaultSymbolType > { a, B, a, d } );
	comp.addRule ( B, std::vector < DefaultSymbolType > { } );
	comp.addRule ( C, std::vector < DefaultSymbolType > { a, C } );
	comp.addRule ( C, std::vector < DefaultSymbolType > { b, B, c } );

//	std::cout << "res " << alib::StringDataFactory::toString ( grammar::Grammar ( res ) ) << std::endl << "comp " << alib::StringDataFactory::toString ( grammar::Grammar ( comp ) ) << std::endl;

	CPPUNIT_ASSERT ( res == comp );

	grammar::parsing::HandleFirstFollowConflict::handleFirstFollowConflict ( res, a, B, { { a, B, C }, { } } );

	grammar::CFG < > comp2 ( A );

	comp2.setTerminalAlphabet ( { a, b, c, d } );
	comp2.setNonterminalAlphabet ( { A, B, Ba, C, X, Xa } );
	comp2.setInitialSymbol ( A );

	comp2.addRule ( A, std::vector < DefaultSymbolType > { a, Xa, C, a } );
	comp2.addRule ( A, std::vector < DefaultSymbolType > { a, X, b, B, c, a } );
	comp2.addRule ( A, std::vector < DefaultSymbolType > { c, C } );
	comp2.addRule ( X, { c, B } );
	comp2.addRule ( Xa, { c, Ba } );
	comp2.addRule ( B, std::vector < DefaultSymbolType > { a, Ba, d } );
	comp2.addRule ( B, std::vector < DefaultSymbolType > { } );
	comp2.addRule ( Ba, std::vector < DefaultSymbolType > { a, Ba, d, a } );
	comp2.addRule ( Ba, std::vector < DefaultSymbolType > { a } );
	comp2.addRule ( C, std::vector < DefaultSymbolType > { a, C } );
	comp2.addRule ( C, std::vector < DefaultSymbolType > { b, B, c } );

//	std::cout << "res2 " << alib::StringDataFactory::toString ( grammar::Grammar ( res ) ) << std::endl << "comp2 " << alib::StringDataFactory::toString ( grammar::Grammar ( comp2 ) ) << std::endl;

	CPPUNIT_ASSERT ( res == comp2 );
}
