#include "BfsTest.h"

#include "graph/traverse/Bfs.h"

#include <unordered_map>

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( GraphBfsTest, "graph" );
CPPUNIT_TEST_SUITE_REGISTRATION( GraphBfsTest );

void GraphBfsTest::testTraverseAll()
{
	std::cout << "BFS test" << std::endl;
	// Common
	int counter;
	graph::Node n1("n1");
	graph::Node n2("n2");
	graph::Node n3("n3");
	graph::Node n4("n4");
	graph::Node n5("n5");
	graph::Node n6("n6");

	// Directed
	graph::AdjacencyListDirectedGraph dg;
	dg.addNode(n1);
	dg.addNode(n2);
	dg.addNode(n3);
	dg.addNode(n4);
	dg.addNode(n5);
	dg.addNode(n6);
	dg.addEdge(graph::DirectedEdge(n1, n2));
	dg.addEdge(graph::DirectedEdge(n1, n2, "multi-edge"));
	dg.addEdge(graph::DirectedEdge(n2, n3));
	dg.addEdge(graph::DirectedEdge(n3, n4));
	dg.addEdge(graph::DirectedEdge(n3, n6));
	dg.addEdge(graph::DirectedEdge(n4, n1));
	dg.addEdge(graph::DirectedEdge(n4, n5));

	counter = 0;
	graph::traverse::Bfs::bfs(dg, n1, [&](const graph::Node &) {
		counter++;
		return true;
	});

	CPPUNIT_ASSERT_EQUAL(6, counter);

	// Undirected
	graph::AdjacencyListUndirectedGraph ug;
	ug.addNode(n1);
	ug.addNode(n2);
	ug.addNode(n3);
	ug.addNode(n4);
	ug.addNode(n5);
	ug.addNode(n6);
	ug.addEdge(graph::UndirectedEdge(n1, n2));
	ug.addEdge(graph::UndirectedEdge(n1, n2, "multi-edge"));
	ug.addEdge(graph::UndirectedEdge(n2, n3));
	ug.addEdge(graph::UndirectedEdge(n3, n4));
	ug.addEdge(graph::UndirectedEdge(n3, n6));
	ug.addEdge(graph::UndirectedEdge(n4, n1));
	ug.addEdge(graph::UndirectedEdge(n4, n5));

	counter = 0;
	graph::traverse::Bfs::bfs(ug, n1, [&](const graph::Node &) {
		counter++;
		return true;
	});

	CPPUNIT_ASSERT_EQUAL(6, counter);
}

void GraphBfsTest::testEarlyReturn()
{
	// Common
	int counter;
	graph::Node n1("n1");
	graph::Node n2("n2");
	graph::Node n3("n3");
	graph::Node n4("n4");

	// Directed
	graph::AdjacencyListDirectedGraph dg;
	dg.addNode(n1);
	dg.addNode(n2);
	dg.addNode(n3);
	dg.addNode(n4);
	dg.addEdge(graph::DirectedEdge(n1, n2));
	dg.addEdge(graph::DirectedEdge(n1, n2, "multi-edge"));
	dg.addEdge(graph::DirectedEdge(n2, n3));
	dg.addEdge(graph::DirectedEdge(n3, n4));

	counter = 0;
	graph::traverse::Bfs::bfs(dg, n1, [&](const graph::Node &) {
		counter++;
		if (counter == 2) {
			return false;
		}
		return true;
	});

	CPPUNIT_ASSERT_EQUAL(2, counter);

	// Undirected
	graph::AdjacencyListUndirectedGraph ug;
	ug.addNode(n1);
	ug.addNode(n2);
	ug.addNode(n3);
	ug.addNode(n4);
	ug.addEdge(graph::UndirectedEdge(n1, n2));
	ug.addEdge(graph::UndirectedEdge(n1, n2, "multi-edge"));
	ug.addEdge(graph::UndirectedEdge(n2, n3));
	ug.addEdge(graph::UndirectedEdge(n3, n4));

	counter = 0;
	graph::traverse::Bfs::bfs(ug, n1, [&](const graph::Node &) {
		counter++;
		if (counter == 2) {
			return false;
		}
		return true;
	});

	CPPUNIT_ASSERT_EQUAL(2, counter);
}

void GraphBfsTest::testDisconnectedGraph()
{
	// Common
	int counter;
	graph::Node n1("n1");
	graph::Node n2("n2");
	graph::Node n3("n3");
	graph::Node n4("n4");
	graph::Node n5("n5");

	// Directed
	graph::AdjacencyListDirectedGraph dg;
	dg.addNode(n1);
	dg.addNode(n2);
	dg.addNode(n3);
	dg.addNode(n4);
	dg.addNode(n5);
	dg.addEdge(graph::DirectedEdge(n1, n2));
	dg.addEdge(graph::DirectedEdge(n1, n2, "multi-edge"));
	dg.addEdge(graph::DirectedEdge(n2, n3));
	dg.addEdge(graph::DirectedEdge(n3, n4));

	counter = 0;
	graph::traverse::Bfs::bfs(dg, n1, [&](const graph::Node &) {
		counter++;
		return true;
	});

	CPPUNIT_ASSERT_EQUAL(4, counter);

	counter = 0;
	graph::traverse::Bfs::bfs(dg, n5, [&](const graph::Node &) {
		counter++;
		return true;
	});

	CPPUNIT_ASSERT_EQUAL(1, counter);

	// Undirected
	graph::AdjacencyListUndirectedGraph ug;
	ug.addNode(n1);
	ug.addNode(n2);
	ug.addNode(n3);
	ug.addNode(n4);
	ug.addNode(n5);
	ug.addEdge(graph::UndirectedEdge(n1, n2));
	ug.addEdge(graph::UndirectedEdge(n1, n2, "multi-edge"));
	ug.addEdge(graph::UndirectedEdge(n2, n3));
	ug.addEdge(graph::UndirectedEdge(n3, n4));

	counter = 0;
	graph::traverse::Bfs::bfs(ug, n1, [&](const graph::Node &) {
		counter++;
		return true;
	});

	CPPUNIT_ASSERT_EQUAL(4, counter);

	counter = 0;
	graph::traverse::Bfs::bfs(ug, n5, [&](const graph::Node &) {
		counter++;
		return true;
	});

	CPPUNIT_ASSERT_EQUAL(1, counter);
}

void GraphBfsTest::testBfs2()
{
	// Common
	std::unordered_map<graph::Node, int> distances;
	std::unordered_map<graph::Node, graph::Node> predecessors;

	graph::Node n1("n1");
	graph::Node n2("n2");
	graph::Node n3("n3");
	graph::Node n4("n4");
	graph::Node n5("n5");

	// Directed
	graph::AdjacencyListDirectedGraph dg;
	dg.addNode(n1);
	dg.addNode(n2);
	dg.addNode(n3);
	dg.addNode(n4);
	dg.addNode(n5);
	dg.addEdge(graph::DirectedEdge(n1, n2));
	dg.addEdge(graph::DirectedEdge(n1, n3));
	dg.addEdge(graph::DirectedEdge(n1, n4));
	dg.addEdge(graph::DirectedEdge(n2, n5));

	distances.clear();
	predecessors.clear();

	graph::Graph g(dg);

	graph::traverse::Bfs::bfs(g, n1, [&](const graph::Node &n, const graph::Node &p, int d) {
		distances[n] = d;
		predecessors.insert(std::make_pair(n, p));
		return true;
	});

	CPPUNIT_ASSERT_EQUAL(0, distances[n1]);
	CPPUNIT_ASSERT_EQUAL(1, distances[n2]);
	CPPUNIT_ASSERT_EQUAL(1, distances[n3]);
	CPPUNIT_ASSERT_EQUAL(1, distances[n4]);
	CPPUNIT_ASSERT_EQUAL(2, distances[n5]);

	CPPUNIT_ASSERT_EQUAL(n1, predecessors.find(n1)->second);
	CPPUNIT_ASSERT_EQUAL(n1, predecessors.find(n2)->second);
	CPPUNIT_ASSERT_EQUAL(n1, predecessors.find(n3)->second);
	CPPUNIT_ASSERT_EQUAL(n1, predecessors.find(n4)->second);
	CPPUNIT_ASSERT_EQUAL(n2, predecessors.find(n5)->second);

	// Undirected
	graph::AdjacencyListUndirectedGraph ug;
	ug.addNode(n1);
	ug.addNode(n2);
	ug.addNode(n3);
	ug.addNode(n4);
	ug.addNode(n5);
	ug.addEdge(graph::UndirectedEdge(n1, n2));
	ug.addEdge(graph::UndirectedEdge(n1, n3));
	ug.addEdge(graph::UndirectedEdge(n1, n4));
	ug.addEdge(graph::UndirectedEdge(n2, n5));

	distances.clear();
	predecessors.clear();

	graph::Graph g2(ug);

	graph::traverse::Bfs::bfs(g2, n1, [&](const graph::Node &n, const graph::Node &p, int d) {
		distances[n] = d;
		predecessors.insert(std::make_pair(n, p));
		return true;
	});

	CPPUNIT_ASSERT_EQUAL(0, distances[n1]);
	CPPUNIT_ASSERT_EQUAL(1, distances[n2]);
	CPPUNIT_ASSERT_EQUAL(1, distances[n3]);
	CPPUNIT_ASSERT_EQUAL(1, distances[n4]);
	CPPUNIT_ASSERT_EQUAL(2, distances[n5]);

	CPPUNIT_ASSERT_EQUAL(n1, predecessors.find(n1)->second);
	CPPUNIT_ASSERT_EQUAL(n1, predecessors.find(n2)->second);
	CPPUNIT_ASSERT_EQUAL(n1, predecessors.find(n3)->second);
	CPPUNIT_ASSERT_EQUAL(n1, predecessors.find(n4)->second);
	CPPUNIT_ASSERT_EQUAL(n2, predecessors.find(n5)->second);
}
