#include "SuffixAutomatonTest.h"

#include "string/LinearString.h"
#include "stringology/exact/SuffixAutomaton.h"
#include "stringology/exact/BackwardDAWGMatching.h"
#include "stringology/exact/ExactFactorMatch.h"

#include "string/generate/RandomStringFactory.h"
#include "string/generate/RandomSubstringFactory.h"

#define CPPUNIT_IMPLY( x, y )    CPPUNIT_ASSERT ( !( x ) || ( y ) )

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( SuffixAutomatonTest, "stringology" );
CPPUNIT_TEST_SUITE_REGISTRATION ( SuffixAutomatonTest );

void SuffixAutomatonTest::setUp ( ) {
}

void SuffixAutomatonTest::tearDown ( ) {
}

void SuffixAutomatonTest::testSuffixAutomatonConstruction ( ) {

    string::LinearString < > pattern ( "atatac" );

    automaton::DFA<> suffixAutomaton = stringology::exact::SuffixAutomaton::construct ( pattern );

    automaton::DFA<> refSuffixAutomaton ( DefaultStateType ( 0 ) );

    refSuffixAutomaton.setInputAlphabet ( pattern.getAlphabet ( ) );

    for ( int i = 1; i <= 6; ++i ) {
        refSuffixAutomaton.addState ( DefaultStateType ( i ) );
    }

    refSuffixAutomaton.addFinalState ( DefaultStateType ( 0 ) );
    refSuffixAutomaton.addFinalState ( DefaultStateType ( 6 ) );

    refSuffixAutomaton.addTransition ( DefaultStateType ( 0 ), DefaultSymbolType ( 'a' ), DefaultStateType ( 1 ) );
    refSuffixAutomaton.addTransition ( DefaultStateType ( 0 ), DefaultSymbolType ( 't' ), DefaultStateType ( 2 ) );
    refSuffixAutomaton.addTransition ( DefaultStateType ( 0 ), DefaultSymbolType ( 'c' ), DefaultStateType ( 6 ) );
    refSuffixAutomaton.addTransition ( DefaultStateType ( 1 ), DefaultSymbolType ( 't' ), DefaultStateType ( 2 ) );
    refSuffixAutomaton.addTransition ( DefaultStateType ( 1 ), DefaultSymbolType ( 'c' ), DefaultStateType ( 6 ) );
    refSuffixAutomaton.addTransition ( DefaultStateType ( 2 ), DefaultSymbolType ( 'a' ), DefaultStateType ( 3 ) );
    refSuffixAutomaton.addTransition ( DefaultStateType ( 3 ), DefaultSymbolType ( 't' ), DefaultStateType ( 4 ) );
    refSuffixAutomaton.addTransition ( DefaultStateType ( 3 ), DefaultSymbolType ( 'c' ), DefaultStateType ( 6 ) );
    refSuffixAutomaton.addTransition ( DefaultStateType ( 4 ), DefaultSymbolType ( 'a' ), DefaultStateType ( 5 ) );
    refSuffixAutomaton.addTransition ( DefaultStateType ( 5 ), DefaultSymbolType ( 'c' ), DefaultStateType ( 6 ) );

    CPPUNIT_ASSERT ( suffixAutomaton == refSuffixAutomaton );
}

void SuffixAutomatonTest::testBackwardDAWGMatching ( ) {
    std::vector<std::string> subjects;
    std::vector<std::string> patterns;
    std::vector<std::set<unsigned>> expectedOccs;

    subjects.push_back("a"); patterns.push_back("a"); expectedOccs.push_back({0});
    subjects.push_back("a"); patterns.push_back("b"); expectedOccs.push_back({});
    subjects.push_back("alfalfalfa"); patterns.push_back("alfalfalfa"); expectedOccs.push_back({0});
    subjects.push_back("alfalfalfa"); patterns.push_back("blfalfalfa"); expectedOccs.push_back({});
    subjects.push_back("alfalfalfa"); patterns.push_back("alfalfalfb"); expectedOccs.push_back({});
    subjects.push_back("alfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfa"); patterns.push_back("alfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfa"); expectedOccs.push_back({0});
    subjects.push_back("alfalfalfaalfalfalfaabfalfalfaalfalfalfaalfalfalfaalfalfalfaalfa"); patterns.push_back("alfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfa"); expectedOccs.push_back({});
    subjects.push_back("alfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfaa"); patterns.push_back("alfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfaa"); expectedOccs.push_back({0});
    subjects.push_back("atggccttgcc"); patterns.push_back("gcc"); expectedOccs.push_back({3,8});
    subjects.push_back("aaaaaaaaaa"); patterns.push_back("a"); expectedOccs.push_back({0,1,2,3,4,5,6,7,8,9});
    subjects.push_back("aaaaaaaaaa"); patterns.push_back("aa"); expectedOccs.push_back({0,1,2,3,4,5,6,7,8});


    for(size_t i = 0; i < subjects.size(); ++i) {
        string::String subject = string::stringFrom ( subjects[i] );
        string::String pattern = string::stringFrom ( patterns[i] );
        std::set < unsigned > res = stringology::exact::BackwardDAWGMatching::match ( subject, pattern );
        std::cout << subjects[i] << ' ' << patterns[i] << ' ' << res << std::endl;
        CPPUNIT_ASSERT ( res == expectedOccs[i] );
    }

    auto longSubject = string::generate::RandomStringFactory::generateLinearString (64 * 64, 512, false, true);
    auto longPattern = string::generate::RandomSubstringFactory::generateSubstring(64 * 32, longSubject );
    std::set < unsigned > res = stringology::exact::BackwardDAWGMatching::match ( longSubject, longPattern );
    std::set < unsigned > resRef = stringology::exact::ExactFactorMatch::match ( longSubject, longPattern );
    std::cout << "long: " << res << std::endl;
    CPPUNIT_ASSERT ( res == resRef);
}

