/*
 * CommonBase.h
 *
 *  Created on: Apr 05, 2014
 *      Author: Jan Travnicek
 */

#ifndef COMMON_BASE_H_
#define COMMON_BASE_H_

#include <typeinfo>
#include <typeindex>
#include <ostream>
#include <stdexcept>

namespace alib {

class CommonBaseBase {
public:
	virtual ~CommonBaseBase ( ) noexcept { }

};

template< typename T >
class CommonBase : public CommonBaseBase {
public:
	virtual T * clone ( ) const = 0;

	virtual T * plunder ( ) && = 0;

	bool operator !=( const T & other ) const {
		return this->compare ( other ) != 0;
	}

	bool operator ==( const T & other ) const {
		return this->compare ( other ) == 0;
	}

	bool operator <( const T & other ) const {
		return this->compare ( other ) < 0;
	}

	bool operator <=( const T & other ) const {
		return this->compare ( other ) <= 0;
	}

	bool operator >( const T & other ) const {
		return this->compare ( other ) > 0;
	}

	bool operator >=( const T & other ) const {
		return this->compare ( other ) >= 0;
	}

	virtual int compare ( const T & other ) const = 0;

	friend std::ostream & operator <<( std::ostream & os, const T & instance ) {
		instance >> os;
		return os;
	}

	virtual void operator >>( std::ostream & ) const = 0;

	virtual explicit operator std::string ( ) const = 0;
};

} /* namespace alib */

namespace std {

template < class T >
struct compare < T, typename std::enable_if < std::is_base_of < alib::CommonBaseBase, typename std::decay < T >::type >::value >::type > {
	int operator()(const T& first, const T& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* COMMON_BASE_H_ */
