/*
 * Void.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef VOID_H_
#define VOID_H_

#include "ObjectBase.h"
#include <deque>
#include "../sax/Token.h"
#include "../core/xmlApi.hpp"

namespace alib {

/**
 * Represents void.
 */
class Void : public ObjectBase {
public:
	/**
	 * Creates a blank symbol.
	 * @param symbol name of the symbol
	 */
	explicit Void ( );

	virtual ObjectBase * clone ( ) const;
	virtual ObjectBase * plunder ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const Void & other ) const;

	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;

	static Void VOID;

	static const std::string & getXmlTagName() {
		static std::string xmlTagName = "Void";

		return xmlTagName;
	}

	static Void parse ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;

	virtual ObjectBase * inc ( ) &&;
};

} /* namespace alib */

namespace alib {

template < >
struct xmlApi < void > {
	static void parse ( std::deque < sax::Token >::iterator & input );
	static bool first ( const std::deque < sax::Token >::const_iterator & input );
	static void compose ( std::deque < sax::Token > & output );
};

} /* namespace alib */

#endif /* VOID_H_ */
