/*
 * Integer.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#include "Integer.h"
#include <sax/FromXMLParserHelper.h>
#include "Primitive.h"
#include <object/Object.h>

namespace primitive {

Integer::Integer(int data) : m_data(data) {

}

PrimitiveBase* Integer::clone() const {
	return new Integer(*this);
}

PrimitiveBase* Integer::plunder() && {
	return new Integer(std::move(*this));
}

int Integer::getData() const {
	return m_data;
}

Integer::operator int ( ) const {
	return m_data;
}

void Integer::setData(int data) {
	this->m_data = data;
}

int Integer::compare(const Integer& other) const {
	return m_data - other.m_data;
}

void Integer::operator>>(std::ostream& out) const {
	out << "(Integer " << m_data << ")";
}

Integer::operator std::string() const {
	return std::to_string(m_data);
}

Integer Integer::parse(std::deque<sax::Token>::iterator& input) {
	return Integer(parseRaw(input));
}

int Integer::parseRaw(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, Integer::getXmlTagName());
	int data = std::from_string < int > (sax::FromXMLParserHelper::popTokenData(input, sax::Token::TokenType::CHARACTER));
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, Integer::getXmlTagName());
	return data;
}

void Integer::compose(std::deque<sax::Token>& out) const {
	compose(out, this->getData());
}

void Integer::compose(std::deque<sax::Token>& out, int primitive) {
	out.emplace_back(Integer::getXmlTagName(), sax::Token::TokenType::START_ELEMENT);
	out.emplace_back(std::to_string(primitive), sax::Token::TokenType::CHARACTER);
	out.emplace_back(Integer::getXmlTagName(), sax::Token::TokenType::END_ELEMENT);
}

PrimitiveBase * Integer::inc() && {
	m_data++;
	return NULL;
}

} /* namespace primitive */

namespace alib {

int xmlApi<int>::parse(std::deque<sax::Token>::iterator& input) {
	return primitive::Integer::parseRaw(input);
}

bool xmlApi<int>::first(const std::deque<sax::Token>::const_iterator& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, primitive::Integer::getXmlTagName());
}

void xmlApi<int>::compose(std::deque<sax::Token>& output, int data) {
	primitive::Integer::compose(output, data);
}

auto integerParserRegister = xmlApi<primitive::Primitive>::ParserRegister<primitive::Integer>();
auto integerParserRegister2 = xmlApi<alib::Object>::ParserRegister<primitive::Integer>();

} /* namespace alib */

