/*
 * Unsigned.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef PRIMITIVE_UNSIGNED_H_
#define PRIMITIVE_UNSIGNED_H_

#include "PrimitiveBase.h"
#include <core/xmlApi.hpp>

namespace primitive {

/**
 * Represents symbol in an alphabet.
 */
class Unsigned : public PrimitiveBase {
protected:
	unsigned m_data;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit Unsigned ( unsigned data );

	virtual PrimitiveBase * clone ( ) const;

	virtual PrimitiveBase * plunder ( ) &&;

	/**
	 * @return name of the symbol
	 */
	unsigned getData ( ) const;

	explicit operator unsigned ( ) const;

	explicit operator size_t ( ) const;

	/**
	 * sets new content of primitive
	 */
	void setData ( unsigned data );

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const Unsigned & other ) const;

	virtual void operator >>( std::ostream & ) const;

	virtual explicit operator std::string ( ) const;

	static const std::string & getXmlTagName() {
		static std::string xmlTagName = "Unsigned";

		return xmlTagName;
	}

	static Unsigned parse ( std::deque < sax::Token >::iterator & input );
	static unsigned parseRaw ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;
	static void compose ( std::deque < sax::Token > & out, unsigned primitive );

	virtual PrimitiveBase * inc ( ) &&;
};

} /* namespace primitive */

namespace alib {

template < >
struct xmlApi < unsigned > {
	static unsigned parse ( std::deque < sax::Token >::iterator & input );
	static bool first ( const std::deque < sax::Token >::const_iterator & input );
	static void compose ( std::deque < sax::Token > & output, unsigned data );
};

} /* namespace alib */

#endif /* PRIMITIVE_UNSIGNED_H_ */
