/*
 * TreeToRawComposer.h
 *
 *  Created on: Nov 16, 2014
 *      Author: Jan Travnicek
 */

#ifndef TREE_TO_RAW_COMPOSER_H_
#define TREE_TO_RAW_COMPOSER_H_

#include <deque>
#include <vector>
#include <core/multipleDispatch.hpp>
#include <tree/Tree.h>
#include <tree/TreeFeatures.h>
#include <alphabet/SymbolFeatures.h>
#include <sax/Token.h>
#include <alphabet/ranked_symbol.hpp>

namespace tree {

/**
 * This class contains methods to print XML representation of tree to the output stream.
 */
class TreeToRawComposer : public std::SingleDispatchFirstStaticParam<TreeToRawComposer, void, std::deque<sax::Token>&, const TreeBase &> {
public:
	static void compose(std::deque<sax::Token>& out, const Tree& tree);

	template < class SymbolType, class RankType >
	static void compose(std::deque<sax::Token>& out, const RankedTree < SymbolType, RankType > & tree);
	template < class SymbolType, class RankType >
	static void compose(std::deque<sax::Token>& out, const PrefixRankedTree < SymbolType, RankType > & tree);
	template < class SymbolType >
	static void compose(std::deque<sax::Token>& out, const PrefixRankedBarTree < SymbolType > & tree);
	template < class SymbolType >
	static void compose(std::deque<sax::Token>& out, const UnrankedTree < SymbolType > & tree);

private:
	template < class SymbolType, class RankType >
	static void composeNode(std::deque<sax::Token>& out, const std::tree < std::ranked_symbol < SymbolType, RankType > > & node);
	template < class SymbolType, class RankType >
	static void composePrefixRankedNotation(std::deque<sax::Token>& out, unsigned& index, const std::vector<std::ranked_symbol < SymbolType, RankType > >& notation);
	template < class SymbolType, class RankType >
	static void composePrefixRankedBarNotation(std::deque<sax::Token>& out, unsigned& index, const std::vector<std::ranked_symbol < SymbolType, RankType > >& notation);
	template < class SymbolType >
	static void composeNode(std::deque<sax::Token>& out, const std::tree < SymbolType > & node);
};

template < class SymbolType, class RankType >
void TreeToRawComposer::composeNode(std::deque<sax::Token>& out, const std::tree < std::ranked_symbol < SymbolType, RankType > > & node) {
	if(node.getChildren().size() == 0) {
		out.emplace_back(sax::Token(std::to_string ( node.getData ( ) ), sax::Token::TokenType::CHARACTER));
	} else {
		out.emplace_back(sax::Token( std::to_string ( node.getData ( ) ), sax::Token::TokenType::START_ELEMENT));
		for(const std::tree < std::ranked_symbol < SymbolType, RankType > > & child : node.getChildren()) {
			composeNode(out, child);
		}

		out.emplace_back(sax::Token( std::to_string ( node.getData ( ) ), sax::Token::TokenType::END_ELEMENT));
	}
}

template < class SymbolType, class RankType >
void TreeToRawComposer::composePrefixRankedNotation(std::deque<sax::Token>& out, unsigned& index, const std::vector<std::ranked_symbol < SymbolType, RankType > > & notation) {
	unsigned size = ( unsigned ) notation[index].getRank();
	unsigned root = index;
	if(size == 0) {
		out.emplace_back(sax::Token( std::to_string ( notation[root] ), sax::Token::TokenType::CHARACTER));
		index += 1;
	} else {
		out.emplace_back(sax::Token( std::to_string ( notation[root] ), sax::Token::TokenType::START_ELEMENT));

		index += 1;
		for(unsigned i = 0; i < size; ++i) {
			composePrefixRankedNotation(out, index, notation);
		}

		out.emplace_back(sax::Token( std::to_string ( notation[root] ), sax::Token::TokenType::END_ELEMENT));
	}
}

template < class SymbolType, class RankType >
void TreeToRawComposer::composePrefixRankedBarNotation(std::deque<sax::Token>& out, unsigned& index, const std::vector<std::ranked_symbol < SymbolType, RankType > > & notation) {
	unsigned size = ( unsigned ) notation[index].getRank();
	unsigned root = index;
	if(size == 0) {
		out.emplace_back(sax::Token( std::to_string ( notation[root] ), sax::Token::TokenType::CHARACTER));
		index += 2;
	} else {
		out.emplace_back(sax::Token( std::to_string ( notation[root] ), sax::Token::TokenType::START_ELEMENT));

		index += 1;
		for(unsigned i = 0; i < size; i++) {
			composePrefixRankedBarNotation(out, index, notation);
		}
		index += 1;

		out.emplace_back(sax::Token(std::to_string ( notation[root] ), sax::Token::TokenType::END_ELEMENT));
	}
}

template < class SymbolType >
void TreeToRawComposer::composeNode(std::deque<sax::Token>& out, const std::tree < SymbolType > & node) {
	if(node.getChildren().size() == 0) {

	} else {
		out.emplace_back(sax::Token( std::to_string ( node.getData ( ) ), sax::Token::TokenType::START_ELEMENT));
		for(const std::tree < SymbolType > & child : node.getChildren()) {
			composeNode(out, child);
		}

		out.emplace_back(sax::Token(std::to_string ( node.getData ( ) ), sax::Token::TokenType::END_ELEMENT));
	}
}

} /* namespace tree */

#endif /* TREE_TO_RAW_COMPOSER_H_ */

