#include "TreeTest.h"

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( TreeTest, "bits" );
CPPUNIT_TEST_SUITE_REGISTRATION ( TreeTest );

void TreeTest::setUp ( ) {
}

void TreeTest::tearDown ( ) {
}

void TreeTest::print_tree ( const std::tree < char > & tree, std::string indent ) {
	std::cout << indent << tree.getData ( ) << std::endl;
	for ( const std::tree < char > & child : tree.getChildren ( ) ) {
		print_tree ( child, indent + " " );
	}
}

void TreeTest::testTreeStdStructure ( ) {
	std::tree < char > t ( 'a', std::tree < char > ( 'a' ), std::tree < char > ( 'b', std::tree < char > ( 'a' ), std::tree < char > ( 'b' ) ) );

	// std::cout << "structure t " << std::boolalpha << t.checkStructure() << std::endl;

	std::tree < char > t2 ( 'c' );

	 // std::cout << "structure t2 " << std::boolalpha << t2.checkStructure() << std::endl;
	t2.insert ( t2.root ( ), t2.begin ( ), { 'c', std::tree < char > ( 'b' ) } );

	std::swap ( t2.getChildren ( ).front (), t.getChildren().front ( ) );
	CPPUNIT_ASSERT ( t2 == std::tree < char > ( 'c', std::tree < char > ( 'a' ) ) );
	CPPUNIT_ASSERT ( t == std::tree < char > ( 'a', std::tree < char > ( 'c', std::tree < char > ( 'b' ) ), std::tree < char > ( 'b', std::tree < char > ( 'a' ), std::tree < char > ( 'b' ) ) ) );
	CPPUNIT_ASSERT ( t2.checkStructure ( ) );
	CPPUNIT_ASSERT ( t.checkStructure ( ) );
	std::swap ( t2.getChildren ( ).front (), t.getChildren().front ( ) );

	CPPUNIT_ASSERT ( t2.checkStructure ( ) );

	t2.insert ( t2.root ( ), t2.end ( ), t.root ( ),  t.root ( ) + 1 );
	CPPUNIT_ASSERT ( t2.checkStructure ( ) );

	std::vector < char > data = { 'd', 'e' };
	t2.insert ( t2.root ( ), t2.end ( ), data.begin ( ), data.end ( ) );
	CPPUNIT_ASSERT ( t2.checkStructure ( ) );

	print_tree ( t, "" );

	t2.erase ( t2.root ( ), std::prev ( t2.end ( ) ) );
	CPPUNIT_ASSERT ( t2.checkStructure ( ) );
	t2.push_back ( t2.root ( ), 'f' );
	CPPUNIT_ASSERT ( t2.checkStructure ( ) );

	print_tree ( t2, "" );

	CPPUNIT_ASSERT_EQUAL ( ( char ) t2 ( ), 'c' );
	CPPUNIT_ASSERT_EQUAL ( ( char ) t2 ( 0 ), 'c' );
	CPPUNIT_ASSERT_EQUAL ( ( char ) t2 ( 0, 0 ), 'b' );

	CPPUNIT_ASSERT_EQUAL ( ( char ) t2 ( 1, 1 ), 'b' );
	t2 ( 1, 1 ) = 'c';
	CPPUNIT_ASSERT_EQUAL ( ( char ) t2 ( 1, 1 ), 'c' );

	CPPUNIT_ASSERT ( t2.checkStructure ( ) );
	std::tree < char >::const_prefix_iterator beg = t2.prefix_begin ( );
	beg++;
	beg++;
	beg++;
	beg++;
	beg++;
	beg++;
	beg++;
	beg--;
	beg--;
	beg--;
	beg--;
	beg--;
	beg--;
	beg--;

	CPPUNIT_ASSERT ( beg == t2.prefix_begin ( ) );
	std::cout << t2 << std::endl;
	std::stringstream ss;
	ss << t2;
	CPPUNIT_ASSERT ( ss.str ( ) == "[0c[1c[2b],1a[2a,2c[3a,3b]],1d,1f]]" );

	std::vector < std::pair < unsigned, char > > expectedPrefix = { { 0, 'c' }, { 1, 'c' }, { 2, 'b' }, { 1, 'a' }, { 2, 'a' }, { 2, 'c' }, { 3, 'a' }, { 3, 'b' }, { 1, 'd' }, { 1, 'f' } };
	std::vector < std::pair < unsigned, char > >::const_iterator ref = expectedPrefix.begin ( );

	for ( std::tree < char >::const_prefix_iterator iter = t2.prefix_begin ( ); iter != t2.prefix_end ( ); ++iter ) {
		CPPUNIT_ASSERT ( iter.getLevel ( ) == ref->first );
		CPPUNIT_ASSERT ( * iter == ref->second );
		++iter;
		--iter;
		++ref;
	}

	std::vector < std::tuple < unsigned, char, bool > > expectedStructure = { std::make_tuple ( 0u, 'c', false ), std::make_tuple ( 1u, 'c', false ), std::make_tuple ( 2u, 'b', false ), std::make_tuple ( 2u, 'b', true ), std::make_tuple ( 1u, 'c', true ), std::make_tuple ( 1u, 'a', false ), std::make_tuple ( 2u, 'a', false ), std::make_tuple ( 2u, 'a', true ), std::make_tuple ( 2u, 'c', false ), std::make_tuple ( 3u, 'a', false ), std::make_tuple ( 3u, 'a', true ), std::make_tuple ( 3u, 'b', false ), std::make_tuple ( 3u, 'b', true ), std::make_tuple ( 2u, 'c', true ), std::make_tuple ( 1u, 'a', true ), std::make_tuple ( 1u, 'd', false ), std::make_tuple ( 1u, 'd', true ), std::make_tuple ( 1u, 'f', false ), std::make_tuple ( 1u, 'f', true ), std::make_tuple ( 0u, 'c', true ) };
	std::vector < std::tuple < unsigned, char, bool > >::const_iterator ref2 = expectedStructure.begin ( );

	for ( std::tree < char >::const_structure_iterator iter = t2.structure_begin ( ); iter != t2.structure_end ( ); ++iter ) {
		CPPUNIT_ASSERT ( iter.getLevel ( ) == std::get < 0 > ( * ref2 ) );
		CPPUNIT_ASSERT ( * iter == std::get < 1 > ( * ref2 ) );
		CPPUNIT_ASSERT ( iter.getVirtual ( ) == std::get < 2 > ( * ref2 ) );
		++iter;
		--iter;
		++ref2;
	}

	std::vector < std::pair < unsigned, char > > expectedPostfix = { { 2, 'b' }, { 1, 'c' }, { 2, 'a' }, { 3, 'a' }, { 3, 'b' }, { 2, 'c' }, { 1, 'a' }, { 1, 'd' }, { 1, 'f' }, { 0, 'c' } };
	std::vector < std::pair < unsigned, char > >::const_iterator ref3 = expectedPostfix.begin ( );

	for ( std::tree < char >::const_postfix_iterator iter = t2.postfix_begin ( ); iter != t2.postfix_end ( ); ++iter ) {
		CPPUNIT_ASSERT ( iter.getLevel ( ) == ref3->first );
		CPPUNIT_ASSERT ( * iter == ref3->second );
		++iter;
		--iter;
		++ref3;
	}

	std::tree < char > cmp1 ( 'a' );
	std::tree < char > cmp2 ( 'b' );
	std::tree < char > cmp3 ( 'c' );

	CPPUNIT_ASSERT ( cmp1 < cmp2 );
	CPPUNIT_ASSERT ( cmp2 < cmp3 );
	CPPUNIT_ASSERT ( cmp2 <= cmp3 );
	CPPUNIT_ASSERT ( cmp2 == cmp2 );
	CPPUNIT_ASSERT ( cmp1 != cmp2 );
}
