/*
 * GrammarToStringComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "GrammarToStringComposer.h"
#include "grammar/GrammarClasses.h"

namespace grammar {

void GrammarToStringComposer::compose(std::ostream& output, const Grammar& grammar) {
	dispatch(output, grammar.getData());
}

auto ObjectToStringComposerGrammar = alib::WrapperBaseDispatcher::RegistratorWrapper<void, Grammar>(GrammarToStringComposer::compose);

template<class SymbolType>
void GrammarToStringComposer::compose(std::ostream& output, const LeftLG < SymbolType > & grammar) {
	output << "LEFT_LG";
	composeCFLikeGrammar(output, grammar);
}

auto GrammarToStringComposerLeftLG = GrammarToStringComposer::RegistratorWrapper<void, LeftLG < > >(GrammarToStringComposer::compose);
auto ObjectToStringComposerLeftLG = alib::ObjectBaseDispatcher::RegistratorWrapper<void, LeftLG < > >( GrammarToStringComposer::compose);

template<class SymbolType>
void GrammarToStringComposer::compose(std::ostream& output, const LeftRG < SymbolType > & grammar) {
	output << "LEFT_RG";
	composeCFLikeGrammar(output, grammar);
}

auto GrammarToStringComposerLeftRG = GrammarToStringComposer::RegistratorWrapper<void, LeftRG < > >(GrammarToStringComposer::compose);
auto ObjectToStringComposerLeftRG = alib::ObjectBaseDispatcher::RegistratorWrapper<void, LeftRG < > >( GrammarToStringComposer::compose);

template<class SymbolType>
void GrammarToStringComposer::compose(std::ostream& output, const RightLG < SymbolType > & grammar) {
	output << "RIGHT_LG";
	composeCFLikeGrammar(output, grammar);
}

auto GrammarToStringComposerRightLG = GrammarToStringComposer::RegistratorWrapper<void, RightLG < > >(GrammarToStringComposer::compose);
auto ObjectToStringComposerRightLG = alib::ObjectBaseDispatcher::RegistratorWrapper<void, RightLG < > >( GrammarToStringComposer::compose);

template<class SymbolType>
void GrammarToStringComposer::compose(std::ostream& output, const RightRG < SymbolType > & grammar) {
	output << "RIGHT_RG";
	composeCFLikeGrammar(output, grammar);
}

auto GrammarToStringComposerRightRG = GrammarToStringComposer::RegistratorWrapper<void, RightRG < > >(GrammarToStringComposer::compose);
auto ObjectToStringComposerRightRG = alib::ObjectBaseDispatcher::RegistratorWrapper<void, RightRG < > >( GrammarToStringComposer::compose);

template<class SymbolType>
void GrammarToStringComposer::compose(std::ostream& output, const LG < SymbolType > & grammar) {
	output << "LG";
	composeCFLikeGrammar(output, grammar);
}

auto GrammarToStringComposerLG = GrammarToStringComposer::RegistratorWrapper<void, LG < > >(GrammarToStringComposer::compose);
auto ObjectToStringComposerLG = alib::ObjectBaseDispatcher::RegistratorWrapper<void, LG < > >( GrammarToStringComposer::compose);

template<class SymbolType>
void GrammarToStringComposer::compose(std::ostream& output, const CFG < SymbolType > & grammar) {
	output << "CFG";
	composeCFLikeGrammar(output, grammar);
}

auto GrammarToStringComposerCFG = GrammarToStringComposer::RegistratorWrapper<void, CFG < > >(GrammarToStringComposer::compose);
auto ObjectToStringComposerCFG = alib::ObjectBaseDispatcher::RegistratorWrapper<void, CFG < > >( GrammarToStringComposer::compose);

template<class SymbolType>
void GrammarToStringComposer::compose(std::ostream& output, const EpsilonFreeCFG < SymbolType > & grammar) {
	output << "EPSILON_FREE_CFG";
	composeCFLikeGrammar(output, grammar);
}

auto GrammarToStringComposerEpsilonFreeCFG = GrammarToStringComposer::RegistratorWrapper<void, EpsilonFreeCFG < > >(GrammarToStringComposer::compose);
auto ObjectToStringComposerEpsilonFreeCFG = alib::ObjectBaseDispatcher::RegistratorWrapper<void, EpsilonFreeCFG < > >( GrammarToStringComposer::compose);

template<class SymbolType>
void GrammarToStringComposer::compose(std::ostream& output, const CNF < SymbolType > & grammar) {
	output << "CNF";
	composeCFLikeGrammar(output, grammar);
}

auto GrammarToStringComposerCNF = GrammarToStringComposer::RegistratorWrapper<void, CNF < > >(GrammarToStringComposer::compose);
auto ObjectToStringComposerCNF = alib::ObjectBaseDispatcher::RegistratorWrapper<void, CNF < > >( GrammarToStringComposer::compose);

template<class SymbolType>
void GrammarToStringComposer::compose(std::ostream& output, const GNF < SymbolType > & grammar) {
	output << "GNF";
	composeCFLikeGrammar(output, grammar);
}

auto GrammarToStringComposerGNF = GrammarToStringComposer::RegistratorWrapper<void, GNF < > >(GrammarToStringComposer::compose);
auto ObjectToStringComposerGNF = alib::ObjectBaseDispatcher::RegistratorWrapper<void, GNF < > >( GrammarToStringComposer::compose);

template<class SymbolType>
void GrammarToStringComposer::compose(std::ostream& output, const CSG < SymbolType > & grammar) {
	output << "CSG";
	composePreservingCSLikeGrammar(output, grammar);
}

auto GrammarToStringComposerCSG = GrammarToStringComposer::RegistratorWrapper<void, CSG < > >(GrammarToStringComposer::compose);
auto ObjectToStringComposerCSG = alib::ObjectBaseDispatcher::RegistratorWrapper<void, CSG < > >( GrammarToStringComposer::compose);

template<class SymbolType>
void GrammarToStringComposer::compose(std::ostream& output, const NonContractingGrammar < SymbolType > & grammar) {
	output << "NON_CONTRACTING_GRAMMAR";
	composeCSLikeGrammar(output, grammar);
}

auto GrammarToStringComposerNonContractingGrammar = GrammarToStringComposer::RegistratorWrapper<void, NonContractingGrammar < > >(GrammarToStringComposer::compose);
auto ObjectToStringComposerNonContractingGrammar = alib::ObjectBaseDispatcher::RegistratorWrapper<void, NonContractingGrammar < > >( GrammarToStringComposer::compose);

template<class SymbolType>
void GrammarToStringComposer::compose(std::ostream& output, const ContextPreservingUnrestrictedGrammar < SymbolType > & grammar) {
	output << "CONTEXT_PRESERVING_UNRESTRICTED_GRAMMAR";
	composePreservingCSLikeGrammar(output, grammar);
}

auto GrammarToStringComposerContextPreservingUnrestructedGrammar = GrammarToStringComposer::RegistratorWrapper<void, ContextPreservingUnrestrictedGrammar < > >(GrammarToStringComposer::compose);
auto ObjectToStringComposerContextPreservingUnrestrictedGrammar = alib::ObjectBaseDispatcher::RegistratorWrapper<void, ContextPreservingUnrestrictedGrammar < > >( GrammarToStringComposer::compose);

template<class SymbolType>
void GrammarToStringComposer::compose(std::ostream& output, const UnrestrictedGrammar < SymbolType > & grammar) {
	output << "UNRESTRICTED_GRAMMAR";
	composeCSLikeGrammar(output, grammar);
}

auto GrammarToStringComposerUnrestructedGrammar = GrammarToStringComposer::RegistratorWrapper<void, UnrestrictedGrammar < > >(GrammarToStringComposer::compose);
auto ObjectToStringComposerUnrestrictedGrammar = alib::ObjectBaseDispatcher::RegistratorWrapper<void, UnrestrictedGrammar < > >( GrammarToStringComposer::compose);

}
