/*
 * LabelFromStringParser.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "LabelFromStringParser.h"
#include "exception/CommonException.h"
#include "label/PrimitiveLabel.h"
#include "label/ObjectLabel.h"
#include "label/LabelSetLabel.h"
#include "label/LabelPairLabel.h"
#include <label/Label.h>

#include "../StringApi.hpp"

namespace label {

Label LabelFromStringParser::parseLabel(std::istream& input) {
	return parseLabel(input, std::set<FEATURES>({FEATURES::PRIMITIVE, FEATURES::HEXAVIGESIMAL, FEATURES::OBJECT, FEATURES::LABEL_SET, FEATURES::LABEL_PAIR, FEATURES::UNIQUE_LABEL}));
}

Label LabelFromStringParser::parseLabel(std::istream& input, const std::set<FEATURES>& features) {
	LabelFromStringLexer::Token token = LabelFromStringLexer::next(input);
	switch(token.type) {
	case LabelFromStringLexer::TokenType::TEOF:
		throw exception::CommonException("Unexpected end of file");
	case LabelFromStringLexer::TokenType::SET_BEGIN: {
		if(!features.count(FEATURES::LABEL_SET)) throw exception::CommonException("Diabled formalism LabelSet");
		token = LabelFromStringLexer::next(input);

		std::set<Label> labelSet;
		if(token.type != LabelFromStringLexer::TokenType::SET_END) while(true) {
			LabelFromStringLexer::putback(input, std::move(token));
			Label innerLabel = parseLabel(input);
			labelSet.insert(innerLabel);

			token = LabelFromStringLexer::next(input);
			if(token.type != LabelFromStringLexer::TokenType::COMMA) 
				break;

			token = LabelFromStringLexer::next(input);
		}

		if(token.type != LabelFromStringLexer::TokenType::SET_END)
			throw exception::CommonException("Expected SET_END token.");
		return Label(label::LabelSetLabel(labelSet));
	}
	case LabelFromStringLexer::TokenType::PAIR_BEGIN: {
		if(!features.count(FEATURES::LABEL_PAIR)) throw exception::CommonException("Diabled formalism LabelPair");

		Label firstLabel = parseLabel(input);

		token = LabelFromStringLexer::next(input);
		if(token.type != LabelFromStringLexer::TokenType::COMMA)
			throw exception::CommonException("Excepted COMMA token.");

		Label secondLabel = parseLabel(input);

		token = LabelFromStringLexer::next(input);
		if(token.type != LabelFromStringLexer::TokenType::PAIR_END)
			throw exception::CommonException("Expected PAIR_END token.");

		return Label(label::LabelPairLabel(std::make_pair(firstLabel, secondLabel)));
	}
	case LabelFromStringLexer::TokenType::SET_END:
		throw exception::CommonException("Unexpected start of Label SET_END.");
	case LabelFromStringLexer::TokenType::PAIR_END:
		throw exception::CommonException("Unexpected start of Label PAIR_END.");
	case LabelFromStringLexer::TokenType::COMMA:
		throw exception::CommonException("Unexpected start of Label COMMA.");
	case LabelFromStringLexer::TokenType::ERROR:
		if(!features.count(FEATURES::PRIMITIVE)) throw exception::CommonException("Disabled formalism PrimitiveLabel");
		LabelFromStringLexer::putback(input, token);
		return Label(PrimitiveLabel(alib::stringApi<primitive::Primitive>::parse(input)));
	}
	throw exception::CommonException("Unrecognised input \"" + token.value + "\"");
}

} /* namespace label */
