/*
 * LabelToStringComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include <object/ObjectToStringComposer.h>

#include "LabelToStringComposer.h"
#include "label/PrimitiveLabel.h"
#include "label/HexavigesimalLabel.h"
#include "label/ObjectLabel.h"
#include "label/LabelSetLabel.h"
#include "label/LabelPairLabel.h"
#include "label/UniqueLabel.h"

#include "../StringApi.hpp"

namespace label {

void LabelToStringComposer::compose(std::ostream& out, const PrimitiveLabel& label) {
	alib::stringApi<primitive::Primitive>::compose(out, label.getData());
}

auto LabelToStringComposerPrimitiveLabel = LabelToStringComposer::RegistratorWrapper<void, PrimitiveLabel>(LabelToStringComposer::compose);
auto ObjectToStringComposerPrimitiveLabel = alib::ObjectBaseDispatcher::RegistratorWrapper<void, PrimitiveLabel >( LabelToStringComposer::compose);

void LabelToStringComposer::compose(std::ostream& out, const HexavigesimalLabel& label) {
	out << (std::string) label;
}

auto LabelToStringComposerHexavigesimalLabel = LabelToStringComposer::RegistratorWrapper<void, HexavigesimalLabel>(LabelToStringComposer::compose);
auto ObjectToStringComposerHexavigesimalLabel = alib::ObjectBaseDispatcher::RegistratorWrapper<void, HexavigesimalLabel >( LabelToStringComposer::compose);

void LabelToStringComposer::compose(std::ostream& out, const ObjectLabel& label) {
	out << (std::string) label;
}

auto LabelToStringComposerObjectLabel = LabelToStringComposer::RegistratorWrapper<void, ObjectLabel>(LabelToStringComposer::compose);
auto ObjectToStringComposerObjectLabel = alib::ObjectBaseDispatcher::RegistratorWrapper<void, ObjectLabel >( LabelToStringComposer::compose);

void LabelToStringComposer::compose(std::ostream& out, const LabelSetLabel& label) {
	out << '[';
	bool first = true;
	for(const Label& innerLabel : label.getData()) {
		if(!first)
			out << ", ";
		else
			first = false;
		dispatch(out, innerLabel.getData());
	}
	out << ']';
}

auto LabelToStringComposerLabelSetLabel = LabelToStringComposer::RegistratorWrapper<void, LabelSetLabel>(LabelToStringComposer::compose);
auto ObjectToStringComposerLabelSetLabel = alib::ObjectBaseDispatcher::RegistratorWrapper<void, LabelSetLabel >( LabelToStringComposer::compose);

void LabelToStringComposer::compose(std::ostream& out, const LabelPairLabel& label) {
	out << '<';
	dispatch(out, label.getData().first.getData());
	out << ", ";
	dispatch(out, label.getData().second.getData());
	out << '>';
}

auto LabelToStringComposerLabelPairLabel = LabelToStringComposer::RegistratorWrapper<void, LabelPairLabel>(LabelToStringComposer::compose);
auto ObjectToStringComposerLabelPairLabel = alib::ObjectBaseDispatcher::RegistratorWrapper<void, LabelPairLabel >( LabelToStringComposer::compose);

void LabelToStringComposer::compose(std::ostream& out, const UniqueLabel& label) {
	out << '<';
	dispatch(out, label.getLabel().getData());
	out << ", ";
	alib::stringApi<primitive::Primitive>::compose(out, primitive::Primitive(label.getId()));
	out << '>';
}

auto LabelToStringComposerUniqueLabel = LabelToStringComposer::RegistratorWrapper<void, UniqueLabel>(LabelToStringComposer::compose);
auto ObjectToStringComposerUniqueLabel = alib::ObjectBaseDispatcher::RegistratorWrapper<void, UniqueLabel >( LabelToStringComposer::compose);

void LabelToStringComposer::compose(std::ostream& output, const Label& label) {
	dispatch(output, label.getData());
}

auto ObjectToStringComposerLabel = alib::WrapperBaseDispatcher::RegistratorWrapper<void, Label>(LabelToStringComposer::compose);

} /* namespace label */
