/*
 * ObjectToStringComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "ObjectToStringComposer.h"

namespace alib {

void ObjectToStringComposer::compose(std::ostream& out, const Object& object) {
	compose( out, object.getData ( ) );
}

auto ObjectToStringComposerObject = WrapperBaseDispatcher::RegistratorWrapper<void, Object>(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose(std::ostream& out, const Void&) {
	out << "Void";
}

auto ObjectToStringComposerVoid = ObjectBaseDispatcher::RegistratorWrapper<void, Void>(ObjectToStringComposer::compose);

template < class T, class R >
void ObjectToStringComposer::compose(std::ostream& out, const container::ObjectsPair < T, R > & object) {
	out << '<';
	compose(out, object.first);
	out << ", ";
	compose(out, object.second);
	out << '>';
}

auto ObjectToStringComposerObjectsPair = ObjectBaseDispatcher::RegistratorWrapper<void, container::ObjectsPair < Object, Object > >(ObjectToStringComposer::compose);

template < class T >
void ObjectToStringComposer::compose(std::ostream& out, const container::ObjectsSet < T > & object) {
	out << '[';
	bool first = true;
	for(const T & innerObject : object) {
		if(!first)
			out << ", ";
		else
			first = false;
		compose(out, innerObject);
	}
	out << ']';
}

auto ObjectToStringComposerObjectsSet = ObjectBaseDispatcher::RegistratorWrapper<void, container::ObjectsSet < Object > >(ObjectToStringComposer::compose);

void ObjectToStringComposer::compose ( std::ostream & out, const WrapperBaseBase & object ) {
	WrapperBaseDispatcher::dispatch(out, object);
}

void ObjectToStringComposer::compose ( std::ostream & out, const CommonBaseBase & object ) {
	ObjectBaseDispatcher::dispatch(out, object);
}

} /* namespace alib */
