/*
 * PrimitiveFromStringLexer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "PrimitiveFromStringLexer.h"

namespace primitive {

PrimitiveFromStringLexer::Token PrimitiveFromStringLexer::next(std::istream& in) {
	PrimitiveFromStringLexer::Token token;
	token.type = TokenType::ERROR;
	token.value = "";
	token.raw = "";
	char character;

L0:
	character = in.get();
	if(in.eof()) {
		token.type = TokenType::TEOF;
		return token;
	} else if(character == ' ' || character == '\n' || character == '\t') {
		token.raw += character;
		goto L0;
	} else if((character >= 'a' && character <= 'z') || (character >= 'A' && character <= 'Z')) {
		token.type = TokenType::CHAR;
		token.value += character;
		token.raw += character;
		return token;
	} else if(character >= '0' && character <= '9') {
		token.type = TokenType::INTEGER;
		token.value += character;
		token.raw += character;
		goto L1;
	} else if(character == '\'') {
		token.type = TokenType::STRING;
		token.raw += character;
		goto L3;
	} else {
		in.putback(character);
		putback(in, std::move(token));
		token.type = TokenType::ERROR;
		return token;
	}
L1:
	character = in.get();
	if(in.eof()) {
		return token;
	} else if(character >= '0' && character <= '9') {
		token.value += character;
		token.raw += character;
		goto L1;
	} else {
		in.unget();
		return token;
	}
L3:
	character = in.get();
	if(in.eof()) {
		token.type = TokenType::TEOF;
		return token;
	} else if(character == '\'') {
		token.raw += character;
		return token;
	} else if(character == '\\') {
		token.raw += character;
		goto L4;
	} else {
		token.value += character;
		token.raw += character;
		goto L3;
	}
L4:
	character = in.get();
	if(in.eof()) {
		token.type = TokenType::TEOF;
		return token;
	} else if(character == '\'' || character == '\\') {
		token.value += character;
		token.raw += character;
		goto L3;
	} else {
		in.putback(character);
		putback(in, std::move(token));
		token.type = TokenType::ERROR;
		return token;
	}
}

void PrimitiveFromStringLexer::putback(std::istream& in, PrimitiveFromStringLexer::Token token) {
	while(!token.raw.empty()) {
		in.putback(token.raw.back());
		token.raw.pop_back();
	}
	in.clear();
}

} /* namespace primitive */
