/*
 * StringFromStringParser.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "StringFromStringParser.h"
#include "exception/CommonException.h"
#include "string/StringClasses.h"
#include <vector>
#include <string/String.h>

#include "../StringApi.hpp"
#include "../object/ObjectStringApi.hpp"
#include "../alphabet/SymbolStringApi.hpp"

namespace string {

String StringFromStringParser::parseString(std::istream& input) {
	return parseString(input, std::set<FEATURES>({FEATURES::LINEAR, FEATURES::CYCLIC, FEATURES::EPSILON}));
}

String StringFromStringParser::parseString(std::istream& input, const std::set<FEATURES>& features) {
	StringFromStringLexer::Token token = StringFromStringLexer::next(input);
	if(token.type == StringFromStringLexer::TokenType::EPSILON) {
		if(!features.count(FEATURES::EPSILON)) throw exception::CommonException("Disabled formalism Epsilon");
		return String(Epsilon < > ());
	} else if(token.type == StringFromStringLexer::TokenType::LESS) {
		if(!features.count(FEATURES::CYCLIC)) throw exception::CommonException("Disabled formalism CyclicString");
		std::vector<DefaultSymbolType> data = parseContent(input);
		token = StringFromStringLexer::next(input);
		if(token.type == StringFromStringLexer::TokenType::GREATER) {
			return String(CyclicString < > (data));
		} else {
			throw exception::CommonException("Invalid cyclic string terminating character");
		}
	} else if(token.type == StringFromStringLexer::TokenType::QUOTE) {
		if(!features.count(FEATURES::LINEAR)) throw exception::CommonException("Disabled formalism LinearString");
		std::vector<DefaultSymbolType> data = parseContent(input);
		token = StringFromStringLexer::next(input);
		if(token.type == StringFromStringLexer::TokenType::QUOTE) {
			return String(LinearString < >(data));
		} else {
			throw exception::CommonException("Invalid linear string terminating character");
		}
	} else {
		throw exception::CommonException("Formalism not recognised (token = \"" + token.value + "\")");
	}
}

std::vector<DefaultSymbolType> StringFromStringParser::parseContent(std::istream& input) {
	std::vector<DefaultSymbolType> data;
	StringFromStringLexer::Token token = StringFromStringLexer::next(input);
	while(token.type != StringFromStringLexer::TokenType::GREATER && token.type != StringFromStringLexer::TokenType::QUOTE) {
		StringFromStringLexer::putback(input, token);
		data.push_back(alib::stringApi<DefaultSymbolType>::parse<alphabet::Symbol>(input));

		token = StringFromStringLexer::next(input);
	}
	StringFromStringLexer::putback(input, token);
	return data;
}

} /* namespace string */
