/*
 * araw.cpp
 *
 *  Created on: 24. 2. 2014
 *      Author: Jan Travnicek
**/

#include <tclap/CmdLine.h>
#include <global/GlobalData.h>
#include <measure>
#include <string>
#include <sax/FromXMLParserHelper.h>
#include <fstream>
#include <exception/CommonException.h>
#include <RawApi.hpp>
#include <factory/XmlDataFactory.hpp>
#include <sax/SaxParseInterface.h>
#include <sax/ParserException.h>
#include <tree/Tree.h>
#include <tree/TreeToRawComposer.h>
#include <string/String.h>
#include <string/StringToRawComposer.h>

int main(int argc, char** argv) {
	try {
		common::GlobalData::argc = argc;
		common::GlobalData::argv = argv;

		TCLAP::CmdLine cmd("raw reading binary", ' ', "0.01");

		TCLAP::ValueArg<std::string> input(	"i",	"input",	"Input to read",		false,	"-",		"file");
		cmd.add( input );

		TCLAP::SwitchArg tree_from_raw(		"", "tree_from_raw",	"Convert tree from raw representation (xml)");
		cmd.add( tree_from_raw );

		TCLAP::SwitchArg tree_to_raw(		"", "tree_to_raw",	"Convert tree to raw representation (xml)");
		cmd.add( tree_to_raw );

		TCLAP::SwitchArg string_from_raw(	"", "string_from_raw",	"Convert string from raw representation (string)");
		cmd.add( string_from_raw );

		TCLAP::SwitchArg string_to_raw(		"", "string_to_raw",	"Convert string to raw representation (string)");
		cmd.add( string_to_raw );

		TCLAP::SwitchArg matrix_from_raw(	"", "matrix_from_raw",	"Convert string from raw representation (csv)");
		cmd.add( matrix_from_raw );

		TCLAP::SwitchArg matrix_to_raw(		"", "matrix_to_raw",	"Convert string to raw representation (csv)");
		cmd.add( matrix_to_raw );

		TCLAP::SwitchArg measure(		"m",	"measure",	"Measure times",		false);
		cmd.add( measure );

		TCLAP::SwitchArg verbose(		"v",	"verbose",	"Be verbose",			false);
		cmd.add( verbose );

		cmd.parse(argc, argv);

		if(verbose.isSet())
			common::GlobalData::verbose = true;
		if(measure.isSet())
			common::GlobalData::measure = true;

		measurements::start("Overal", measurements::Type::OVERALL);
		measurements::start("Input read", measurements::Type::AUXILIARY);

		if(tree_from_raw.getValue()) {
			std::deque<sax::Token> tokens = sax::FromXMLParserHelper::parseInput(input);

			std::deque<sax::Token>::iterator iter = tokens.begin();
			tree::Tree tree = alib::FromRawParsers::treeParser.parseTree(iter);

			measurements::end();
			measurements::start("Output write", measurements::Type::AUXILIARY);

			alib::XmlDataFactory::toStdout(tree);
		} else if(tree_to_raw.getValue()) {
			tree::Tree tree = alib::XmlDataFactory::fromTokens<tree::Tree>(sax::FromXMLParserHelper::parseInput(input));

			measurements::end();
			measurements::start("Output write", measurements::Type::AUXILIARY);

			std::deque<sax::Token> tokens2;
			tree::TreeToRawComposer::compose(tokens2, tree);
			sax::SaxComposeInterface::printStdout(tokens2);
		} else if(string_from_raw.getValue()) {
			std::ifstream inFile;
			std::istream_iterator<char> iter;
			if(input.isSet()) {
				if(input.getValue() == "-") {
					iter = std::cin;
				} else {
					inFile.open(input.getValue());
					iter = inFile;
				}
			} else {
				iter = std::cin;
			}

			string::String string = alib::FromRawParsers::stringParser.parseString(iter);

			measurements::end();
			measurements::start("Output write", measurements::Type::AUXILIARY);

			alib::XmlDataFactory::toStdout(string);
		} else if(string_to_raw.getValue()) {
			string::String string = alib::XmlDataFactory::fromTokens<string::String>(sax::FromXMLParserHelper::parseInput(input));

			measurements::end();
			measurements::start("Output write", measurements::Type::AUXILIARY);

			string::StringToRawComposer::compose(std::cout, string);
		} else {
			throw exception::CommonException("Invalid input format specified");
		}

		measurements::end();
		measurements::end();

		if(measure.getValue()) std::cmeasure << measurements::results() << std::endl;

		return 0;
	} catch(const exception::CommonException& exception) {
		alib::XmlDataFactory::toStdout(exception);
		return 1;
	} catch(const TCLAP::ArgException& exception) {
		std::cerr << exception.error() << std::endl;
		return 2;
	} catch (const std::exception& exception) {
		std::cerr << "Exception caught: " << exception.what() << std::endl;
		return 3;
	} catch(...) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
